package appl.parallel;

import java.io.Serializable;
import java.util.Properties;

/**
 * Information about a {@link ComputingResource}. All entries may return
 * <code>null</code> if no information is available. Notice that the 
 * information represents the state of the remote Object at the time it was
 * created. So it may be outdated. The following Properties should be available
 * on all systems <br>
 * <ul>
 * <li>ActiveModel</li>
 * <li>isAvailable - says if the resource is currently available for use</li>
 * <li>SystemInformation - Identifies the current System/OS</li>
 * <li>Name - a name which will be displayed to the user</li>
 * <li>IP - the IP address of the Object<br>
 * </li>
 * </ul>
 * <br>
 * 
 * @author Dominik Appl
 */
public interface ComputingResourceProperties extends Serializable {

	Properties getProperties();

	/**
	 * Sets a property
	 * @param key the key
	 * @param value the value
	 */
	public void setProperty(String key, String value);

	/**
	 * 
	 * @param key the key
	 * @return the property property associated with that key or null if not found
	 */
	public String getProperty(String key);

	/**
	 * @return the name of the resource (usually the hostname)
	 */
	public String getName();

	/**
	 * @return says if the resource is available for computation (of in use).
	 */
	public boolean isAvailable();

	/**
	 * @return the ip of the resource
	 */
	public String getIP();

	/**
	 * @return port the resouce is bound to
	 */
	public String getPort();

}
