package appl.parallel.data;

import java.awt.Rectangle;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import appl.ext.XuluConfig;
import appl.parallel.client.ClientDataServer;

/**
 * Reads out of {@link XuluConfig} which {@link PartitionDataHandler} needs to
 * be used for data loading/unloading and constructs an instance of this class.
 * The classname is read out of the property <br>
 * <br>
 * Parallel.partitionhandlerfactory.classname
 * 
 * @author Dominik Appl
 */
public class PartitionHandlerFactory {

	private static Logger LOG = LogManager
			.getLogger("appl.data.PartitionHandlerFactory");

	/**
	 * @param rootID
	 *            the id of the data
	 * @param partitionBounds
	 *            the bounds of the partition to be retrieved on server side
	 * @param unloadBounds
	 *            the bounds of the partition which is to be uploaded to the
	 *            client after calculation
	 * @return a new {@link PartitionDataHandler} instance
	 */
	public static PartitionDataHandler newInstance(int rootID,
			ClientDataServer client, Rectangle partitionBounds,
			Rectangle unloadBounds) {
		PartitionDataHandler returnhandler = null;
		// try to get info from xulu config
		String classname = XuluConfig.getXuluConfig().getProperty(
				"Parallel.partitionhandlerfactory.classname");
		if (classname != null) {
			try {
				returnhandler = (PartitionDataHandler) Class.forName(classname)
						.newInstance();
			} catch (InstantiationException e) {
				LOG.error(e);
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				LOG.error(e);
				e.printStackTrace();
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				LOG.error("Could not find PartitionHandler class '" + classname
						+ "'", e);
			}
		}
		if (returnhandler != null) {
			return returnhandler.newInstance(rootID, client, partitionBounds,
					unloadBounds);
		}
		return null;
	}

}
