package appl.parallel.data.splittable;

import java.awt.Rectangle;
import java.io.Serializable;

import schmitzm.data.WritableGrid;
import appl.data.DataLoader;
import appl.data.WritableGridLLProxy;
import appl.parallel.spmd.split.DataPartition;
import appl.parallel.spmd.split.SplittableGrid;
import appl.parallel.spmd.split.SplittableResource;
import appl.parallel.spmd.split.WritableGridPartition;
import appl.parallel.util.PartitionUtil;
import appl.util.RasterMetaData;
import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;

/**
 * @version 1.0
 */
public class SplittableLLProxyGrid extends WritableGridLLProxy implements
		SplittableGrid, Serializable {

	private final int id;

	public SplittableLLProxyGrid(ImportFactory importFac,
			RasterMetaData metaData, Object inputPara, XuluRegistry reg) {
		super(importFac, metaData, inputPara, reg);
		id = this.hashCode();
	}

	public SplittableLLProxyGrid(WritableGridFactory targetFactory,
			RasterMetaData metaData) {
		super(targetFactory, metaData);
		id = this.hashCode();
	}

	public SplittableLLProxyGrid(RasterMetaData metaData, int id) {
		super(metaData);
		this.id = id;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.SplittableResource#getFileSystemLoader()
	 */
	public DataLoader getLocalLoader() {
		return this.intialDataLoader;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.SplittableResource#getID()
	 */
	public int getRootID() {
		return this.id;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.SplittableResource#getPartition(java.awt.Rectangle)
	 */
	public WritableGridPartition getPartition(Rectangle partitionBounds) {
		if (!loaded)
			tryLoadingGrid();
		return PartitionUtil.getPartitialGrid2D(this, partitionBounds,
				getRootID());
	}

	/**
	 * Overwrites the data at the location specified by the {@link Rectangle}
	 * with the given partition-data.
	 *
	 * @param partition
	 *            the grid to be inserted which MUST be an instance of
	 *            {@link WritableGrid}!
	 * @param partitionBounds
	 *            the target location of the data
	 * @see appl.parallel.spmd.split.DataPartition#setPartition(DataPartition,
	 *      java.awt.Rectangle)
	 */
	public void setPartition(DataPartition partition, Rectangle partitionBounds) {
		if (!loaded)
			tryLoadingGrid();
		if (!(partition instanceof WritableGrid))
			throw new UnsupportedOperationException(
					"The partition must be an instance of WritableGrid!");
		PartitionUtil.setPartition(this, (WritableGrid) partition,
				partitionBounds);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.SplittableResource#getSplitHeight()
	 */
	public int getSplitHeight() {
		return this.getHeight();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.SplittableResource#getSplitWidth()
	 */
	public int getSplitWidth() {
		return this.getWidth();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.DataPartition#getPartitionBounds()
	 */
	public Rectangle getPartitionBounds() {
		return new Rectangle(0, 0, this.getWidth(), this.getHeight());
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see appl.parallel.spmd.split.DataPartition#getEmpty()
	 */
	public DataPartition getEmpty(int id) {
		return new SplittableLLProxyGrid(this.metaData, id);
	}

}
