package appl.parallel.data.xulugridfile;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;
import java.io.IOException;
// fuer Doku
import java.io.RandomAccessFile;

/**
 * The methods of this class can be used to efficiently read or write values
 * into a Bytestream (it is e.g. used by the {@link XuluGridFile}).
 *
 * @author Dominik Appl
 *
 */
public class BufferedHelper {

	/**
	 * Reads a long value out of the given bytestream Used for buffered reading.
	 * Used the code of {@link RandomAccessFile#readInt()}
	 *
	 * @exception IOException
	 *                if an I/O error occurs.
	 */
	public static final int readIntFromStream(ByteArrayInputStream input)
			throws IOException {
		int ch1 = input.read();
		int ch2 = input.read();
		int ch3 = input.read();
		int ch4 = input.read();
		if ((ch1 | ch2 | ch3 | ch4) < 0)
			throw new EOFException();
		return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
	}

	/**
	 * Writes a int value in the the given bytestream Used for buffered writing.
	 * Used the code of {@link RandomAccessFile#writeInt(int)}
	 *
	 * @exception IOException
	 *                if an I/O error occurs.
	 *
	 */
	public static final void writeInt(int v, ByteArrayOutputStream output)
			throws IOException {
		output.write((v >>> 24) & 0xFF);
		output.write((v >>> 16) & 0xFF);
		output.write((v >>> 8) & 0xFF);
		output.write((v >>> 0) & 0xFF);
		// written += 4;
	}

	/**
	 * Writes a <code>long</code> to the stream as eight bytes, high byte
	 * first. The write starts at the current position of the file pointer. Used
	 * for buffered writing. Used the code of
	 * {@link RandomAccessFile#writeLong(long)}
	 *
	 * @param output the stream to be written to
	 * @param v
	 *            a <code>long</code> to be written.
	 * @exception IOException
	 *                if an I/O error occurs.
	 */
	public static final void writeLong(long v, ByteArrayOutputStream output)
			throws IOException {
		output.write((int) (v >>> 56) & 0xFF);
		output.write((int) (v >>> 48) & 0xFF);
		output.write((int) (v >>> 40) & 0xFF);
		output.write((int) (v >>> 32) & 0xFF);
		output.write((int) (v >>> 24) & 0xFF);
		output.write((int) (v >>> 16) & 0xFF);
		output.write((int) (v >>> 8) & 0xFF);
		output.write((int) (v >>> 0) & 0xFF);
		// written += 8;
	}

	/**
	 * Reads a long value out of the given {@link ByteArrayInputStream}. Used for buffered
	 * reading. Used the code of {@link RandomAccessFile#readLong()} *
	 *
	 * @exception IOException
	 *                if an I/O error occurs.
	 */
	public static final long readLongFromStream(ByteArrayInputStream input)
			throws IOException {
		return ((long) (readIntFromStream(input) << 32) + (readIntFromStream(input) & 0xFFFFFFFFL));
	}

}
