package appl.parallel.data.xulugridfile;

import schmitzm.data.WritableGrid;
import appl.data.DataLoader;
import appl.data.LoadingException;

/**
 * This {@link DataLoader} is responsible to load data from a
 * {@link XuluGridFile}. When {@link #load()} is called it tries to load the
 * whole(!) grid into the memory. Notice that using this loader will destroy all
 * memory advantages of the {@link XuluGridFile}. It is intended as standard loader in
 * the xulu application. The user can therefore force the loading of the grid
 * into the local memory e.g. for visualization.
 * 
 * @author Dominik Appl
 */
public class GridFileDataLoader implements DataLoader {

	private final XuluGridFile gridFile;

	public GridFileDataLoader(XuluGridFile gridFile) {
		this.gridFile = gridFile;
	}

	/* (non-Javadoc)
	 * @see appl.data.DataLoader#getLoadInfo()
	 */
	public String getLoadInfo() {
		// TODO Auto-generated method stub
		return "Loading gridfile " + gridFile.getOutputFile().getPath() + ". ";
	}

	/** loads the whole grid into memory and returns it
	 * @see appl.data.DataLoader#load()
	 */
	public WritableGrid load() throws LoadingException {
		try {
			return gridFile.getWholeGrid();
		} catch (XuluGridFileException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

}
