package appl.parallel.data.xulugridfile;

import java.awt.Rectangle;
import java.io.File;
import java.io.FileNotFoundException;

import schmitzm.data.WritableGrid;

import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import appl.data.DataLoader;
import appl.data.WritableGridArrayLoader;
import appl.data.WritableGridLLProxy;
import appl.parallel.spmd.split.DataPartition;
import appl.parallel.spmd.split.SplittableGrid;
import appl.parallel.spmd.split.SplittableResource;
import appl.util.RasterMetaData;

/**
 * An implementation of a WritableGrid based on a {@link XuluGridFile}.
 * 
 * @author Dominik Appl
 */
public class XuluWritableGridFile extends WritableGridLLProxy implements SplittableGrid{
	
	private XuluGridFile gridFile;
	private boolean wasWriteAccessed = false;
	private final String directory;
	private int rootID = this.hashCode();
	private final File baseFile;

	/**
	 * Opens the GridFile at the given destination
	 * @param gridfileName the filename of the XuluGridfile
	 * 
	 */
	public XuluWritableGridFile(File gridfileName) {
		super(null);
		this.baseFile = gridfileName;
		try {
			gridFile = new XuluGridFile(gridfileName);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XuluGridFileException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.metaData = gridFile.metaData;
		this.intialDataLoader = new GridFileDataLoader(gridFile);
		this.dataLoader = new GridFileDataLoader(gridFile);
		directory = null;
	}
	
	
	/**
	 * Creates NEW a GridFile in the given directory with the given metadata-sample
	 * 
	 * @param metaData the metadata
	 */
	public XuluWritableGridFile(RasterMetaData metaData, String directory) {
		super(metaData);
		this.directory = directory;
		baseFile = new File(directory + File.separatorChar + "xulugridFile"+ this.hashCode() + ".xgrid");
		try {
			gridFile = new XuluGridFile(baseFile,metaData);
			this.intialDataLoader = new GridFileDataLoader(gridFile);
			this.dataLoader = new GridFileDataLoader(gridFile);
		} catch (XuluGridFileException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			LOG.error(e);
		
		}
		
	}
	
	/** creates a instance out of a {@link XuluGridFile} */
	public XuluWritableGridFile(XuluGridFile xuluGridFile) {
		super(xuluGridFile.getMetaData());
		baseFile = xuluGridFile.getOutputFile();
		gridFile = xuluGridFile;
		this.metaData = xuluGridFile.getMetaData();
		this.intialDataLoader = new GridFileDataLoader(gridFile);
		this.dataLoader = new GridFileDataLoader(gridFile);
		// TODO Auto-generated constructor stub
		this.directory=null;
	}


	/**
	 * Creates a new XuluWritableGridFile in the specified directory, with the given metadata. 
	 * The ID given in this constuctor will be the RootID of this Grid (used e.g. for 
	 * identification in remoteComputing) 
	 * 
	 * @param newID the to be set as the new RootID (permanent)
	 * @param metaData the metadata is used for creation of the bounds and the type of the new {@link XuluGridFile}
	 * @param directory the directory in which the file should be created
	 * @see SplittableResource#getRootID()
	 */
	public XuluWritableGridFile(int newID, RasterMetaData metaData, String directory) {
		this(metaData,directory);
		this.rootID  = newID;
	}


	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.SplittableResource#getLocalLoader()
	 */
	public DataLoader getLocalLoader() {
		throw new UnsupportedOperationException("Feature not yet supported!");
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.SplittableResource#getRootID()
	 */
	public int getRootID() {
		return rootID;
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.SplittableResource#getSplitHeight()
	 */
	public int getSplitHeight() {
		return metaData.getHeight();
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.SplittableResource#getSplitWidth()
	 */
	public int getSplitWidth() {
		return metaData.getWidth();
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.DataPartition#getEmpty(int)
	 */
	public DataPartition getEmpty(int newID) {
		if(directory==null)
		   throw new UnsupportedOperationException("Directory == null. Cant create a empty grid!");
		return new XuluWritableGridFile(newID,this.metaData,this.directory);
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.DataPartition#getPartition(java.awt.Rectangle)
	 */
	public DataPartition getPartition(Rectangle partitionBounds) {
		try {
			return gridFile.getPartitialGrid2D(partitionBounds);
		} catch (XuluGridFileException e) {
			e.printStackTrace();
			LOG.error("Could not retrieve partition from underlaying gridfile");
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.DataPartition#getPartitionBounds()
	 */
	public Rectangle getPartitionBounds() {
		return new Rectangle(0,0,getWidth(),getHeight());
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.split.DataPartition#setPartition(appl.parallel.spmd.split.DataPartition, java.awt.Rectangle)
	 */
	public void setPartition(DataPartition partition, Rectangle partitionBounds) {
		try {
			gridFile.setPartition((WritableGrid) partition, partitionBounds);
		} catch (XuluGridFileException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#setGridSample(java.lang.Object, double[])
	 */
	@Override
	public void setGridSample(Object value, double... coord) {
		throw new UnsupportedOperationException("Writing on cell level is not yet supported. Use setPartition(..) to write to the Grid");
		//wasWriteAccessed =true;
//		super.setGridSample(value, coord);
	}
	
	 /* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#setRasterSample(java.lang.Object, int[])
	 */
	public void setRasterSample(Object value, int... cell) {
		 throw new UnsupportedOperationException("Writing on cell level is not yet supported. Use setPartition(..) to write to the Grid");

	 }


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSample(double[])
	 */
	@Override
	public Object getGridSample(double... coord) {
		 throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsByte(double[])
	 */
	@Override
	public byte getGridSampleAsByte(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsDouble(double[])
	 */
	@Override
	public double getGridSampleAsDouble(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsFloat(double[])
	 */
	@Override
	public float getGridSampleAsFloat(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsInt(double[])
	 */
	@Override
	public int getGridSampleAsInt(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsLong(double[])
	 */
	@Override
	public long getGridSampleAsLong(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}


	/* (non-Javadoc)
	 * @see appl.data.WritableGridLLProxy#getGridSampleAsShort(double[])
	 */
	@Override
	public short getGridSampleAsShort(double... coord) {
		throw new UnsupportedOperationException("Reading on cell level is not yet supported. Use getPartition(..) to read from the to the Grid");
	}
	/**
	 * unloading is not necessary (method does nothing
	 * 
	 * @see appl.data.LateLoadingProxy#unloadData()
	 */
	@Override
	public synchronized void unloadData() {
	}


	public String getGridFileName() {
		return baseFile.getPath();
	}
}
