package appl.parallel.event;

/**
 * A {@link CommEvent} is a {@link RemoteEvent}. It gives Information about
 * Events related to Communication.
 * 
 * @author Dominik Appl
 */
public class CommEvent extends RemoteEvent {

	/**
	 * The target of the event
	 */
	protected final String target;

	/**
	 * The source of the event
	 */
	protected final String src;

	/**
	 * the type of the event
	 */
	protected final CommType type;

	/**
	 * @author Dominik Appl
	 */
	public enum CommType {
		/**
		 * A not further specified type
		 */
		UNDEFINED,
		/**
		 * A Server has made an update from a source (possibly another server)
		 */
		REMOTE_UPDATE,
		/**
		 * A Server has executed a task (without client-server communication)
		 */
		REMOTE_EXECUTION,
		/**
		 * A Client has executed a Task (including communication)
		 */
		CLIENT_EXECUTION,
		/**
		 * A client has invoked an update
		 */
		CLIENT_UPDATE,
		/**
		 * A client has invoked a merge
		 */
		CLIENT_MERGE,
		/**
		 * Parameters were transfered
		 */
		TRANSFER_PARAMETERS,
		/**
		 * meta data was transfered
		 */
		TRANSFER_METADATA,
		/**
		 * data was transfered
		 */
		TRANSFER_DATA,
		/**
		 * Connection to servers were made
		 */
		CONNECT,
		/**
		 * The client disconnects from the servers
		 */
		DISCONNECT,
	}

	/**
	 * A new CommEvent
	 * 
	 * @param src
	 *            the source of the event
	 * @param target
	 *            the target
	 * @param type
	 *            the type
	 */
	CommEvent(String src, String target, CommType type) {
		this.target = (target == null) ? "" : target;
		this.src = (src == null) ? "" : src;
		this.type = (type == null) ? CommType.UNDEFINED : type;
	}

	/**
	 * @return the target
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * @return the source
	 */
	public String getSrc() {
		return src;
	}

	/**
	 * @return the type
	 */
	public CommType getType() {
		return type;
	}

	public String typeName() {
		return type.toString();
	}

}