package appl.parallel.event;

/**
 * A {@link TimeEvent} is a {@link CommEvent} which is associated with a 
 * execution time of a operation. 
 * 
 * @author Dominik Appl
 */
public class TimeEvent extends CommEvent {

	private final long time;

	/**
	 * @param timeNanos
	 *            the execution time to be associated with this event in nano
	 *            seconds
	 * @param src
	 *            the source of the event
	 * @param target
	 *            the target of the event
	 * @param type
	 *            the type of the event
	 * @see System#nanoTime()
	 */
	public TimeEvent(long timeNanos, String src, String target, CommType type) {
		super(src, target, type);
		this.time = timeNanos;
	}

	/**
	 * @return the execution time
	 */
	public long getTime() {
		return time;
	}

	public String toString() {
		StringBuffer returnString = new StringBuffer();
		returnString.append(type.toString());
		returnString.append(" execution time: ").append((time) / 1000000f)
				.append(" ms");
		if (!src.equals(""))
			returnString.append(" source: ").append(src);
		if (!target.equals(""))
			returnString.append(" target: ").append(target);

		return returnString.toString();
	}

}
