package appl.parallel.gui;

import appl.parallel.ComputingResourceContainer;
import appl.parallel.client.RemoteExecutionController;
import appl.parallel.model.AbstractParallelStepModel;
import appl.parallel.model.ParallelStepModel;
import appl.parallel.spmd.SPMDClientInterface;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.model.AbstractStepModel;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.gui.ModelControlContainer;

/**
 * This class extends the functionality of the container to support the execution
 * of a {@link ParallelStepModel}. It retrieves the {@link RemoteExecutionController}
 * and uses it to prepare the model for parallel execution.
 * 
 * @author Dominik Appl
 */
public class ModelControlContainer_parallel extends ModelControlContainer {

	private final XuluModellingPlatform appl;

	private final ParallelControlPanelEngine pcpEngine;

	/**
	 * A new instance
	 * @param model the XuluModel which should be executed
	 * @param pcpEngine the controlling engine for parallel execution
	 * @param appl the Xulu application
	 */
	public ModelControlContainer_parallel(XuluModel model,
			ParallelControlPanelEngine pcpEngine, XuluModellingPlatform appl) {
		super(model);
		this.pcpEngine = pcpEngine;
		// TODO Auto-generated constructor stub
		this.appl = appl;
	}

	@Override
	protected void createNewThread() {
		if (model instanceof ParallelStepModel) {
			// create the ClientController
			RemoteExecutionController remoteController = RemoteExecutionController
					.getRemoteExecutionController(appl);
			if (remoteController == null)
				throw new UnsupportedOperationException(
						"You must load the RemoteExecutionController-Plugin to execute parallel Models!");
			remoteController.prepareModelForSPMDExecution(
					(ParallelStepModel) model, pcpEngine
							.getSelectedResourceContainers());

		}
		super.createNewThread();
	}
}
