package appl.parallel.gui;

import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluConstants;

import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;
import edu.bonn.xulu.gui.ModelContentManagerContainer;

/**
 * This class introduces tab functionality into the ModelControlFrame
 * 
 * @author Dominik Appl
 * @version 1.0
 */
public class ModelControlFrame_Tabbed extends ModelControlFrame_Basic implements
		XuluConstants {

	protected JTabbedPane tabbedPane;

	/**
	 * Creates a new controlling window
	 * 
	 * @param appl
	 *            application in which the model is executed
	 * @param model
	 *            the model to be controlled through the frame
	 */
	public ModelControlFrame_Tabbed(XuluModellingPlatform appl, XuluModel model) {
		super(appl, model);
		this.pack();
	}

	/**
	 * inits the content pane as tabbed pane with one tab (the resource mapping)
	 * 
	 * @see edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic#initContentPane()
	 */
	@Override
	protected void initContentPane() {

		// ####### Daten-Komponente #######
		contentContainer = new ModelContentManagerContainer(model
				.getContentManager(), appl.getDataPool());
		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		tabbedPane.setPreferredSize(new Dimension((int) contentContainer
				.getPreferredSize().getWidth() + 5, (int) contentContainer
				.getPreferredSize().getHeight() + 25));
		JPanel panel = new JPanel(new GridBagLayout());

		panel.add(contentContainer, new GridBagConstraints(0, 1, 2, 1, 1.0,
				0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 25));

		tabbedPane.addTab("Resource Mapping", panel);
		contentPane.add(tabbedPane, new GridBagConstraints(0, 1, 2, 1, 1.0,
				0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
	}

	@Override
	protected ModelControlFrame_Tabbed newInstance(XuluModel newModel) {
		return new ModelControlFrame_Tabbed(appl, newModel);
	}

}
