package appl.parallel.server;

import java.awt.Rectangle;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Vector;

import appl.parallel.client.DataServer;
import appl.parallel.spmd.MultiDataInfo;
import appl.parallel.spmd.MultiDataObject;
import appl.parallel.spmd.split.DataPartition;
import appl.parallel.spmd.split.PartitionInfo;
import appl.parallel.spmd.split.SinglePartitionInfo;
import appl.parallel.spmd.split.SplittableResource;

/**
 * Manages data and partitions on servers. The method descriptions for details.
 * 
 * @see PartitionDataManager
 *
 * @author Dominik Appl
 */
public interface PartitionDataServer extends Remote, DataServer {

	/**
	 * gets the data with the id of a SplittableRessource
	 *
	 * @param id
	 *            the {@link SplittableResource#getRootID() ID} of a the base data
	 * @param bounds
	 *            the bounds of the partition to retrieve (using global
	 *            coordinates)
	 * @return the partition
	 * @throws RemoteException
	 */
	public DataPartition getPartition(int id, Rectangle bounds)
			throws RemoteException;

	/**
	 * sets the data with the id of a SplittableRessource
	 *
	 * @param id
	 *            the {@link SplittableResource#getRootID() ID} of a the base data
	 * @param bounds
	 *            the locatition where the partition is to be updated (using
	 *            global coordinates)
	 * @param updateData
	 *            the updateData (which may only a partitial update, depending
	 *            on the bounds set with the last parameter)
	 * @throws RemoteException
	 */
	public void updatePartition(int id, DataPartition updateData,
			Rectangle bounds) throws RemoteException;

	/**
	 * Unloads the specified resource to the source it was loaded from
	 *
	 * @param rootID
	 *            the id of the resource
	 */
	public void unloadToSource(int rootID) throws RemoteException;

	/**
	 * requests an update for the neighborhood region for the given resource
	 *
	 * @param rootID
	 */
	public void updateFromNeighbors(int rootID) throws RemoteException;

	/**
	 * Adds or updates BaseParameters
	 */
	public void updateBaseParameter(HashMap<String, Object> newParameters)
			throws RemoteException;

	/**
	 * Adds {@link PartitionInfo}s to the server. They are submitted in a
	 * vector to avoid multiple remote calls.
	 *
	 * @param singlePartitionInfos
	 * @throws RemoteException
	 */
	public void addPartitionInfos(
			Vector<SinglePartitionInfo> singlePartitionInfos)
			throws RemoteException;

	/**
	 * Adds {@link MultiDataInfo}s to the server.
	 *
	 * @param toTransferMultiDataObjects
	 */
	public void addMultiDataInfos(
			HashMap<String, MultiDataInfo> toTransferMultiDataObjects)
			throws RemoteException;

	/**
	 * @param name
	 *            the name of the associated multi data
	 * @return the {@link MultiDataInfo} info
	 * @throws RemoteException
	 */
	public MultiDataInfo getMultiDataInfo(String name) throws RemoteException;

	/**
	 * @param name
	 *            the name of the multiDataObject
	 * @return the MultiDataObject associated with that name
	 * @throws RemoteException
	 */
	public MultiDataObject getMultiDataObject(String name)
			throws RemoteException;

	/**
	 * @param rootID
	 *            the id of the according partition
	 * @return the the {@link PartitionInfo}
	 * @throws RemoteException
	 */
	public SinglePartitionInfo getPartitionInfo(int rootID)
			throws RemoteException;

}
