package appl.parallel.spmd;

import java.awt.Rectangle;

import schmitzm.data.WritableGrid;

/**
 * This class may be used as the superclass to all {@link SPMDTask SPMDTasks}
 * which should be executed on servers using the SPMD-paradigm. Subclasses
 * should override the {@link #init()} and the {@link #run(Object...)} method,
 * but never the {@link #initialize()} method. <br>
 * <br>
 * The Task provides access to the {@link SPMDClientInterface} and the
 * {@link AdvancedSPMDClientInterface}. <br>
 * <br>
 * Notice that the {@link #init()} method is called exactly one time before
 * execution for every instance (in multithreading there are multiple parallel
 * instances of the same task, running on different parts of the resource).
 * 
 * @author Dominik Appl
 */
public abstract class AbstractSPMDTask implements SPMDTask {

	protected transient AdvancedSPMDServerInterface serverController;

	private boolean initalized = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#incomingUpdate(schmitzm.data.WritableGrid,
	 *      java.awt.Rectangle)
	 */
	public void incomingUpdate(WritableGrid outgoingGrid, Rectangle location) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#outgoingUpdate(schmitzm.data.WritableGrid,
	 *      java.awt.Rectangle)
	 */
	public void outgoingUpdate(WritableGrid incomingGrid, Rectangle location) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#run()
	 */
	public abstract Object run(Object... parameters);

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#getSPMDServerController()
	 */
	public SPMDServerInterface getSPMDServerController() {
		return serverController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#getAdvancedSPMDServerController()
	 */
	public AdvancedSPMDServerInterface getAdvancedSPMDServerController() {
		return serverController;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#setSPMDServerController(appl.parallel.spmd.SPMDServerController)
	 */
	public void setSPMDServerController(AdvancedSPMDServerInterface controller) {
		this.serverController = controller;
	}

	/**
	 * Use this method to implement if you want t
	 */
	public abstract void init();

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#initialize()
	 */
	public final void initialize() {
		init();
		initalized = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see appl.parallel.spmd.SPMDTask#isInitialized()
	 */
	public boolean isInitialized() {
		return initalized;
	}

	/**
	 * returns false. Override this method to enable multithreading
	 * 
	 * @see appl.parallel.spmd.SPMDTask#supportsMultiThreading()
	 */
	public boolean supportsMultiThreading() {
		return false;
	}

}
