package appl.parallel.spmd;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Vector;

import appl.parallel.server.PartitionDataServer;
import appl.parallel.server.PartitionDataManager;

/**
 * Instances of this class encapsulate information about multiple objects
 * of the same type, e.g. lists of grids. In contrast to the MultiDataObject there
 * are no objects stored, but only the IDs of the objects. That simplifies the
 * the synchronization of multidata information over a network.
 *

 *
 * @author Dominik Appl
 */
public class MultiDataInfo implements Serializable {
	Vector<Integer> ids = new Vector<Integer>();

	String name;

	/**
	 * This constant is used as a divider for adding multiple data associated with the same
	 * name.
	 * The dividing should be as follows: name + MULTISPLITCONSTANT + index
	 */
	public static final String MULTISPLITCONSTANT = ";)|(;";

	public MultiDataInfo(int[] ids,  String name){
		this.name = name;
		for (int i : ids) {
			this.ids.add(i);
		}
	}

	/**
	 * Adds a new Element with the specified id.
	 * @param id The ID of the new element.
	 * @return the name of the new Element
	 *
	 */
	public String addElement(int id){
		ids.add(id);
		return getNameWithIdx(getCount()-1, name);
	}


	public int getMultiID(int idx){
		return ids.get(idx);
	}

	/**
	 * Generates a identification String based only on the given name and the given idx.
	 * It can be used for identification of the same data over multiple resources
	 *
	 * @param idx the index of the element in the multidata
	 * @param name the name of the element
	 * @return the identification string
	 *
	 * @see #getIndexFromNameWithIdx(String)
	 *
	 */
	public static String getNameWithIdx(int idx, String name){
		return  name + MULTISPLITCONSTANT + idx;
	}


	/**
	 * Extracts the the the index out of a identification
	 * @param identificationName the identification
	 * @return the index
	 *
	 * @see #getNameWithIdx(int, String)
	 */
	public static int getIndexFromNameWithIdx(String identificationName){
		String index = identificationName.split(MULTISPLITCONSTANT)[1];
		int i = Integer.valueOf(index);
		System.out.println("MulitDataInfo: Returning " + i);
		return i;
	}

	/**
	 * Generates a new id for a given index, based on the id of the first element
	 * in the multidata.
	 * @param idx the index for which a new id should be generated
	 * @return the new id
	 */
	public int getNewIDForIndex(int idx){
		return getMultiID(0) * (-1) - idx;
	}

	/**
	 * @return the number of elements currently in the multi-info
	 */
	public int getCount(){
		return ids.size();
	}

	/**
	 * @return the name associated with this multi-id
	 */
	public String getName(){
		return name;
	}
}
