package appl.parallel.spmd;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.Vector;

import appl.parallel.client.DataServer;
import appl.parallel.data.PartitionDataHandler;
import appl.parallel.server.PartitionDataServer;
import appl.parallel.server.PartitionDataManager;
import appl.parallel.spmd.split.DataPartition;
import appl.parallel.spmd.split.PartitionInfo;
import appl.parallel.spmd.split.SinglePartitionInfo;

/**
 *  A extension of the {@link MultiDataObject} for storing {@link DataPartition DataPartitions}.
 *    
 * @author Dominik Appl
 */
public class MultiDataPartitionObject extends MultiDataObject {

	private final PartitionDataServer partitionDataServer;
	private final MultiDataInfo info;

	public MultiDataPartitionObject(MultiDataInfo info, PartitionDataServer dataServer) {
		super(info, dataServer);
		this.info = info;
		partitionDataServer = dataServer;
	}

	/**
	 * Adds an element to the PartitionObject as {@link MultiDataObject} does. Also 
	 * adds the needed {@link PartitionInfo} Object for the new Element
	 * to the {@link PartitionDataServer}.
	 * 
	 * @return the index of the last element
	 */
	@Override
	public int addElement() {
		int lastIdx = super.addElement();
		int newID = this.getMultiInfo().getMultiID(lastIdx);
		//get PartitionInfo of the first grid and duplicate it
		try {
			SinglePartitionInfo partitionInfo = (SinglePartitionInfo) partitionDataServer.getPartitionInfo(info.getMultiID(0)).clone(newID);
			//add the new PartitionInfo to the server
			Vector<SinglePartitionInfo> partitionVector = new Vector<SinglePartitionInfo>();
			partitionVector.add(partitionInfo);
			partitionDataServer.addPartitionInfos(partitionVector);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return lastIdx;
	}
	

}
