package appl.parallel.spmd.split;

import java.awt.Rectangle;

/**
 * Represents a data partition. It has an id which identifies the resource, which is the mother
 * of the partition.
 *
 * @author Dominik Appl
 */
public interface DataPartition {

	/**
	 * Returns the id of the super data structure this partition is a part of.
	 * @return the id of the super data structure this partition is a part of.
	 */
	public int getRootID();

	 /**
	  * Returns the bounds of the partition as a rectangle. The coordinates
	  * of the rectangle are global (which means relative to the root structure).
	  * @return the bounds of the partition as a rectangle. The coordinates
	  * of the rectangle are global (which means relative to the root structure).
	 */
	 public Rectangle getPartitionBounds();
	 /**
     * Returns the partition specified by the given {@link Rectangle}. Note, that
     * partitioning by a rectangle does not mean, that the underlying datastructure
     * has to be 2D, but only that the splitting has to be at max. 2D.
     * @param partitionBounds the bounds of the partition which are to be retrieved
     * in global coordinates (the coordinates should be relative to the root-structure)
     */
    public DataPartition getPartition(Rectangle partitionBounds);

    /**
     * Overwrites the data at the location specified by the {@link Rectangle} with
     * the given partition-data.
     *
     * @param partition the new data
     * @param partitionBounds the target location of the data (in coordinates of the root structure)
     */
    public void setPartition(DataPartition partition, Rectangle partitionBounds);

	/**
	 * Creates an empty Object with the given id. (Used e.g. by Multi-Data-Objects)
	 * @param newID the id of the new Object
	 * @return an Empty element with the same dimension as this element.
	 */
	public DataPartition getEmpty(int newID);


}
