package appl.parallel.test;

import java.awt.Rectangle;

import schmitzm.data.WritableGrid;
import appl.parallel.spmd.AbstractSPMDTask;
import appl.parallel.spmd.SPMDServerInterface;
import appl.util.RasterUtil;

/**
 * @author Dominik Appl
 */
public class AverageNeighborhoodTestTask extends AbstractSPMDTask {

	/**
	 * 
	 */
	private static final long serialVersionUID = 13L;

    /* (non-Javadoc)
	 * @see appl.parallel.spmd.SPMDTask#run()
	 */
	public Object run(Object...parameters) {
		
		SPMDServerInterface controller = getSPMDServerController();
		WritableGrid gridInput = (WritableGrid) controller
				.getPartition("inputGrid");
		WritableGrid gridOutput = (WritableGrid) controller
				.getPartition("outputGrid");
		int neighborhoodRange = (Integer) parameters[0];
		/********************** START REMOTE ALGORITHM *********************
		 * start the calculation (notice that only the outer loop variables have changed)
		 * *****************************************************************/
		float overallSum = 0;
		Rectangle partition = controller.getLocalBounds();
		//System.out.println("290,64: " + gridInput.getRasterSampleAsFloat(controller.getLocalCalcMinX()+289,64));
		for (int y = controller.getLocalCalcMinY(); y < controller
				.getLocalCalcMaxY() + 1; y++)
			for (int x = controller.getLocalCalcMinX(); x < controller
					.getLocalCalcMaxX() + 1; x++) {
				float tmp = gridInput.getRasterSampleAsFloat(x,y);
				if(!Float.isNaN(tmp))
					overallSum += tmp;
				//the local sum is simply the sum over all elements in the neighborhood
				float localSum = 0;
				//number of cells over which the sum is calculated
				int noOfCells = 0;
				//for each cell: calculate the sum of all neighbors
				for (int y2 = y - neighborhoodRange; y2 <= y
						+ neighborhoodRange; y2++)
					for (int x2 = x - neighborhoodRange; x2 <= x
							+ neighborhoodRange; x2++)
						//check if the coordinates are valid (inside the grid and not NaN)
						if (partition.contains(x2, y2))
							if (!Float.isNaN(gridInput
									.getRasterSampleAsFloat(x2, y2))) {
								localSum += gridInput.getRasterSampleAsFloat(
										x2, y2);
								noOfCells++;
							}
				gridOutput.setRasterSample(localSum / (float) (noOfCells), x,y);

			}
	//copy values into results from output to input (for next step)
	RasterUtil.copyInto(gridOutput, gridInput);
	return overallSum;
	}

	/* (non-Javadoc)
	 * @see appl.parallel.spmd.AbstractSPMDTask#init()
	 */
	@Override
	public void init() {
		// TODO Auto-generated method stub
		
	}

	
	
}