package appl.parallel.test;

import java.awt.Rectangle;
import java.io.File;
import java.io.FileInputStream;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import appl.parallel.data.splittable.SplittableGridLLFactory;
import appl.parallel.data.splittable.SplittableLLProxyGrid;
import appl.parallel.spmd.split.AbstractSplitMap;
import appl.parallel.spmd.split.SplitMap;
import appl.parallel.spmd.split.SplitMap1DHorizontal;
import appl.parallel.spmd.split.SplitMap1DVertical;
import appl.parallel.spmd.split.SplitMap2D;
import appl.parallel.util.Helper;
import appl.util.GeneralUtil;
import appl.util.RasterMetaData;
import appl.util.RasterUtil;

import schmitzm.data.WritableGrid;
import schmitzm.data.WritableGridRaster;
import schmitzm.geotools.io.GeoImportUtil;

/**
 * Generated code for the test suite <b>SplitMapTest</b> located at
 * <i>/XuluSVN/javasrc/appl/parallel/test/SplitMapTest.testsuite</i>.
 *
 * Tests different splitmaps
 */
public class SplitMapTest extends TestCase {

	Rectangle globalCalcBounds;

	Rectangle partitionCalcBounds;

	Rectangle partitionNeighborhoodBounds;

	Rectangle globalNeighborhoodBounds;

	Rectangle localCalcBounds;

	Rectangle localNeighborhoodBounds;

	private SplittableLLProxyGrid baseGrid;

	private SplitMap map;

	/**
	 * Constructor for SplitMapTest.
	 * @param name
	 */
	public SplitMapTest(String name) {
		super(name);
	}

	/**
	 * Returns the JUnit test suite that implements the <b>SplitMapTest</b>
	 * definition.
	 */
	public static Test suite() {
		TestSuite splitMapTest = new TestSuite("SplitMapTest");
		//		splitMapTest.setArbiter(DefaultTestArbiter.INSTANCE).setId(
		//				"D35DB07792EF13413F23D9D0B19111DB");
		//	
		//		splitMapTest.addTest(new SplitMapTest("test1DSplitMap").setId(
		//				"D35DB07792EF134149AEAC40B19111DB").setTestInvocationId(
		//				"D35DB07792EF134168064FE0B19111DB"));
		return splitMapTest;
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		//		load a very simple 10x10 Grid 
		WritableGrid loadGrid = GeoImportUtil
				.readGridRasterFromArcInfoASCII(new File(
						"../Xulu-Data/regularGrid.arc"));
		baseGrid = new SplittableLLProxyGrid(new RasterMetaData(loadGrid),2);
		RasterUtil.copyInto(loadGrid, baseGrid);
		//RasterUtil.printGrid(baseGrid, "Base grid");
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
	}

	public void basicTest() {
		//make simply 2 Partitions without neighborhood
		map.makeMap();
		//check first partition
		initVariables(map, 0);
		assertEquals(new Rectangle(0, 0, 10, 10), globalCalcBounds);
		assertEquals(new Rectangle(0, 0, 10, 10), globalNeighborhoodBounds);
		assertEquals(new Rectangle(0, 0, 5, 10), partitionCalcBounds);
		assertEquals(new Rectangle(0, 0, 5, 10), partitionNeighborhoodBounds);
		assertEquals(new Rectangle(0, 0, 5, 10), localCalcBounds);
		assertEquals(new Rectangle(0, 0, 5, 10), localNeighborhoodBounds);
		//check second partition
		initVariables(map, 1);
		assertEquals(globalCalcBounds, new Rectangle(0, 0, 10, 10));
		assertEquals(globalNeighborhoodBounds, new Rectangle(0, 0, 10, 10));
		assertEquals(partitionCalcBounds, new Rectangle(5, 0, 5, 10));
		assertEquals(partitionNeighborhoodBounds, new Rectangle(5, 0, 5, 10));
		assertEquals(localCalcBounds, new Rectangle(0, 0, 5, 10));
		assertEquals(localNeighborhoodBounds, new Rectangle(0, 0, 5, 10));
	}

	public void test1DHorizontal() {
        //2 Partitions
		map = new SplitMap1DHorizontal(baseGrid.getWidth(), baseGrid.getHeight(),
				1, 2, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
		
		
	    
	    map = new SplitMap1DHorizontal(baseGrid.getWidth(), baseGrid.getHeight(),
				1, 3, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
	   
	    
	}
	
	public void print(){
		for(int i=0; i < map.getCount(); i++){
			RasterUtil.printGrid(baseGrid.getPartition(map.getPartitionCalculationBounds(i)),3,0,"Calculation Partition " + (i+1) + "/" + map.getCount());
		}
		for(int i=0; i < map.getCount(); i++){
			RasterUtil.printGrid(baseGrid.getPartition(map.getPartitionNeighborhoodBounds(i)),3,0,"Neighborhood Partition " + (i+1) + "/" + map.getCount());
		}
	}
	
	public void test2DIrregular(){
//		map = new SplitMap2D(10, 10,
//				1, 3, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
		
	    map = new SplitMap2D(baseGrid.getWidth(), baseGrid.getHeight(),
				1, 9, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
	    print();
	}

	/**
	 * 1DSplitMap
	 * @throws Exception
	 */
	public void test1DVertical() throws Exception {
		//2 Partitions
		map = new SplitMap1DVertical(baseGrid.getWidth(), baseGrid.getHeight(),
				0, 2, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
		basicTest();
		//make 3 Partitions
		map = new SplitMap1DVertical(baseGrid.getWidth(), baseGrid.getHeight(),
				0, 3, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
		
		//make 2 Partitions with neighborhoodrange 2
		map = new SplitMap1DVertical(baseGrid.getWidth(), baseGrid.getHeight(),
				2, 2, AbstractSplitMap.NeighborhoodBoxingMode.inBoxing);
		//intersect the partitions and see if you have the right rect

		Rectangle r = map.getPartitionNeighborhoodBounds(0).intersection(
				map.getPartitionNeighborhoodBounds(1));
		assertEquals(new Rectangle(3, 0, 4, 10), r);

		initVariables(map, 1);
	}

	/**
	 * @param map
	 */
	private void initVariables(SplitMap map, int pos) {
		globalCalcBounds = map.getGlobalCalculationBounds();
		partitionCalcBounds = map.getPartitionCalculationBounds(pos);
		partitionNeighborhoodBounds = map.getPartitionNeighborhoodBounds(pos);
		globalNeighborhoodBounds = map.getGlobalBounds();
		localCalcBounds = map.getLocalCalculationBounds(pos);
		localNeighborhoodBounds = map.getLocalNeighborhoodBounds(pos);

	}

}
