package appl.parallel.thread;
/**
 * Executes a method call in a single Thread
 *
 * @author Dominik Appl
 */
public abstract class OneMethodThread implements Runnable {
	private volatile Thread thisThread;

	private final String threadName;

	private final int priority;

	private final Object[] parameters;

	protected Object executionResult;

	/**
	 * Creates a new Thread. Do not forget to {@link #start()} it.
	 * @param threadName a name for the thread (usefull while debugging)
	 * @param priority a priority for the Thread
	 * @param parameters the parameters may be used later in the run method by calling the {@link #getParameter(int)} method
	 *
	 */
	public OneMethodThread(String threadName, int priority, Object... parameters) {
		this.threadName = threadName;
		this.priority = priority;
		// TODO Auto-generated constructor stub
		this.parameters = parameters;
	}

	public abstract void run();

	/**
	 * Starts the thread
	 */
	public void start() {
		thisThread = new Thread(this);
		thisThread.setName(threadName);
		thisThread.setPriority(priority);
		thisThread.start();
	}

	/**
	 * @param paraNo the position in the given parameter array
	 * @return the parameter given in the constructor
	 */
	public Object getParameter(int paraNo) {
		return parameters[paraNo];
	}

	/**
	 * waits until the encapulated Thread has finished
	 *
	 */
	public void join(){
		try {
			thisThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}


	/**
	 * the execution result must be explicitly set in the {@link #run()} method.
	 *
	 * @return the result
	 */
	public Object getExecutionResult(){
		return executionResult;
	}

}
