package appl.plugin.multimodelcontrol;

import java.awt.Color;
import java.awt.PopupMenu;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import appl.parallel.gui.ParallelControlPanelEngine;

import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectListener;
import schmitzm.lang.WorkingThread;
import schmitzm.lang.WorkingThreadAdapter;
import schmitzm.lang.WorkingThreadListener;
import schmitzm.swing.TextAreaPrintStream;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.ModelControlManager;
import edu.bonn.xulu.gui.ModelControlFrame;
import edu.bonn.xulu.model.StepModel;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.model.event.ModelEvent;
import edu.bonn.xulu.model.event.ModelListener;
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;

/**
 * The class allows to control several models at once. At the moment it provides
 * a batch like execution functionality. It is still under development but
 * should basically work. It was developed in very short time and is therefore
 * not as well commented, as other classes.
 * 
 * @author Dominik Appl
 */
public class MultiModelControlHandler implements ListSelectionListener,
		ObjectListener, ActionListener {

	private MultiModelControlFrame frame;

	private final XuluModellingPlatform xulu;

	private DefaultListModel modelListModel;

	private ParallelControlPanelEngine parallelControlPanelEngine;

	private DefaultListModel commandListModel;

	private InterpretingThread interpretingThread;

	private TextAreaPrintStream consoleStream;

	private String string;

	public MultiModelControlHandler(XuluModellingPlatform xulu) {
		this.xulu = xulu;
		frame = new MultiModelControlFrame();
		initGUI();
		consoleStream = new TextAreaPrintStream(frame.console);
	}

	/**
	 * 
	 */
	private void initGUI() {
		// register as listener for changes in the ModelControlManager
		xulu.getModelControlManager().addObjectListener(this);
		// init model list
		initModelList();
		initCommandList();
		registerListeners();
	}

	/**
	 * 
	 */
	private void registerListeners() {
		frame.addStepCommand.addActionListener(this);
		frame.newOverAllStep.addActionListener(this);
		frame.clearModelSteps.addActionListener(this);
		frame.startButton.addActionListener(this);
		frame.stopButton.addActionListener(this);
		frame.stepButton.addActionListener(this);
		frame.modelList.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(java.awt.event.MouseEvent e) {
				if (e.getClickCount() == 2) {
					handleModelListDoubleClick();
				}
			}
		});
	}

	/**
	 * 
	 */
	private void initCommandList() {
		commandListModel = new DefaultListModel();
		frame.syncCommandList.setModel(commandListModel);
		commandListModel.addElement(new StartCommand());
		commandListModel.addElement(new StopCommand());
		frame.syncCommandList
				.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		frame.syncCommandList.setSelectedIndex(0);
	}

	/**
	 * inits the modelList
	 */
	private void initModelList() {
		modelListModel = new DefaultListModel();
		frame.modelList.setModel(modelListModel);
		frame.modelList.addListSelectionListener(this);
		int modelCount = xulu.getModelControlManager().getCount();
		for (int i = 0; i < modelCount; i++) {
			ModelControlFrame frame = xulu.getModelControlManager().getAll()[i];
			this.addModelControlFrame(frame);
		}
		// frame.modelList.add(frame.modelPopupMenu);
	}

	public MultiModelControlFrame getFrame() {
		return frame;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see schmitzm.data.event.ObjectListener#performObjectEvent(schmitzm.data.event.ObjectEvent)
	 */
	public void performObjectEvent(ObjectEvent e) {
		// only listen for ChangeEvents
		if (!(e instanceof ModelControlManager.ChangeEvent))
			return;
		ModelControlManager.ChangeEvent event = (ModelControlManager.ChangeEvent) e;
		// if model was added:
		if (event.getOldValue() == null && event.getNewValue() != null
				&& event.getNewValue() instanceof ModelControlFrame)
			addModelControlFrame((ModelControlFrame) event.getNewValue());
		// if model was removed
		if (event.getOldValue() != null && event.getNewValue() == null
				&& event.getOldValue() instanceof ModelControlFrame)
			removeModelControlFrame((ModelControlFrame) event.getOldValue());
	}

	/**
	 * @param model
	 */
	private void addModelControlFrame(ModelControlFrame frame2) {
		modelListModel.addElement(new ModelListObject(frame2,
				frame2.getModel(), frame2.getModel().getName()));

	}

	/**
	 * Removes the model, which belongs to the specified ModelControlFrame from
	 * the list in the MCC
	 * 
	 * @param frame2
	 *            the ModelControlFrame from the removed model
	 */
	private void removeModelControlFrame(ModelControlFrame frame2) {
		// find the index of the model in the vector
		for (int i = 0; i < modelListModel.size(); i++) {
			ModelListObject m = (ModelListObject) modelListModel.get(i);
			if (frame2.equals(m.getFrame()))
				modelListModel.remove(i);
		}
	}

	/**
	 * @return the actually selected ModelListObject from the model list
	 */
	private ModelListObject getSelectedModelListObject() {
		return (ModelListObject) frame.modelList.getSelectedValue();

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged(ListSelectionEvent e) {
		// TODO Auto-generated method stub
	}

	abstract class ModelCommand {
		protected boolean finished = false;

		public abstract String toString();

		public void execute() {
			finished = true;
		}

		public void reset() {
			finished = false;
		}

		// /**
		// * executed after all commands are executed
		// */
		// public void dispose()
		// {
		//			
		// }

		/**
		 * @return true, if the execution is finished and no further execution
		 *         is required
		 */
		public boolean hasFinished() {
			return finished;
		}

		/**
		 * tries to stop the execution of the command
		 */
		public void stop() {
		};

		/**
		 * @return Message to be displayed in the console
		 */
		public String startMessage() {
			return null;
		}

		/**
		 * @return Message to be displayed
		 */
		public String finshedMessage() {
			return null;
		}
	}

	class RepeatStartCommand extends ModelCommand {
		private final int execTimes;

		private int alreadyExec;

		/**
		 * @param execTimes
		 */
		public RepeatStartCommand(int execTimes) {
			this.execTimes = execTimes;
			alreadyExec = 0;
			// TODO Auto-generated constructor stub
		}

		public String toString() {
			return "Start Repeat (" + execTimes + " exec)";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#finshedMessage()
		 */
		@Override
		public String finshedMessage() {
			return "Starting Repeat (" + (alreadyExec) + "/" + execTimes + ")";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#startMessage()
		 */
		@Override
		public String startMessage() {
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#execute()
		 */
		@Override
		public void execute() {
			alreadyExec++;
			finished = true;
		}

		public void reset() {
			finished = false;
			alreadyExec = 0;
		}
	}

	/**
	 * The commands between start and stop are repeated
	 * 
	 * @author Dominik Appl
	 */
	class RepeatStopCommand extends ModelCommand {
		private final int execTimes;

		private int alreadyExec;

		/**
		 * @param execTimes
		 */
		public RepeatStopCommand(int execTimes) {
			this.execTimes = execTimes;
			alreadyExec = 0;
		}

		public String toString() {
			return "End Repeat (" + execTimes + " exec)";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#finshedMessage()
		 */
		@Override
		public String finshedMessage() {
			return "Loop iteration finished.(" + alreadyExec + "/" + execTimes
					+ ")";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#startMessage()
		 */
		@Override
		public String startMessage() {
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#execute()
		 */
		@Override
		public void execute() {
			alreadyExec++;
			if (alreadyExec == execTimes)
				finished = true;
		}

		public void reset() {
			finished = false;
			alreadyExec = 0;
		}
	}

	class StartCommand extends ModelCommand {
		public String toString() {
			return "------- START MODELLING -------";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#finshedMessage()
		 */
		@Override
		public String finshedMessage() {
			return "Starting execution...";
		}

	}

	class StopCommand extends ModelCommand {
		public String toString() {
			return "------- MODELLING FINISHED -------";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#finshedMessage()
		 */
		@Override
		public String finshedMessage() {
			return "Finished execution";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#startMessage()
		 */
		@Override
		public String startMessage() {
			return null;
		}
	}

	class ModelStepCommand extends ModelCommand {
		private ModelControlFrame_Basic controlFrame;

		private final Lock lock = new ReentrantLock();

		private final Condition threadStillRunning = lock.newCondition();

		private String name;

		private XuluModel model;

		private final int execTimes;

		private int finishedExecutions = 0;

		public void reset() {
			finishedExecutions = 0;
		}

		/**
		 * 
		 */
		public ModelStepCommand(ModelListObject mlo, int execTimes) {
			this.execTimes = execTimes;
			if (!(mlo.getFrame() instanceof ModelControlFrame_Basic))
				throw new UnsupportedOperationException(
						"The modelframe must be an instance of ModelControlFrame_Basic");
			controlFrame = (ModelControlFrame_Basic) mlo.getFrame();
			name = mlo.getName();
			model = mlo.getModel();
		}

		public void execute() {
			// check first if the model must be initialised
			if (!model.isInitialised() && !model.isRunning()) {
				controlFrame.setModelResources();
				controlFrame.getControlContainer().init();
			}
			lock.lock();
			try {
				WorkingThreadAdapter adapter = new WorkingThreadAdapter() {
					/*
					 * (non-Javadoc)
					 * 
					 * @see schmitzm.lang.WorkingThreadAdapter#threadPaused(schmitzm.lang.WorkingThread)
					 */
					@Override
					public void threadPaused(WorkingThread thread) {
						try {
							lock.lock();
							// signal finished if the step is finished
							threadStillRunning.signal();
						} finally {
							lock.unlock();
						}
					}

					/*
					 * (non-Javadoc)
					 * 
					 * @see schmitzm.lang.WorkingThreadAdapter#threadStopped(schmitzm.lang.WorkingThread)
					 */
					@Override
					public void threadStopped(WorkingThread thread) {
						try {
							lock.lock();
							// signal finished if the step is finished
							threadStillRunning.signal();
						} finally {
							lock.unlock();
						}
					}
				};
				controlFrame.getControlContainer().getModelThread()
						.addThreadListener(adapter);
				controlFrame.getControlContainer().step();
				// wait for the model step to finish
				threadStillRunning.await();
				controlFrame.getControlContainer().getModelThread()
						.removeThreadListener(adapter);
				finishedExecutions++;
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} finally {
				lock.unlock();
			}
		}

		public void stop() {
			if (!model.isStopped())
				controlFrame.getControlContainer().stop();
			// try {
			// controlFrame.getControlContainer().getModelThread().join();
			// } catch (InterruptedException e) {
			// // TODO Auto-generated catch block
			// e.printStackTrace();
			// }
		}

		@Override
		public boolean hasFinished() {
			return finishedExecutions >= execTimes;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#toString()
		 */
		@Override
		public String toString() {
			return name + " (" + execTimes + " exec)";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#finshedMessage()
		 */
		@Override
		public String finshedMessage() {
			return "...finished";
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see appl.plugin.multimodelcontrol.MultiModelControlHandler.ModelCommand#startMessage()
		 */
		@Override
		public String startMessage() {
			return "Executing step of " + name;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == frame.addStepCommand) {
			handleAddNewStep();
		}
		if (e.getSource() == frame.newOverAllStep) {
			handleRepeatStep();
		}
		if (e.getSource() == frame.startButton) {
			startInterpreting();
		}
		if (e.getSource() == frame.stopButton) {
			if (interpretingThread != null) {
				interpretingThread.stopThread();
			}
			// ToDO: dispose all models

		}
		if (e.getSource() == frame.clearModelSteps) {
			int delIdx = frame.syncCommandList.getSelectedIndex();
			// do not delete first or last entry
			if (delIdx > 0 && delIdx < commandListModel.getSize() - 1)
				commandListModel.remove(delIdx);
			frame.syncCommandList.setSelectedIndex(delIdx);

		}
	}

	/**
	 * 
	 */
	private void handleAddNewStep() {
		if (checkInsertAllowed()) {
			if (getSelectedModelListObject() == null) {
				JOptionPane.showMessageDialog(null, "No model selected",
						"No model selected", JOptionPane.ERROR_MESSAGE);
				return;
			}
			int selIdx = frame.syncCommandList.getSelectedIndex();
			int execTimes = 1;
			// get execution times from user
			try {
				string = JOptionPane.showInputDialog(frame,
						"How many steps do you want to execute for this enty?",
						1);
				execTimes = Integer.valueOf(string);
				if (execTimes <= 0)
					throw new UnsupportedOperationException(
							"Only positive number allowed!");
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(null,
						"Only positive number allowed", "Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			commandListModel.add(selIdx + 1, new ModelStepCommand(
					getSelectedModelListObject(), execTimes));
			// select new entry
			frame.syncCommandList.setSelectedIndex(selIdx + 1);
		}
	}

	/**
	 * 
	 */
	protected void handleModelListDoubleClick() {
		ModelListObject selectedModelListObject = getSelectedModelListObject();
		if (selectedModelListObject != null) {
			selectedModelListObject.getFrame().setVisible(true);
			selectedModelListObject.getFrame().toFront();
		}

	}

	/**
	 * 
	 */
	private void handleRepeatStep() {
		if (checkInsertAllowed()) {
			int selIdx = frame.syncCommandList.getSelectedIndex();
			int execTimes = 1;
			// get execution times from user
			try {
				string = JOptionPane.showInputDialog(frame,
						"How many steps do you want to execute for this enty?",
						1);
				execTimes = Integer.valueOf(string);
				if (execTimes <= 0)
					throw new UnsupportedOperationException(
							"Only positive numbers allowed!");
			} catch (Exception ex) {
				JOptionPane.showMessageDialog(null,
						"Only positive numbers allowed", "Error",
						JOptionPane.ERROR_MESSAGE);
				return;
			}
			commandListModel.add(selIdx + 1, new RepeatStartCommand(execTimes));
			commandListModel.add(selIdx + 2, new RepeatStopCommand(execTimes));

			// select new entry
			frame.syncCommandList.setSelectedIndex(selIdx + 1);
		}
	}

	private void startInterpreting() {
		// clear console and start Thread
		frame.console.setText("");
		interpretingThread = new InterpretingThread();
		interpretingThread.start();

	}

	public void toggleGUI(boolean enabled) {
		frame.modelList.setEnabled(enabled);
		frame.addStepCommand.setEnabled(enabled);
		frame.newOverAllStep.setEnabled(enabled);
		frame.clearModelSteps.setEnabled(enabled);
		if (enabled)
			frame.syncCommandList.setSelectionBackground(Color.LIGHT_GRAY);
		else
			frame.syncCommandList.setSelectionBackground(Color.red);
	}

	/**
	 * insert is allowed if its not the last position
	 */
	private boolean checkInsertAllowed() {
		if (commandListModel.getSize() - 1 > frame.syncCommandList
				.getSelectedIndex())
			return true;
		JOptionPane.showMessageDialog(null,
				"You can add nothing after the end command!", "Error",
				JOptionPane.ERROR_MESSAGE);
		return false;
	}

	/**
	 * Interprets the commandlist
	 * 
	 * @author Dominik Appl
	 */
	class InterpretingThread extends Thread {

		private boolean exit;

		private ModelCommand currentCommand;

		/**
		 * Creates a new Thread
		 */
		public InterpretingThread() {
			exit = false;
		}

		/**
		 * stops the Thread execution
		 */
		public void stopThread() {
			exit = true;
			if (currentCommand != null)
				currentCommand.stop();
		}

		public void run() {
			toggleGUI(false);
			// reset all commands
			for (int i = 0; i < frame.syncCommandList.getModel().getSize(); i++)
				((ModelCommand) frame.syncCommandList.getModel()
						.getElementAt(i)).reset();
			// select first element
			try {
				frame.syncCommandList.setSelectedIndex(0);
				int currentIdx = 0;
				while (exit != true) {
					currentCommand = (ModelCommand) commandListModel
							.getElementAt(currentIdx);
					// if the current command is an end of of a global loop
					// than go back in the list until the start of the loop is
					// found
					if (currentCommand instanceof RepeatStopCommand) {
						currentCommand.execute();
						// if the all steps are done, just go on
						if (currentCommand.hasFinished()) {
							currentIdx++;
							currentCommand = (ModelCommand) commandListModel
									.getElementAt(currentIdx);
						}
						// else do the repetition by decreasing the index
						// until the RepeatStartCommand is reached
						else {
							while (!(currentCommand instanceof RepeatStartCommand)) {
								currentIdx--;
								// if some structure error occured:
								if (currentIdx <= 0) {
									JOptionPane
											.showMessageDialog(
													frame,
													"Something is wrong with the command structure. Do not use nested loops or other komplex structures!");
									exit = true;
									break;
								}
								currentCommand = (ModelCommand) commandListModel
										.getElementAt(currentIdx);
							}
						}

					}
					if (currentCommand.startMessage() != null)
						consoleStream.print(currentCommand.startMessage());
					frame.syncCommandList.setSelectedIndex(currentIdx);
					currentCommand.execute();
					if (currentCommand.finshedMessage() != null)
						consoleStream.println(currentCommand.finshedMessage());
					// if the command has finished executing, get the next
					// command
					if (currentCommand.hasFinished())
						currentIdx++;
					if (currentIdx == commandListModel.getSize())
						exit = true;
				}
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				toggleGUI(true);
			}
		}
	}

}
