package appl.util;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

/**
 * Simple {@link TableModel} that allows to specify columns which are not editable
 * 
 * @author Dominik Appl
 */
public class NonEditableTableModel extends DefaultTableModel {
	private final boolean[] isEditable;

	/**
	 * @param columnNames
	 *            <code>array</code> containing the names of the new columns;
	 *            if this is <code>null</code> then the model has no columns
	 * 
	 * @param isEditable Specifies for each column if editable or not
	 * 
	 * @param rowCount
	 *            the number of rows the table holds
	 */
	public NonEditableTableModel(Object[] columnNames, boolean[] isEditable,
			int rowCount) {
		super(columnNames, rowCount);
		this.isEditable = isEditable;
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return isEditable[column];
	}

}
