/** GnuR Plugin - A XULU-plugin for applying R-powered statistics on objects in the XULU-datapool.
    Copyright (C) 2007  Stefan A. Krüger

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/   
package de.skrueger.xulu.plugin.gnur;

/**
 * TODO 
 * Ausführen von .R skripten
 * Löschen des Workspace!?
 * Übergabe von Grids
 * Eingabefeld zu TextFied für mherzeilige Eingaben
 */

import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.RList;
import org.rosuda.JRI.RVector;
import org.rosuda.JRI.Rengine;

import schmitzm.data.RasterCalculator;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.property.ListProperty;
import schmitzm.data.property.ListPropertyReadAccess;
import schmitzm.data.property.Property;
import schmitzm.swing.ManualInputOption;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.DataPool;
import edu.bonn.xulu.gui.XuluInternalFrame;
import edu.bonn.xulu.plugin.data.grid.MultiGrid;
import edu.bonn.xulu.plugin.data.grid.SingleGrid;

public class GnuRGUI extends XuluInternalFrame {
  /** Speichert eine Referenz auf das Fenster-Objekt, um aus verschachtelten
   *  Klassen einfach darauf zugreifen zu koennen. */
  protected final GnuRGUI THIS = this;
  /** Speichert eine Referenz auf die Xulu-Applikation, in der der Code-Generator
   *  ausgefuehrt wird. */
  protected XuluModellingPlatform appl = null;
  /** Tabelle in der die Eingabe-Raster angegeben werden. */
  protected RVarsTable            rVarsTable = null;
  private   RVarsTableModel 	  rVarsTableModel = null;
  private   JScrollPane           rVarsScrollPane = null;

  /** Eingabefeld fuer die Formel. */
  protected ManualInputOption.Text rule = null;
  /** Button to Import the Variables to R from the datapool. */
  protected JButton importButton = null;
  /** Button to Export the Variables from R to the datapool. */
  protected JButton exportButton = null;

  /** Referenz auf den Container des Fensters. */
  protected Container contentPane = null;
  /** The RConsole-Panel contains of ab Output- and an Inputfied for R */
  private RConsolePanel rConsole = null;

  /**
   * Erzeugt eine neue GUI fuer den {@link RasterCalculator}.
   * @param appl Instanz der Xulu-Applikation, in der der Rechner
   *             ausgefuehrt wird
   */
  public GnuRGUI(XuluModellingPlatform appl) {
    super("GNU-R toolbox");
    this.appl = appl;

    this.setVisible(false);
    this.setSize( new Dimension(500,400) );
    this.setClosable(true);
    this.setDefaultCloseOperation( HIDE_ON_CLOSE );
    this.getContentPane().setLayout( new GridBagLayout() );
    contentPane  = this.getContentPane();
    initGUI();
    refresh();

    // Listener auf den Datenpool setzen, damit Auswahlfelder automatisch
    // aktualisiert werden
    appl.getDataPool().addObjectListener( new ObjectListener() {
      public void performObjectEvent(ObjectEvent e) {
        if ( e instanceof DataPool.DataPoolChangeEvent )
          refresh();
      }
    });
  }

  /**
   * Initalisiert die GUI des Fensters.
   */
  protected void initGUI() {
	// Input-Raster-Tabelle
	rVarsTableModel = new RVarsTableModel(appl);
	rVarsTable = new RVarsTable(rVarsTableModel);
	rVarsTable.setToolTipText("Simply do drang and drop");
	rVarsTable.setPreferredScrollableViewportSize( new Dimension(400,200));
	rVarsScrollPane = rVarsTable.createScrollPane();
	// Formel
	rule = new ManualInputOption.Text("Rule:", true);
	rule
		.setToolTipText("Insert your arithmetical rule here (Use arithmetical operators, constants and #1, #2, ... as raster reference identifier)");
	// Button zum Starten
	importButton = new JButton("Import");
	importButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		importRVars();
		// After Import of vars into R, show all vars..
		rConsole.sendToConsole("ls()");
	    }
	});

	exportButton = new JButton("Export");
	exportButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		exportRVars();
	    }
	});

	// Drag & Drop fuer RVars
	DropTarget dropTarget = new DropTarget(appl.getMainFrame(),
		new DropTargetAdapter() {
		    public void dragEnter(DropTargetDragEvent e) {

			// Pruefen, ob mind, ein akzeptables Objekt im Datenpool
                        // selektiert ist
			Object[] selObjects = appl.getMainFrame()
				.getDataPoolFrame().getSelectedObjects();
			for (Object obj : selObjects) {
			    try {
				RVarAdapter.createFrom((Property) obj, true);
        			return;
			    } catch (PropertytypeNotConvertableToRVar ee) {
				// ee.printStackTrace();
				System.out.println("Not convertable..");
			    } 
			    System.out.println(obj.getClass());
			}
			// Reject drop, if nothing convertible was found.
			e.rejectDrag();
		    }

		    public void drop(DropTargetDropEvent e) {
			Object[] selObjects = appl.getMainFrame()
				.getDataPoolFrame().getSelectedObjects();

			for (Object obj : selObjects) {
			    // rVarsTableModel.addRVariable(
				    //RVarAdapter.createFrom(obj, false)  );
			    refresh();
			}
		    }
		});

	rVarsTable.setDropTarget(dropTarget);
	rVarsScrollPane.setDropTarget(dropTarget);

	/*
         * The R-Console Window
         */
	try {
	    rConsole = new RConsolePanel();
	    rConsole.init();
	} catch (IOException e1) {
	    e1.printStackTrace();
	}

	// Komponenten in Fenster anordnen
	contentPane.add(new JLabel("Input/Output-Rasters (use context menu or simply drag rasters from xulu data pool):"), new GridBagConstraints(
		0, 0, 2, 1, 0, 0,
		GridBagConstraints.NORTHWEST,
		GridBagConstraints.HORIZONTAL, 
		new Insets(5, 10, 0,10), 0, 0));
	
	contentPane.add(rVarsScrollPane, new GridBagConstraints(
		0, 1, 2, 3, 0.1, 0.1, 
		GridBagConstraints.CENTER, 
		GridBagConstraints.BOTH,
		new Insets(0, 10, 5, 10), 0, 0));

	contentPane.add(rConsole, new GridBagConstraints(
		0, 4, 2, 2, 0.5, 0.5,
		GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH, 
		new Insets(0, 10, 5, 10), 0, 0));

	contentPane.add(importButton, new GridBagConstraints(
		0, 6, 1, 1, 0.0, 0, 
		GridBagConstraints.WEST, 
		GridBagConstraints.NONE,
		new Insets(0, 10, 5, 10), 0, 0));
	contentPane.add(exportButton, new GridBagConstraints(
		1, 6, 1, 1, 0.0, 0, 
		GridBagConstraints.WEST, 
		GridBagConstraints.NONE,
		new Insets(0, 10, 5, 10), 0, 0));
    }


/**
 * Aktualisiert die Auswahl-Felder auf Basis des Datenpools.
 */
  public void refresh() {
    // Aktualisierung der Tabelle
    rVarsTableModel.refreshInputOption();
    rVarsTableModel.fireTableDataChanged();
  }




  /**
   */
  private void exportRVars() {
		if (rConsole.getREngine() == null) return;
		System.out.println("Reusing Rengine.. Exporting from R... ");
		
		Vector<RVarAdapter> vars = rVarsTableModel.getRVars();
		for (RVarAdapter rv : vars) {
			//TODO  rv.fromR( rConsole.getREngine() );
		}
  }

  private void importRVars() {
	Vector<RVarAdapter> vars = rVarsTableModel.getRVars();
	for (RVarAdapter rv : vars) {
		rv.toR(rConsole.getREngine());
	}
  }

  
  private void doRstuff(Rengine re) {
		/* High-level API - do not use RNI methods unless there is no other way
			to accomplish what you want */
		try {
			REXP x;
			re.eval("data(iris)",false);
			System.out.println(x=re.eval("iris"));
			System.out.println(x=re.eval("a <- 2*3"));
			System.out.println(re.eval("a"));
			// generic vectors are RVector to accomodate names
			RVector v = x.asVector();
			if (v.getNames()!=null) {
				System.out.println("has names:");
				for (Enumeration e = v.getNames().elements() ; e.hasMoreElements() ;) {
					System.out.println(e.nextElement());
				}
			}
			// for compatibility with Rserve we allow casting of vectors to lists
			RList vl = x.asList();
			String[] k = vl.keys();
			if (k!=null) {
				System.out.println("and once again from the list:");
				int i=0; while (i<k.length) System.out.println(k[i++]);
			}			
			
			// now for a real dotted-pair list:
			System.out.println(x=re.eval("pairlist(a=1,b='foo',c=1:5)"));
			RList l = x.asList();
			if (l!=null) {
				int i=0;
				String [] a = l.keys();
				System.out.println("Keys:");
				while (i<a.length) System.out.println(a[i++]);
				System.out.println("Contents:");
				i=0;
				while (i<a.length) System.out.println(l.at(i++));
			}
			System.out.println(re.eval("sqrt(36)"));
		} catch (Exception e) {
			System.out.println("EX:"+e);
			e.printStackTrace();
		}
		
		// Part 2 - low-level API - for illustration purposes only!
		//System.exit(0);
		
	    // simple assignment like a<-"hello" (env=0 means use R_GlobalEnv)
	    long xp1 = re.rniPutString("hello");
	    re.rniAssign("a", xp1, 0);

	    // Example: how to create a named list or data.frame
	    double da[] = {1.2, 2.3, 4.5};
	    double db[] = {1.4, 2.6, 4.2};
	    long xp3 = re.rniPutDoubleArray(da);
	    long xp4 = re.rniPutDoubleArray(db);
	    
	    // now build a list (generic vector is how that's called in R)
	    long la[] = {xp3, xp4};
	    long xp5 = re.rniPutVector(la);

	    // now let's add names
	    String sa[] = {"a","b"};
	    long xp2 = re.rniPutStringArray(sa);
	    re.rniSetAttr(xp5, "names", xp2);

	    // ok, we have a proper list now
	    // we could use assign and then eval "b<-data.frame(b)", but for now let's build it by hand:       
	    String rn[] = {"1", "2", "3"};
	    long xp7 = re.rniPutStringArray(rn);
	    re.rniSetAttr(xp5, "row.names", xp7);
	    
	    long xp6 = re.rniPutString("data.frame");
	    re.rniSetAttr(xp5, "class", xp6);
	    
	    // assign the whole thing to the "b" variable
	    re.rniAssign("b", xp5, 0);
	    {
	        System.out.println("Parsing");
	        long e=re.rniParse("data(iris)", 1);
	        System.out.println("Result = "+e+", running eval");
	        long r=re.rniEval(e, 0);
	        System.out.println("Result = "+r+", building REXP");
	        REXP x=new REXP(re, r);
	        System.out.println("REXP result = "+x);
	    }
	    {
	        System.out.println("Parsing");
	        long e=re.rniParse("iris", 1);
	        System.out.println("Result = "+e+", running eval");
	        long r=re.rniEval(e, 0);
	        System.out.println("Result = "+r+", building REXP");
	        REXP x=new REXP(re, r);
	        System.out.println("REXP result = "+x);
	    }
	    {
	        System.out.println("Parsing");
	        long e=re.rniParse("names(iris)", 1);
	        System.out.println("Result = "+e+", running eval");
	        long r=re.rniEval(e, 0);
	        System.out.println("Result = "+r+", building REXP");
	        REXP x=new REXP(re, r);
	        System.out.println("REXP result = "+x);
	        String s[]=x.asStringArray();
	        if (s!=null) {
	            int i=0; while (i<s.length) { System.out.println("["+i+"] \""+s[i]+"\""); i++; }
	        }
	    }
	    {
	        System.out.println("Parsing");
	        long e=re.rniParse("rnorm(10)", 1);
	        System.out.println("Result = "+e+", running eval");
	        long r=re.rniEval(e, 0);
	        System.out.println("Result = "+r+", building REXP");
	        REXP x=new REXP(re, r);
	        System.out.println("REXP result = "+x);
	        double d[]=x.asDoubleArray();
	        if (d!=null) {
	            int i=0; while (i<d.length) { System.out.print(((i==0)?"":", ")+d[i]); i++; }
	            System.out.println("");
	        }
	        System.out.println("");
	    }
	    {
	        REXP x=re.eval("1:10");
	        System.out.println("REXP result = "+x);
	        int d[]=x.asIntArray();
	        if (d!=null) {
	            int i=0; while (i<d.length) { System.out.print(((i==0)?"":", ")+d[i]); i++; }
	            System.out.println("");
	        }
	    }

	    re.eval("print(1:10/3)");
	    
	if (false) {
	    // so far we used R as a computational slave without REPL
	    // now we start the loop, so the user can use the console
	    System.out.println("Now the console is yours ... have fun");
	    re.startMainLoop();
	} else {
	    re.end();
	    System.out.println("end");
	}    
	  
  }
  
  private static ListProperty[] getListPropertysFromObject(Object obj){
	  if ( obj instanceof SingleGrid )
	      obj = ((SingleGrid)obj).getProperty( SingleGrid.PROP_GRID );
	    if ( obj instanceof MultiGrid )
	      obj = ((MultiGrid)obj).getProperty( MultiGrid.PROP_GRIDS );

	    
	    	    	
    if ( obj instanceof ListProperty ) {
        ListPropertyReadAccess a = ((ListProperty)obj).getReadAccess(obj);
        ListProperty[] lists = new ListProperty[ a.getCount() ];
        for (int i=0; i<lists.length; i++)
          lists[i] = (ListProperty)a.getValue(i);
        a.release();
        return lists;
      }
      return null;
  }

    

}
