/** GnuR Plugin - A XULU-plugin for applying R-powered statistics on objects in the XULU-datapool.
    Copyright (C) 2007  Stefan A. Krüger

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/
package de.skrueger.xulu.plugin.gnur;

import java.awt.Cursor;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.IOException;
import java.io.PipedInputStream;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

import org.apache.log4j.Logger;
import org.rosuda.JRI.RMainLoopCallbacks;
import org.rosuda.JRI.Rengine;

public class RConsolePanel extends JPanel implements RMainLoopCallbacks {
    Logger log; // TODO .. some smarter logging that System.out

    /** There may only be one REngine for the whole XULU Framework.*/
    private static Rengine re = null;

    boolean readInput = false;		// Is true when the InputField is ready to be sent to R
    boolean readInput2 = false;		// Is true when the internal InputField is ready to be sent to R

    // Output from R
    public JTextArea rOut = new JRTextArea();
    private JScrollPane cmdOutSp;

    // Inputf to R
    public JTextArea rCmdIn = new JTextArea(4,80);
    private JScrollPane cmdInSp;

    // Keeps a list of the last commands entered in R.
    Vector<String> history    = new Vector<String>();
    int    historyPos = 0;

    protected int maxLines = 200;

    /* Cursor when waiting for R */
    Cursor hourglassCursor = new Cursor(Cursor.WAIT_CURSOR);
    Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);

    public RConsolePanel() throws IOException {
        rOut.setEditable(false);
        rOut.setRows(200);
        rOut.setBorder( BorderFactory.createEtchedBorder(EtchedBorder.LOWERED) );

        rCmdIn.addKeyListener(keyL);
        rCmdIn.setBorder( BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));

        setLayout(new GridBagLayout());
        cmdOutSp = new JScrollPane( rOut );
	add(  cmdOutSp ,
        	new GridBagConstraints(
        		0, 0, 2, 1, 1, 1,
        		GridBagConstraints.NORTHWEST,
        		GridBagConstraints.BOTH,
        		new Insets(5, 0, 2, 0),
        		0, 0) );
        cmdInSp = new JScrollPane( rCmdIn );
        	add(  cmdInSp, new GridBagConstraints(
        		0, 1, 2, 1, 1, 1,
        		GridBagConstraints.NORTHWEST,
        		GridBagConstraints.BOTH,
        		new Insets(2, 0, 5, 0),
        		0, 0) );
    }

    public void init() {
	if (re == null) {
	    System.out.println("Creating Rengine (with arguments)");
            String[] args = new String[1];
            args[0] = "--no-save";
            // 1) we pass the arguments from the command line
            // 2) we won't use the main loop at first, we'll start it later
            // (that's the "false" as second argument)
            // 3) the callbacks are implemented by the TextConsole class above
            re = new Rengine(args, false, this);
            System.out.println("Rengine created, waiting for R");
            // the engine creates R is a new thread, so we should wait until
            // it's ready
            if (!re.waitForR()) {
        	System.out.println("Cannot load R");
    		return;
    	    }
    	    re.startMainLoop();
    	};
    }

    final KeyListener keyL = new KeyAdapter() {
        public void keyPressed(KeyEvent ev) {

            if(ev.getKeyCode() == KeyEvent.VK_UP + KeyEvent.CTRL_DOWN_MASK) {
                if(historyPos > 0) {
                  String line = history.elementAt(historyPos-1);
                  historyPos--;
                  rCmdIn.setText(line);
                }
              }
            else if(ev.getKeyCode() == KeyEvent.VK_DOWN + KeyEvent.CTRL_DOWN_MASK) {
                if(historyPos < history.size()-1) {
                  String line = (String)history.elementAt(historyPos+1);
                  historyPos++;
                  rCmdIn.setText(line);
                }
              }
            else
        	// No Up- or Down-key pressed.
            {
            String line = rCmdIn.getText();
            if (    (ev.getModifiers() == KeyEvent.CTRL_MASK)
        	 && ( ev.getKeyCode() == KeyEvent.VK_ENTER)   ) {

        	// If something worth saving was typed...
        	if(!("".equals(line) ||
                     "\n".equals(line) ||
                     "\n\r".equals(line) ||
                     "\r\n".equals(line))) {
                    history.add( line.trim() );
                }

                if("clear".equals(line)) {
                    // Wipes the R-Output window.
                    rOut.setText("");
                }
                else
                {
                    showLastLine();
                    readInput = true;
                }
            }
            }
        }
    };

    private String throwInCmd;


    /**
     * Scrolls down the ScrollWindow of the ROutput.
     */
    public void showLastLine() {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            if (maxLines > 0) {
              int linesToRemove = rOut.getLineCount() - maxLines;
              while (linesToRemove < 0 ){
        	 rOut.setText( "\n" + rOut.getText());
        	 linesToRemove++;
              }
              if (linesToRemove > 0) {
        	    int index = 0;
                    for (int i=0; i<linesToRemove; i++) {
                      index = rOut.getText().indexOf('\n', index) + 1;
                    }
                    rOut.setText(rOut.getText().substring(index));
              } else {

              }
            }
          }
        });
      }

    class ReaderThread extends Thread {
        PipedInputStream pi;
        ReaderThread(PipedInputStream pi) {
            this.pi = pi;
        }

        public void run() {
            final byte[] buf = new byte[1024];
            try {
               while (true) {
                    final int len = pi.read(buf);
                    if (len == -1) {
                        break;
                    }
                    SwingUtilities.invokeLater(new Runnable() {
                        public void run() {
                            rOut.append(new String(buf, 0, len));

                            // Make sure the last line is always visible
                            rOut.setCaretPosition(rOut.getDocument().getLength());

                            // Keep the text area down to a certain character size
                            int idealSize = 1000;
                            int maxExcess = 500;
                            int excess = rOut.getDocument().getLength() - idealSize;
                            if (excess >= maxExcess) {
                                rOut.replaceRange("", 0, excess);
                            }
                        }
                    });
                }
            } catch (IOException e) {
            }
        }
    }

    /**
     * called by the R-Engine, when it want's to print to STDOUT
     */
//    @Override
    public void rWriteConsole(Rengine re, String text) {
    	rOut.append(text);
    	showLastLine();
    }

//    @Override
    public void rBusy(Rengine re, int which) {
    	if (which == 1) {
    		System.out.println("busy");
    		rCmdIn.setEnabled(false);
    		setCursor(hourglassCursor);
    	} else {
    		rCmdIn.setEnabled(true);
    		System.out.println("not busy");
    		setCursor(defaultCursor);
    		rCmdIn.requestFocus();
    	}
    }

//    @Override
    public String rReadConsole(Rengine re, String prompt, int addToHistory) {
    	rCmdIn.setEnabled(true);
    	while ((!readInput) && (!readInput2) ) {
    		try {
    			Thread.sleep(100);
    		} catch (InterruptedException e) {
    			e.printStackTrace();
    		}
    	}

    	String  text = "";
    	// Input comes from the GNU-R Plugin GUI
    	if (readInput) {
    	    readInput=false;
    	    text = rCmdIn.getText()+"\n";
    	    rCmdIn.setText("");
    	}
    	else if (readInput2) {
    	    // Sent to R-Console by cmd : public void sendToConsole(String s)
    	    readInput2 = false;
    	    text = throwInCmd+"\n";
    	    throwInCmd = "";
    	}

    	System.out.print("Sending as Console Input = "+text);
    	return text;
    }

    /**
     * Easy way to send a command to R
     * The result is shown on the R Console
     * @param s
     */
    public void sendToConsole(String s) {
	throwInCmd = s;
	readInput2 = true;
    }

//    @Override
    public void rShowMessage(Rengine re, String message) {
    	rOut.append("rShowMessage \"" + message + "\"");
    }

//    @Override
    public String rChooseFile(Rengine re, int newFile) {
    	FileDialog fd = new FileDialog(new Frame(),
    			(newFile == 0) ? "Select a file" : "Select a new file",
    			(newFile == 0) ? FileDialog.LOAD : FileDialog.SAVE);
    	fd.setVisible(true);
    	String res = null;
    	if (fd.getDirectory() != null)
    		res = fd.getDirectory();
    	if (fd.getFile() != null)
    		res = (res == null) ? fd.getFile() : (res + fd.getFile());
    	return res;
    }

//    @Override
    public void rFlushConsole(Rengine re) {
    }

//    @Override
    public void rLoadHistory(Rengine re, String filename) {
    }

    public void rSaveHistory(Rengine re, String filename) {
    	System.out.println("Maybe I want to export the stuff here!?");
    }

    /**
     * Returns the static R-Engine
     */
    public Rengine getREngine() {
        return re;
    }
}
