package de.skrueger.xulu.plugin.gnur;

import org.rosuda.JRI.Rengine;

import schmitzm.data.property.MatrixProperty;
import schmitzm.data.property.Property;
import schmitzm.data.property.ScalarProperty;

/**
 * Link between datapool and R-Engine
 * @author stefan
 */

public abstract class RVarAdapter {
	/**
	 * Soll diese Datenstruktur in die R Engine importiert und-/oder aus der
	 * Engine heraus exportiert werden?
	 */
	private Boolean exp = true;
	private Boolean imp = true;

	protected Property prop = null;
	private String RVarName = null;

	/**
	 *
	 * @param obj
	 * @param testOnly	if true, than only check if Property is compatible-.. otherwise throw PropertytypeNotConvertableToRVarException
	 * @throws PropertytypeNotConvertableToRVar
	 */
	static public RVarAdapter createFrom ( Property obj, boolean testOnly) throws PropertytypeNotConvertableToRVar {
		if (obj instanceof ScalarProperty) {
		    System.out.print("Scalprop");
		    if (!testOnly) return new RVarScalar( (ScalarProperty)obj );
		} else if (obj instanceof MatrixProperty) {
		    System.out.print("MatrixProp");
		    if (!testOnly) return new RVarMatrix( (MatrixProperty)obj );
		} else {
			throw new PropertytypeNotConvertableToRVar( obj );
		}
		return null;
	}

	public Boolean isExport() {
		return exp;
	}

	public Boolean isImport() {
		return imp;
	}


	/**
	 * @return the name of the variabl ein the R environment
	 */
	public String getRName() {
		return RVarName;
	}


	/**
	 * Set the name that will reference the R-variable inside the RVar.
	 * TODO: Check for doubles, name validity
	 * @param rname
	 */
	public void setRName(String rname) {
		RVarName = rname.replace(" ", "_");
	}

	/**
	 * @return the name of the object in the datapool
	 */
	public String getName() {
		return prop.getName();
	}

	public abstract void toR(Rengine re);

	public abstract void fromR(Rengine re);

	protected RVarAdapter(Property prop) {
	    super();
	    this.prop = prop;

	    // Change name of var so that its R-sytax-proof.
	    RVarName = prop.getName().replaceAll(" ", "_");
	}
}
