package de.skrueger.xulu.plugin.gnur;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.Rengine;

import schmitzm.data.property.MatrixProperty;

public class RVarMatrix extends RVarAdapter {
    
    MatrixProperty prop = (MatrixProperty) super.prop;
    private Class cType;		// Class/type of the MatrixProperties' values
    int[] size;				// Getting the size of the matrix in X and Y direction
    
    protected RVarMatrix(MatrixProperty prop) {
	super(prop);
	cType = prop.getType();
	size =  prop.getSize();;
	System.out.println("MatrixProp created...");
    }

    @Override
    /**
     * Export a matrix from the R environment into a MatrixProperty
     * @author Stefan A. Krüger
     */    
    public void fromR(Rengine re) {
	if (!isImport()) return;
	
	// So far we can only handle two-dimensional matrixes.
	int dims = prop.getDimension(); 
	if (dims != 2) {
	    throw new PropertytypeNotConvertableToRVar(prop,"Sorry. So far the GNU-R plugin can only handle two-dimensional matrixes.");
	}

	
    	REXP ex = re.eval( getRName() );
    	if ((cType == Integer.class) || (cType == Short.class) || (cType == Float.class) ) {
    	    double ee[][] = ex.asMatrix();
    	    if (ee==null) 
    		throw new PropertytypeNotConvertableToRVar(prop, cType.toString() + " - error was not expected."); 
    	    	System.out.println("len von ee[] = "+ee.length);
    		System.out.println("ee[1] = "+ee[1]);
    		
    	    	// Filling the PropertyMatrix with the double-matrix from R
    	    	for (int x = 0; x < size[0]; x++)  {
    	    	    for (int y = 0; y < size[1]; y++)  {
    	    		Double d = Double.valueOf(ee[x][y]);
    	    		if ( cType == Float.class ) prop.getOneTimeWriteAccess().setValue( d.floatValue(), x, y );
    	    		if ( cType == Double.class ) prop.getOneTimeWriteAccess().setValue( d, x, y );
    	    		if ( cType == Integer.class ) prop.getOneTimeWriteAccess().setValue( Integer.valueOf( d.intValue()), x, y );
    	    		if ( cType == Short.class ) prop.getOneTimeWriteAccess().setValue( Integer.valueOf( d.shortValue()), x, y );
    	    	    }
    	    	}
    	    	return;
    	} else
        if (cType == String.class) {
            throw new PropertytypeNotConvertableToRVar(prop, cType.toString() + " - cant import that type from R. Ask the programmer kindly to implement it.");
        } else {
            throw new PropertytypeNotConvertableToRVar(prop, cType.toString() + " - cant import that type from R. Sorry.");
        }
    	
//    	// Filling the PropertyMatrix fomr a 1-dim object-array
//    	for (int x = 0; x < size[0]; x++)  {
//    	    for (int y = 0; y < size[1]; y++)  {
//    		// System.out.println( prop.getOneTimeReadAccess().getValue( x,y ) );
//    		int idx = y + (x*size[1]);
//    		prop.getOneTimeWriteAccess().setValue( longarray[idx], x, y );
//    	    }
//    	}
    	
    }


    
    
    
    @Override
    /**
     * Import a MatrixProperty into the R environment
     */
    public void toR(Rengine re) {
	if (!isExport()) return;
	
	// So far we can only handle two-dimensional matrixes.
	int dims = prop.getDimension(); 
	if (dims != 2) {
	    throw new PropertytypeNotConvertableToRVar(prop,"Sorry. So far the GNU-R plugin can only handle two-dimensional matrixes.");
	}
	
    	// Filling a 1-dim object-array
    	Object longarray[] = new Object [ size[0] * size[1] ];
    	for (int x = 0; x < size[0]; x++)  {
    	    for (int y = 0; y < size[1]; y++)  {
    		// System.out.println( prop.getOneTimeReadAccess().getValue( x,y ) );
    		int idx = y + (x*size[1]);
    		longarray[idx] = prop.getOneTimeReadAccess().getValueAsInt( x,y );
    	    }
    	}
    	// System.out.println ("Length of the longarray = " + longarray.length );
    	
    	// LongArray referenze in R 
        long xpLA = -1;
        
    	// Check which types we can import into R:
        
    	if ((cType == Integer.class) || (cType == Short.class)) {
    	    // Converting object-array to int[]
    	    int[] aa = new int[longarray.length];
    	    for (int i = 0; i<longarray.length-1; i++) {
    		aa[i] = (Integer) longarray[i];
    	    }
    	    // importing to R
    	    xpLA = re.rniPutIntArray(  aa );
    	} else 
        if (cType == Float.class) {
    	    double[] aa = new double[longarray.length];
    	    for (int i = 0; i<longarray.length-1; i++) {
    		aa[i] = (Double) longarray[i];
    	    }
    	    // importing to R
    	    xpLA = re.rniPutDoubleArray( aa );
        } else
        if (cType == String.class) {
            String[] aa = new String[longarray.length];
    	    for (int i = 0; i<longarray.length-1; i++) {
    		aa[i] = (String) longarray[i];
    	    }
    	    // importing to R
	    xpLA = re.rniPutStringArray( aa );
        } else {
            throw new PropertytypeNotConvertableToRVar(prop, cType.toString() + " - cant convert that type to R. Sorry.");
        }
    		

    	// Convert xpLA to a R-matrix
    	re.rniAssign( getRName(), xpLA, 0);
    	re.eval( getRName()+"<-matrix ( "+getRName()+","+size[0]+" ) ");
    }
}
