package de.skrueger.xulu.plugin.gnur;

import java.util.Vector;

import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.property.ScalarProperty;
import schmitzm.geotools.gui.GeotoolsGUIUtil;
import schmitzm.swing.SelectionInputOption;
import schmitzm.swing.table.AbstractMutableTableModel;
import schmitzm.swing.table.MutableTableModel;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.DataPool;

///////////////////////////////////////////////////////////////////
//////////////   TableModel for the DnD-Tabel der R Variables  //////////////
///////////////////////////////////////////////////////////////////

class RVarsTableModel extends AbstractMutableTableModel {
  /** Speichert die in der Tabelle gespeicherten Raster */
	protected Vector<RVarAdapter> objects = new Vector<RVarAdapter>();
  
  /** Speichert die Instanz der Xulu-Applikation. */
  protected XuluModellingPlatform appl = null;
  /** Eingabe-Option fuer ein einzelnes Objekt. */
  protected SelectionInputOption.Combo objectInputOption = null;

  /** Flag, ob sich der Inhalt des Datenpools geaendert hat und die Arrays
   *  {@link #fitObjects} und {@link #fitObjectNames} aktualisiert werden
   *  muessen */
  protected boolean  dataPoolChanged = true;
  /** Passende Objekte im Datenpool */
  protected Object[] fitObjects = null;
  /** Bezeichnungen der passenden Objekte im Datenpool */
  protected String[] fitObjectNames = null;

  /**
   * Erzeugt ein neues Tabellen-Modell.
   * @param appl Instanz der Xulu-Applikation
   * @param rowIndexPrefix Praefix, das den Zeilen-Indizes vorangestellt wird
   */
  public RVarsTableModel(XuluModellingPlatform appl) {
    super();
    this.appl = appl;
    objectInputOption = new SelectionInputOption.Combo(getColumnName(1),true);
    refreshInputOption();
    // Listener auf den Datenpool setzen, damit das Auswahlfeld aktualisiert
    // werden kann (wenn notwendig)
    appl.getDataPool().addObjectListener( new ObjectListener() {
      public void performObjectEvent(ObjectEvent e) {
        if ( e instanceof DataPool.DataPoolChangeEvent ) {
          dataPoolChanged = true;
        }
      }
    });
  }

  /**
   * Liefert die Spaltennamen der Tabelle.
   */
  @Override
  public String[] createColumnNames() {
    return new String[] {null,"Datapool-Object","Names in R","Import","Export"};
  }

  /**
   * Ruft {@link #determineFitObjectsAndNames()}, wenn das {@link #dataPoolChanged}-Flag
   * auf {@code true} steht und setzt das Flag zurueck.
   */
  protected void refreshFitObjects() {
    if ( dataPoolChanged )
      determineFitObjectsAndNames();
    dataPoolChanged = false;
  }

  /**
   * Aktualisiert das Dialog-Auswahlfeld, in dam alle passenden Objekte
   * aus dem Datenpool vorgeblendet werden.
   */
  protected void refreshInputOption() {
    refreshFitObjects();
    objectInputOption.setSelectionObjects(fitObjects,fitObjectNames);
  }

  /**
   * Wird aufgerufen, wenn sich der Inhalt des Xulu-Datenpools aendert.
   * Muss die Arrays {@link #fitObjects} und {@link #fitObjectNames} mit
   * allen passenden Objekten des Datenpools und deren Bezeichnungen
   * befuellen.
   */
  protected void determineFitObjectsAndNames() {
}


  /**
   * Loescht eine Tabellenzeile.
   * @param row Zeilenindex (beginnend bei 0)
   */
  public void performRemoveRow(int row) {
    objects.remove(row);
  }

  /**
   * Macht nichts, da in der Tabelle keine Daten veraendert werden koennen.
   * @param row Zeilenindex (beginnend bei 0)
   * @param col Spaltenindex (beginnend bei 0)
   */
  public void performChangeData(int row, int col) {
  }

  /**
   * Liefert die Anzahl an Zeilen.
   */
  public int getRowCount() {
    return objects.size();
  }

  /**
   * Liefert den Inhalt einer Tabellenzelle.
   * @param row Zeilenindex (beginnend bei 0)
   * @param col Spaltenindex (beginnend bei 0)
   */
  public Object getValueAt(int row, int col) {
    switch ( col ) {
      case 0: return row;
      case 1: return objects.elementAt(row).getRName(); // TODO 
      case 2: return objects.elementAt(row).getRName();
      case 3: return objects.elementAt(row).isImport();
      case 4: return objects.elementAt(row).isExport();
    }
    return null;
  }


public void performAddRow() {
	System.out.println("?!");
}


public void addRVariable(RVarAdapter rv) {
	objects.add(rv);
	refreshFitObjects();
}

/**
 * Returns the 
 * @return
 */
public Vector<RVarAdapter> getRVars() {
	return objects;
}

}
