#########################################################################
################# function definitions  #################################
#########################################################################

### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###                        File operations                                 
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
# --------------------------------------------------------------
### sort filelist
### args:
### 1. List of input files
### 2. seperator string. where to split the numbers from mai file name

sort.filelist <- function (FileList, s="_") {
  f=strsplit(FileList, s)
  v=numeric()
  # extract number element out of list, which is simulation year
  for (i in 1:length(f)){v=c(v,f[[i]][2])}
  v=as.numeric(v)
  # sort years with index values
  vs=sort(v, index.return=T)
  # apply sortet year list to Filelist
  FileList.sort = FileList[vs[[2]]]
  return(FileList.sort)
}

### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###                        Read ASCII GRID data                            
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

### ---------------------------------------------------------------------
### read several ascii grid into list                                   
### ---------------------------------------------------------------------

read.grd <- function (FileList, n=length(FileList), path="") {
  #test for right / in path string
  if (path!="" & length(grep("/$",path))>0) path=path else {path=paste(path,"/",sep="")} 
  
  grid.lst = list()
  for (i in 1:n) {
    grid.lst[[i]] <- read.asciigrid(paste(path,FileList[i],sep=""), as.image=T )
    
  }
  names(grid.lst) <- FileList
  return(grid.lst)
}


### ---------------------------------------------------------------------
### read several ascii grid into one dataframe ONLY DATA!                 
### ---------------------------------------------------------------------

read.asciigrid.df <- function(GridList, nPixels) {
  for (i in 1:length(GridList)) {
    InData = read.asciigrid(GridList[i], as.image=T )
    InData = as.numeric(InData$z)
    if (length(na.omit(InData)) != nPixels ) {
      print("amount of pixel values doesn't match!!!!!")
      print(GridList[i])
      }
    if (i==1) {
      ascii.df = as.data.frame(InData)
      }
    else {  
    ascii.df = cbind(ascii.df,InData) 
      }
    }
}

### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###                        Sampling functions                            
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ---------------------------------------------------------------------
# balanced sampling of lc pixels
# ---------------------------------------------------------------------


balanced.sampling <- function(dataframe, lc.type, max_samp) {

  dataframe$lu <- ifelse (dataframe[[1]] == lc.type, 1, 0)

  dataframe.S1 = subset(dataframe, lu==1)
  
  if (sum(dataframe.S1$lu, na.rm = T) > 2000) {
    RandomList = runif(max_samp,min=1, max=length(dataframe.S1$lu))
    dataframe.S1 = dataframe.S1[RandomList,]
  }
  
  # set maximum sampling amount to number of cases if <2000
  if (sum(dataframe$lu, na.rm = T) < 2000) {
    print (c("LU type", lc.type, "has", sum(dataframe$lu, na.rm = T), "occurances, do balanced sampling"))
    max_samp = sum(dataframe$lu, na.rm = T)
    }
    
  dataframe.S0 = subset(dataframe, lu==0)
   RandomList = runif(max_samp,min=1, max=length(dataframe.S0$lu))
  dataframe.S0 = dataframe.S0[RandomList,]

  dataframe.S = rbind(dataframe.S1,dataframe.S0)
  colnames(dataframe.S) <- colnames(dataframe)
  return(dataframe.S)
  }

### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###                 write results from logit model to file                 
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ---------------------------------------------------------------------
# write AUV values to file
# ---------------------------------------------------------------------

WriteAUC <- function (fitstep, BeninData, i) {
  pred = prediction(fitted(fitstep),as.vector(BeninData$lu))
  # ---------------------------------------------------------------------
  # compute AUC
  # ---------------------------------------------------------------------
  auc = performance(pred, measure="auc")
  # auc is object of class performance, data stored in slots
  # to access slots (and datas) use @ (in Newsletter 4/1 is said
  # don't use it but I don't know an other way
  aucValue = auc@y.values[[1]]
  write (paste("AUC for LC no",i, ":", aucValue, sep=" "), outputRegEval, append=TRUE)
}


# ---------------------------------------------------------------------
# write betas in a CLUE-S compatible file
# ---------------------------------------------------------------------

WriteCoefClue <- function(fitstep, outRegressionFile, i, description="new run") {
  coef <- summary(fitstep)$coefficients
  betas <- as.vector(coef[,1])
  # insert number of dforces
  betas <- append(betas, length(betas)-1 , after=1)
  # insert number of landcover class
  betas <- append(betas, i, after=0)
  # insert second collumn for dforce id
  betas <- cbind(betas, "") [, c(1,2)]
  # match dforce id of coef list with value list above
  res_dforces_names <- as.data.frame(dimnames(coef)[[1]])
  res_dforces <- merge(dforce, res_dforces_names, by=1, sort=FALSE)
  # insert dforce id into second collumn
  z <- length(dimnames(coef)[[1]]) -1
  for (j in 1:z) {
  	betas[j+3,2] <- res_dforces$dforce_number[j]
  }
  if (i == 0) {
    write("\n----------------------------------",outRegressionFile,append=TRUE) 
    write(description,outRegressionFile,append=TRUE) 
  }
  write(t(betas), outRegressionFile, ncolumns=2, append=TRUE)
}


# ---------------------------------------------------------------------
# write betas in a XULU compatible file
# ---------------------------------------------------------------------

WriteCoefXulu <- function(fitstep, outeval, i , description="new run") {
  coef <- summary(fitstep)$coefficients
  betas <- as.vector(coef[,1])

  coef.val <- numeric(length=length(dforce_number)+1 )

  # assign intercept
  coef.val[1] = betas[1]
  # assign coef
  # get position (index) of df name
  df.pos = match(dimnames(coef)[[1]][-1], dforce_name)
  # assign the coef at appropriate postition in vector
  coef.val[df.pos+1] = betas[-1]
  # insert semicolon
  coef.val = paste(coef.val, ";", sep="")
  if (i==0) {
     write(paste("--------------- (", description, ")", sep=""),outeval,append=TRUE) 
     #write(description,outeval,append=TRUE) 
  }
  write(paste("\n ----- LC ",i, " ------", sep=""),outeval,append=TRUE) 
  write(coef.val, outeval, ncolumns=length(dforce_number)+1, append=TRUE)
}


### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###                    Plot results from logit model                       
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
### ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


# ---------------------------------------------------------------------
# plot fitted values from logistic regression model
# ---------------------------------------------------------------------

plot.fitted <- function(FileList, fitted.data, dataset, numrows=580, numcols=539) {

  InData = read.asciigrid(FileList[1], as.image=T )
  # convert data matrix to numeric
  v=as.numeric(InData$z)
  ## get index of data values
  x=which(v>=0)
  # calculate probabilities for whole data set (if subset used)
  prd=predict(fitted.data, newdata=dataset, type="response")
  prd=na.omit(prd)
  ## assign new values to x position
  v[x]=prd
  # transform it back to matrix
  v=matrix(v,  nrow=numrows, ncol=numcols)
  # get matrix back to image
  predict.grid = InData
  predict.grid$z = v
  image(predict.grid, col=terrain.colors(24))
  legend.entries = seq(0,1, by=1/10,digits=1)
  legend("bottomright",legend=legend.entries, fill=terrain.colors(11))

}

### color ramp
### -> see colorRampPalette function and RColorBrewer

#image(grid.prob[[2]], col=rainbow(24, start=0, end=2/6))
#
#colorRamp.redblue <- function (n) 
#{
#    x <- ramp(seq(0, 1, length = n))
#    rgb(x[, 1], x[, 2], x[, 3], max = 255)
#}
#
#