/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.appl;

import java.util.Vector;

import edu.bonn.xulu.appl.VisualisationTool;
import edu.bonn.xulu.appl.AbstractManager;

// nur fuer Doku
import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.AbstractObjectTraceable;

/**
 * Der VisualisationManager verwaltet saemtliche in der Xulu-Applikation
 * <u>instanziierten</u> Visualisierungstools. An dieser Stelle ist der
 * Unterschied zu den in der Xulu-Registry registierten Tools zu bemerken.
 * Waehrend in der Registry nur die Klassen der Visualisierungstools
 * hinterlegt werden, verwaltet der VisualisationManager deren Instanzen
 * (also die einzelenen Fenster).<br>
 * Der VisualisationManager fungiert als <code>AbstractObjectTraceable</code>,
 * so dass {@link ObjectListener} angeschlossen werden
 * koennen, welche automatisch durch ein {@link AbstractManager.ChangeEvent}
 * informiert werden, wenn sich der Inhalt des VisualisationManagers aendert.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class VisualisationManager extends AbstractManager {

  /**
   * Erzeugt einen neuen (leeren) Visualisierungsmanager.
   */
  public VisualisationManager() {
    this.content = new Vector<VisualisationTool>();
  }

  /**
   * Prueft, ob eine Komponente im Visualisierungsmanager verwaltet werden
   * kann.
   * @param comp Komponente
   * @exception UnsupportedOperationException falls es sich bei der Komponente
   *            nicht um eine Instanz von {@link VisualisationTool} handelt
   */
  public void checkComponentAndError(ContentType comp) {
    if ( !(comp instanceof VisualisationTool) )
      throw new UnsupportedOperationException(getClass().getSimpleName().concat(" can not manage instances of ").concat(comp.getClass().getName()));
  }

  /**
   * Liefert alle registrierte Visualisierungstool-Instanzen als Array.
   * @return leerer <code>VisualisationTool[]</code>-Array falls keine Instanzen
   *         im Manager registriert sind.
   */
  public VisualisationTool[] getAll() {
    return (VisualisationTool[])content.toArray(new VisualisationTool[0]);
  }
}
