/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.appl;

import java.io.File;
import javax.swing.ImageIcon;
import java.util.Locale;

import schmitzm.lang.ResourceProvider;
import schmitzm.swing.SwingUtil;
import edu.bonn.xulu.XuluModellingPlatform;

// nur fuer Doku

/**
 * Dieses Interface enthaelt Kostanten fuer die Xulu-Modelling-Platform.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface XuluConstants {
  /** Versionsnummer der Xulu-Modelling-Platform. **/
  public static final String VERSION = "1.8.1";
  /** Versionsdatum der Xulu-Modelling-Platform. **/
  public static final String VERSION_DATE = "2010-02-10";
  /** Verzeichnis, in dem Icons und Uebersetzungen hinterlegt sind. */
  public static final String RESOURCE_DIR = "resource";
  /** Verzeichnis, in dem Icons hinterlegt sind. */
  public static final String ICONS_DIR = RESOURCE_DIR+File.separatorChar+"icons";
  /** Icon, das fuer Xulu-Fenster verwendet wird. */
  public static final ImageIcon XULU_ICON      = new ImageIcon(ICONS_DIR+File.separatorChar+"xulu_icon.png");
  /** Xulu-Logo, das auf dem Start-Fenster angezeigt wird. */
  public static final ImageIcon XULU_INFOICON  = new ImageIcon(ICONS_DIR+File.separatorChar+"xulu_info.png");
  /** Xulu-Logo, das auf dem Info-Fenster angezeigt wird. */
  public static final ImageIcon XULU_STARTICON = new ImageIcon(ICONS_DIR+File.separatorChar+"xulu_start.png");

  /** Enthaelt eine Referenz auf die Standard-Swing-Ressourcen des Package
   *  {@code schmitzm.swing}.
   *  @see SwingUtil#RESOURCE */
  public static final ResourceProvider SWING_RES = SwingUtil.RESOURCE;

  /** Enthaelt den Basis-Pfad (in Punkt-Notation, basierend auf dem Classpath)
   *  der Xulu-spezifischen Ressourcen.<br>
   *  Konstant: {@code locales} (Verzeichnis unterhalb von {@code <xulu_main>/resource)}*/
  public static final String LOCALES_BASE = "locales";

  /** Pfad (in Punkt-Notation), auf den die Bundles aus Standard-Paketen umgelenkt
   *  werden. Konstant: {@code locales.StandardBundleExtension}
   *  @see #LOCALES_BASE */
  public static final String EXTENTION_BUNDLE = XuluConstants.LOCALES_BASE+".StandardBundleExtension";

  /** Enthaelt eine Referenz auf die Xulu-spezifischen Ressourcen fuer
   *  die Haupt-Applikation. */
  public static final ResourceProvider XULUGUI_RES = XuluModellingPlatform.registerResourceProvider("XuluMainAppl",Locale.ENGLISH);

  /** Enthaelt eine Referenz auf die Xulu-spezifischen Ressourcen fuer
   *  die Fehlermeldungen in der Haupt-Applikation. */
  public static final ResourceProvider XULUERR_RES = XuluModellingPlatform.registerResourceProvider("XuluMainError",Locale.ENGLISH);

  /** Enthaelt eine Referenz auf die Xulu-spezifischen Ressourcen fuer
   *  die Visualisierungs-Plugins. */
  public static final ResourceProvider XULU_VIS_RES = XuluModellingPlatform.registerResourceProvider("XuluVisualisationTool",Locale.ENGLISH);
}
