/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.appl;

import java.io.InputStream;
import java.io.IOException;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluRegistry;

/**
 * Dieses Interface bildet die Basis fuer eine Klasse, die die XuluRegistry
 * einlesen kann.
 * @see edu.bonn.xulu.appl.XuluRegistry
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface XuluRegistryReader {
  /**
   * Erweitert eine bestehende XuluRegistry. Bestehende Eintraege werden
   * ueberschrieben. Ist die uebergebene Registy <code>null</code> wird
   * eine neue (leere) Instanz erstellt.
   * @param appl     Instanz der Xulu-Applikation fuer die die Registry eingelesen
   *                 werden soll
   * @param input    Eingabe-Stream aus dem die Registry-Daten gelesen werden
   * @param registry XuluRegistry die erweitert werden soll (kan <code>null</code> sein
   * @return Instanz der XuluRegistry, die erstellt oder erweitert wurde
   * @throws java.io.IOException falls ein Fehler beim Einlesen auftritt
   */
  public XuluRegistry readRegistry(XuluModellingPlatform appl, InputStream input, XuluRegistry registry) throws IOException;

  /**
   * Erstellt eine neue XuluRegistry und befuellt diese.
   * @param appl     Instanz der Xulu-Applikation fuer die die Registry eingelesen
   *                 werden soll
   * @param input    Eingabe-Stream aus dem die Registry-Daten gelesen werden
   * @return Instanz der XuluRegistry, die erstellt wurde
   * @throws java.io.IOException falls ein Fehler beim Einlesen auftritt
   */
  public XuluRegistry readRegistry(XuluModellingPlatform appl, InputStream input) throws IOException;

}
