/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.data;

import schmitzm.data.property.Properties;
import schmitzm.data.property.Property;
import schmitzm.data.property.PropertySet;
import schmitzm.data.event.ObjectTraceable;
import schmitzm.lang.NamedObject;

import edu.bonn.xulu.appl.XuluComponent;

/**
 * Diese Interface bildet die Oberklasse fuer alle vom Xulu-Datenpool
 * verwaltbare Datentypen. Es dient in der Hauptsache dazu, alle notwendigen
 * Interfaces zusammenzufassen.<br>
 * Darueber hinaus spezifiziert es einige weitere Funktionen, die fuer die
 * Verwaltung des Datenpools wichtig sind.<br>
 * Eine Basis-Implementierung stellt die Klasse {@link AbstractXuluObject} zur
 * Verfuegung. Hiervon sollten neue Xulu-Objekte abgeleitet werden.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public interface XuluObject extends Properties, ObjectTraceable, XuluComponent, NamedObject {
  /** Name der Eigenschaft, die den Objekt-Style beinhaltet. Es ist nicht
   *  gewaehrleistet, dass das {@link XuluObject} tatsaechlich eine Eigenschaft
   *  mit diesem Namen enthaelt. Doch wenn es eine solche bereitstellt, sollte
   *  sie unter folgendem Einheitlichen Namen angesprochen werden koennen! */
  public static final String PROP_STYLE = "Style";

  /**
   * Liefert eine numerische ID, ueber die das Objekt eindeutig im Datenpool
   * identifiziert wird.
   */
  public long getID();

  /**
   * Liefert eine Beschreibung (Namen) des Objekts. Ueber diese wird das Objekt
   * dem Anwender gegenueber im Datenpool identifiziert.
   */
  public String getDescription();

  /**
   * Setzt die Beschreibung (Namen) des Objekts. Ueber diese wird das Objekt
   * dem Anwender gegenueber im Datenpool identifiziert.
   */
  public void setDescription(String desc);

  /**
   * Zerstoert das Objekt und gibt alle internen Ressourcen (z.B. Zugriffsrechte)
   * wieder frei.
   */
  public void dispose();

  /**
   * Prueft, ob das Objekt "zerstoert" ist.
   * @see #dispose()
   */
  public boolean isDisposed();

  /**
   * Prueft, ob zur Zeit Zugriffsrechte auf das Objekt (oder seine Properties)
   * verteilt sind.
   * @return boolean
   */
  public boolean isAccessed();

}
