/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import schmitzm.swing.BrowseInputOption;
import schmitzm.swing.TreeSelectionDialog;

import schmitzm.data.property.Property;

import edu.bonn.xulu.appl.DataPool;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.data.XuluObject;

// nur fuer Doku
import schmitzm.swing.MultipleOptionPane;

/**
 * Diese Klasse stellt eine Eingabe-Option fuer einen {@link MultipleOptionPane}-Dialog
 * dar, in der ein Daten-Objekt aus dem {@linkplain DataPool Xulu-Datenpool}
 * eingegeben werden muss. Hierzu besteht fuer den Anwender die Moeglichkeit,
 * ueber einen Browse-Dialog aus allen vorhandenen Datenpool-Objekten auszuwaehlen.<br>
 * <br>
 * Zur Zeit ist die manuelle Eingabe eines Objekts in das Textfeld der Eingabe-Option
 * gesperrt (es kann nur gebrowsed werden). Die Methode {@link #convertFromString(String)}
 * muss noch implementiert werden.
 * @todo Manuelle Eingabe eines Objekts implementieren
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DataPoolInputOption extends BrowseInputOption {
  /** Speichert den Datenpool in den gebrowsed wird. */
  protected DataPool dataPool = null;

  /**
   * Erzeugt eine neue Eingabe-Option.
   * @param dp Datenpool, aus dem ein Objekt angegeben werden soll
   * @param label Label-Text fuer die Eingabe-Option
   * @param inputNeeded gibt an, ob eine Eingabe in der Option erforderlich ist
   * @param defValue Standard-Belegung fuer die Eingabe-Option
   */
  public DataPoolInputOption(DataPool dp, String label, boolean inputNeeded, Object defValue) {
    super(label, inputNeeded, defValue);
    this.dataPool = dp;

    // Eingabe-Feld sperren (bis Objekterkennung aus manuellem Textstring
    // implementiert ist!)
    inpComp.setEnabled(false);
  }

  /**
   * Fuehrt die Browse-Aktion aus, wenn der Anwender auf den entsprechenden
   * Button der Eingabe-Option klickt. Es wird ein Anwender-Dialog angezeigt,
   * der in einer Baumstruktur alle Datenpool-Objekte zur Auswahl stellt.
   * @param defValue Standard-Auswahl fuer den Browse-Dialog
   * @return <code>null</code> wenn der Browse-Dialog abgebrochen wurde
   * @see TreeSelectionDialog
   */
  public Object performBrowse(Object defValue) {
    TreeSelectionDialog dialog = new TreeSelectionDialog(new DataPoolTreeModel(dataPool,false,"DUMMY"));
    // Objekt von Anwender abfragen
    Object[] o = dialog.show(this,XuluConstants.XULUGUI_RES.getString("Object"),false);
    if ( o == null || o.length < 1 )
      return null; // Dialog abgebrochen
    return o[0];
  }

  /**
   * <b>Noch nicht implementiert</b><br>
   * Parst eine textuellen Angabe und liefert ein entsprechendes Objekt aus dem
   * Datenpool.
   * @param objectStr String, der das Datenpool-Objekt identifiziert.
   * @exception UnsupportedOperationException bei <b>jedem</b> Aufruf
   */
  public Object convertFromString(String objectStr) {
    throw new UnsupportedOperationException(XuluConstants.SWING_RES.getString("InvalidInputMess"));
    // wenn diese Methode implementiert ist, kann auch eine manuelle (textuelle)
    // Eingabe im Text-Feld erlaubt werden (-> Konstruktor!)
  }

  /**
   * Liefert die textuelle Bezeichnung zu einem Datenpool-Objekt.
   * @param object Datenpool-Objekt
   */
  public String convertToString(Object object) {
    if ( object instanceof XuluObject )
      return ((XuluObject)object).getDescription();
    if ( object instanceof Property )
      return ((Property)object).getName();
    return object.toString();
  }
}
