/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import javax.swing.tree.DefaultTreeModel;

import schmitzm.swing.tree.EmptyNode;
import schmitzm.swing.tree.ContentNode;

import edu.bonn.xulu.appl.ModelControlManager;
import edu.bonn.xulu.model.XuluModel;

// nur fuer Doku
import schmitzm.swing.TreeSelectionDialog;

/**
 * Diese Klasse stellt ein TreeModel fuer den {@link ModelControlManager Xulu-Modell-Manager}
 * dar. Zwar handelt es sich bei den Modellen lediglich um eine Liste, die
 * eigentlich kein Baumstruktur erfordert, jedoch wird das <code>ModelManagerTreeModel</code>
 * dazu genutzt, Xulu-Datenobjekte und Xulu-Modelle in einem <b>gemeinsamen</b>
 * Eingabe-Dialog zur Auswahl zu stellen.
 * @see ObjectOrModelInputOption
 * @see TreeSelectionDialog
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class ModelManagerTreeModel extends DefaultTreeModel {
  private ModelControlManager modelManager = null;

  /**
   * Erzeugt ein neues Daten-Modell fuer einen {@link ModelControlManager Modell-Manager}.
   * Die Baum-Eintraege sind <b>nicht</b> editierbar.
   * @param mm Instanz eines Modell-Managers
   * @param rootName Bezeichnung fuer den Wurzelknoten
   */
  public ModelManagerTreeModel(ModelControlManager mm, String rootName) {
    super( new EmptyNode(rootName) );
    this.modelManager = mm;
    reorganize();

//    dp.addObjectListener( new ObjectListener() {
//      public void performObjectEvent(ObjectEvent e) {
//        DataPool.DataPoolChangeEvent dpEvent = (DataPool.DataPoolChangeEvent)e;
//        switch (dpEvent.getType()) {
//          // Objekt wurde hinzugefuegt
//          case DataPool.DataPoolChangeEvent.OBJECT_ADDED:
//            add( (XuluObject)dpEvent.getSource().getRoot() );
//            break;
//          // Objekt wurde entfernt
//          case DataPool.DataPoolChangeEvent.OBJECT_REMOVED:
//            remove( (XuluObject)dpEvent.getSource().getRoot() );
//            break;
//        }
//        reload();
//      }
//    });
  }

  /**
   * Baut den Baum komplett neu auf.
   */
  public void reorganize() {
    ((EmptyNode)getRoot()).removeAllChildren();
    // Fuer alle Modelle einen Knoten einfuegen
    for ( int i=0; i<modelManager.getCount(); i++ )
      add( ((ModelControlFrame)modelManager.get(i)).getModel() );
    reload();
  }

  /**
   * Fuegt dem Baum auf der Root-Ebene ein Xulu-Modell hinzu.
   * @param model hinzuzufuegendes Xulu-Modell
   */
  public void add(XuluModel model) {
    ((EmptyNode)getRoot()).add( new ModelNode(model) );
  }


  /**
   * Stellt einen Baum-Knoten fuer ein Xulu-Modell dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class ModelNode extends ContentNode {
    /**
     * Erzeugt einen neuen Modell-Knoten
     * @param model Xulu-Model, das durch den Knoten repraesentiert wird
     */
    public ModelNode(XuluModel model) {
      super(model,model.getName(),false);
    }
  }
}
