/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.io.PrintStream;
import java.io.OutputStream;
import javax.swing.JMenu;
import edu.bonn.xulu.XuluModellingPlatform;
import java.awt.event.ActionListener;
import schmitzm.swing.event.PopupMenuListener;
import java.awt.Dimension;

/**
 * Diese Klasse bildet eine abstrakte Oberklasse fuer alle grafischen
 * Implementierungen des Status-Fensters.<br>
 * Darin koennen beliebige Statusmeldungen ausgegeben werden. Dies kann ueber
 * einzelne Zeilen geschehen ({@link #print(String)}, {@link #println(String)})
 * oder ueber einen {@link #out PrintStream}. So ist es z.B. auch moeglich
 * mittels {@link System#setErr(PrintStream)} die Standard-Fehlerausgabe auf
 * dieses Fenster umzulenken.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public abstract class StatusFrame extends XuluInternalFrame {
  /** Ausgabe-Stream, der an die interne Text-Komponente des Statusfensters gekoppelt ist.
   *  Ruft sukzessibve {@link #print(String)} fuer jedes in den <code>PrintStream</code>
   *  geschriebene Zeichen auf.*/
  public final PrintStream out = new PrintStream(new OutputStream() {
    public void write(int b) {
      print( String.valueOf((char)b) );
    }
  });

  /**
   * Erzeugt einen neue Instanz des grafischen Status-Fensters. Dieses hat
   * den Namen "Status-Ausgaben".
   */
  public StatusFrame() {
    super(XULUGUI_RES.getString("Comp_Log"));
  }

  /**
   * Schreibt einen Text in das Statusfenster (ohne anschliessenden Zeilenumbruch!).
   */
  public abstract void print(String text);

  /**
   * Schreibt einen Text mit anschliessendem Zeilenumbruch in das
   * Statusfenster.
   */
  public void println(String text) {
    print(text);
    print("\n");
  }
}
