/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Vector;
import java.util.Enumeration;
import java.util.ResourceBundle;

import java.awt.Component;
import java.awt.BorderLayout;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;

import appl.parallel.gui.ModelControlFrame_parallel;

import schmitzm.lang.LangUtil;
import schmitzm.lang.AlreadyHandledException;
import schmitzm.lang.OperationCanceledException;
import schmitzm.data.property.Property;
import schmitzm.data.property.ListProperty;
import schmitzm.data.property.ListPropertyReadAccess;
import schmitzm.data.property.ScalarProperty;

import schmitzm.data.property.Properties;
import schmitzm.io.IOUtil;
import schmitzm.lang.DuplicateException;
import schmitzm.swing.SwingWorker;
import schmitzm.swing.MultipleOptionPane;
import schmitzm.swing.InputOption;
import schmitzm.swing.SelectionInputOption;
import schmitzm.swing.ManualInputOption;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.io.IODefinition;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.io.ExportFactory;
import edu.bonn.xulu.io.FactoryCanceledException;
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.data.XuluDataException;
import edu.bonn.xulu.data.DuplicateXuluObjectNameException;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.appl.DataPool;
import edu.bonn.xulu.appl.ScriptInterpreter;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.gui.XuluGUIMessages;
import edu.bonn.xulu.model.ModelResource;
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;

// nur fuer Doku
import schmitzm.swing.StatusDialog;

/**
 * Diese Klasse stellt Funktionen fuer die Xulu-GUI zur Verfuegung.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class XuluGUIUtil implements XuluConstants {
  //######################################################################
  //######################   A L L G E M E I N   #########################
  //######################################################################
  private static Object askObject(Object[] objects, boolean askIfClear) {
    return null;
  }

  /**
   * Fragt eines der in Xulu geladenen Modelle ab.
   * @param appl Xulu-Instanz
   * @param askIfClear Flag, ob der Dialog auch angezeigt werden soll,
   *                   wenn nur ein einziges Modell geladen ist (die Auswahl
   *                   also eindeutig ist)
   * @exception UnsupportedOperationException, falls aktuell <b>kein</b> Modell
   *            in Xulu geladen ist
   */
  public static ModelControlFrame askModelFrame(XuluModellingPlatform appl, boolean askIfClear) {
    // Auswahl der geladenen Modellen aus dem Modell-Manager ermitteln
    ModelControlFrame[] modelFrame         = appl.getModelControlManager().getAll();
    ModelControlFrame   selectedModelFrame = null;
    if ( modelFrame == null || modelFrame.length == 0 )
      throw new UnsupportedOperationException("No models loaded!");
    // wenn nur ein einziges Modell geladen ist, wird kein Auswahl-Dialog
    // angezeigt
    if ( modelFrame.length == 1 && !askIfClear )
      selectedModelFrame = modelFrame[0];
    else {
      String[] modelName = new String[modelFrame.length];
      for (int i = 0; i < modelName.length; i++)
        modelName[i] = modelFrame[i].getTitle();
      // Dialog zum Auswaehlen eines Modells
      Object[] inputOption = MultipleOptionPane.showMultipleInputDialog(
          null,
          XULUGUI_RES.getString("ModelChoiceMess"),
          new SelectionInputOption.Combo(XULUGUI_RES.getString("Model"), true,
                                         modelFrame, 0, modelName)
          );
      // Dialog abgebrochen
      if (inputOption == null)
        return null;
      selectedModelFrame = (ModelControlFrame) inputOption[0];
    }
    return selectedModelFrame;
  }

  /**
   * Fragt eine der in der Xulu-Registry geladenen Modell-Klassen ab.
   * @param appl Xulu-Instanz
   * @param askIfClear Flag, ob der Dialog auch angezeigt werden soll,
   *                   wenn nur eine einzige Modell-Klasse geladen ist (die Auswahl
   *                   also eindeutig ist)
   * @exception UnsupportedOperationException, falls aktuell <b>keine</b> Modell-Klassen
   *            in Xulu registriert sind
   */
  public static Class<XuluModel> askModelClass(XuluModellingPlatform appl, boolean askIfClear) {
    // Auswahl der geladenen Modellen aus dem Modell-Manager ermitteln
    Class[] modelClass = appl.getRegistry().getModelClasses();
    Class   selectedModelClass = null;
    if ( modelClass == null || modelClass.length == 0 )
      throw new UnsupportedOperationException("No models classes registrated in registry!");
    // wenn nur ein einziges Modell geladen ist, wird kein Auswahl-Dialog
    // angezeigt
    if ( modelClass.length == 1 && !askIfClear )
      selectedModelClass = modelClass[0];
    else {
      String[] modelClassName = new String[modelClass.length];
      for (int i = 0; i < modelClassName.length; i++)
        modelClassName[i] = modelClass[i].getName();
      // Dialog zum Auswaehlen eines Modells
      Object[] inputOption = MultipleOptionPane.showMultipleInputDialog(
          null,
          XULUGUI_RES.getString("ModelClassChoiceMess"),
          new SelectionInputOption.Combo(XULUGUI_RES.getString("Model"), true,
                                         modelClass, 0, modelClassName)
          );
      // Dialog abgebrochen
      if (inputOption == null)
        return null;
      selectedModelClass = (Class)inputOption[0];
    }
    return selectedModelClass;
  }

  //######################################################################
  //#########################   I M P O R T   ############################
  //######################################################################
  /**
   * Importiert Objekte ueber eine {@link ImportFactory}. Je nachdem welche
   * Eingabequelle die Factory benoetigt, wird diese ueber einen passenden
   * Anwender Dialog ermittelt. Zur Zeit werden folgende Verfahren unterstuetzt:<br>
   * <br>
   * <table width="80%" align=center border=2 cellpadding=5><code>
   * <tr><th>Art</th><th>benoetigte Quelle</th><th>Ablauf</th></tr>
   * <tr><td>Import aus Datei(en)</td>
   *     <td><code>File<br>File[]<br>InputStream<br>InputStream[]<br>FileInputStream<br>FileInputStream[]</code></td>
   *     <td>Es wird ein Dateiauswahl-Dialog angezeigt, ueber den der Anwender
   *         eine Datei (bzw. mehrere, falls von der Factory gefordert)
   *         fuer den Datenimport bestimmt. Ggf. werden die ausgewaehlten
   *         Files noch in {@link FileInputStream}-Instanzen umgewandelt.
   *         (siehe {@link #importObjectsFromFiles(Component,ImportFactory,XuluModellingPlatform)})</td></tr>
   * </table>
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param fac    fuer den Import zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @return alle von der Factory erzeugten Objekte
   * @throws UnsupportedOperationException falls fuer die von der Factory benoetigte Quelle (noch) kein
   *         Ablauf implementiert ist
   * @throws Exception falls ein unerwarteter Fehler beim Import auftritt
   */
  public static Object[] importObjects(Component parent, ImportFactory fac, XuluModellingPlatform appl) throws Exception{
    Class neededSource = fac.getImportSourceType();
    // Passendes Import-Verfahren fuer die Factory auswaehlen
    if ( neededSource.isAssignableFrom( File.class ) ||
         neededSource.isAssignableFrom( File[].class ) ||
         neededSource.isAssignableFrom( FileInputStream.class ) ||
         neededSource.isAssignableFrom( FileInputStream[].class ) )
      return importObjectsFromFiles(parent,fac,appl);

    // keine passendes Verfahren verfuegbar
    throw new UnsupportedOperationException("Die von der ImportFactory benoetigte Eingabequelle ".concat(neededSource.getSimpleName()).concat(" kann von der Xulu-Plattform zur Zeit (noch) nicht bereitgestellt werden!!"));

  }

  /**
   * Importiert Objekte ueber eine {@link ImportFactory}. Hierzu werden vom
   * Anwender ueber einen {@linkplain JFileChooser Auswahl-Dialog} Dateien als
   * Import-Quelle ausgewahlt. Je nach von der Factory benoetigter
   * Eingabequelle, werden diese in {@link FileInputStream}s umgewandelt.<br>
   * Benoetigt die Factory nur eine Datei (bzw. einen InputStream) als Quelle,
   * wird sie fuer jede vom Anwender ausgewaehlte Datei ausgefuehrt und
   * entsprechend mehrere Objekte erzeugt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param fac    fuer den Import zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @return alle von der Factory erzeugten Objekte
   * @throws UnsupportedOperationException falls fuer die von der Factory
   *         benoetigte Quelle kein <code>File</code>, <code>File[]</code>,
   *         <code>InputStream</code> oder <code>InputStream[]</code> ist.
   * @throws Exception falls ein unerwarteter Fehler beim Import auftritt
   */
  public static Object[] importObjectsFromFiles(final Component parent, final ImportFactory fac, final XuluModellingPlatform appl) throws Exception {
    final Class neededSource = fac.getImportSourceType();

    if ( !neededSource.isAssignableFrom( File.class ) &&
         !neededSource.isAssignableFrom( File[].class ) &&
         !neededSource.isAssignableFrom( FileInputStream.class ) &&
         !neededSource.isAssignableFrom( FileInputStream[].class ) )
      throw new UnsupportedOperationException("Die von der ImportFactory benoetigte Eingabequelle ".concat(neededSource.getSimpleName()).concat(" kann nicht ueber einen Datei-Dialog bereitgestellt werden!!"));

    // Datei(en) auswaehlen
    final JFileChooser fileChooser = appl.getMainFrame().FILE_CHOOSER;
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
//    fileChooser.setDialogTitle("Import > "+fac.getName());
    fileChooser.setMultiSelectionEnabled( true );
    if (fileChooser.showOpenDialog(parent) != JFileChooser.APPROVE_OPTION)
      return new Object[0];

    // Arbeit definieren fuer SwingWorker
    SwingWorker.Work importWork =  new SwingWorker.Work() {
      public Object execute() throws Exception {
        // Quell-Objekt(e) erzeugen
        File[] sourceFile = fileChooser.getSelectedFiles();

        // Factory benoetigt mehrere Objekte als Quelle
        if ( Object[].class.isAssignableFrom( neededSource ) ) {
          Object sourceObject = convertImportSourceAsNeeded(fac,sourceFile);
          // Import in die Liste der Recent-Imports aufnehmen (am Anfang)
          appl.getRegistry().getRecentImports().add( new IODefinition.ImportDefinition(sourceFile,fac,appl.getRegistry()) );
          // Objekte importieren
          Object newObject = fac.importObject(sourceObject,appl.getRegistry());
          // Streams schliessen (falls dies von Factory noch nicht getan wurde!)
          if ( sourceObject instanceof InputStream[] )
            IOUtil.closeInputStream((InputStream[])sourceObject);

          // Rueckgabe-Array erzeugen
          if ( newObject instanceof Object[] )
            return (Object[])newObject;
          return new Object[] { newObject };
        }

        // Factory benoetigt nur ein Objekt als Quelle
        // >> Fuer jeden selektieren File die Factory einmal aufrufen
        //    und erzeugte Objekte in einem Vector sammeln
        Vector newObjects = new Vector();
        for (int i=0; i<sourceFile.length; i++) {
          Object sourceObject = convertImportSourceAsNeeded(fac,sourceFile[i]);
          // Import in die Liste der Recent-Imports aufnehmen
          appl.getRegistry().getRecentImports().add( new IODefinition.ImportDefinition(sourceFile[i],fac,appl.getRegistry()) );
          // Objekte importieren
          try {
            Object newObject = fac.importObject(sourceObject,appl.getRegistry());
            // Objekte im Vector sammeln
            if (newObject instanceof Object[])
              LangUtil.addObjectsToVector( (Object[]) newObject, newObjects);
            else
              newObjects.add(newObject);
          } catch (FactoryCanceledException err) {
            // manueller/gewollter Abbruch der Factory
          } catch (XuluDataException err) {
            XuluGUIMessages.showError(parent, err, "Objekt kann nicht erzeugt werden!");
          } catch (AlreadyHandledException err) {
            // Exception wurde bereits verarbeitet
          } catch (Exception err) {
            XuluGUIMessages.showError(parent, err);
          }
          // Stream schliessen (falls dies von Factory noch nicht getan wurde!)
          if ( sourceObject instanceof InputStream )
            IOUtil.closeInputStream( (InputStream)sourceObject );
        }
        return newObjects.toArray(new Object[0]);
      }
    };

    // Skript im Thread ausfuehren
    SwingWorker worker = new SwingWorker(importWork,appl.getMainFrame(),XULUGUI_RES.getString("ImportProgressMess"),0.2,0.2);
    worker.start();
    Object[] newObjects = (Object[])worker.getWorkResult();
    return newObjects == null ? new Object[0] : newObjects;
  }


  /**
   * Konvertiert ein durch die Plattform zur Verfuegung gestelltes Objekt
   * in ein Objekt, das von der Factory angefordert wird.
   * @param fac Import-Factory
   * @param givenSource gegebenes Objekt
   * @throws Exception falls eine Konvertierung nicht moeglich ist
   */
  public static Object convertImportSourceAsNeeded(ImportFactory fac, Object givenSource) throws Exception {
    Class neededSource = fac.getImportSourceType();

    // Nichts machen, wenn inputSource bereits im benoetigten Format
    if ( neededSource.isAssignableFrom( givenSource.getClass() ) )
      return givenSource;

    // Stream aus Datei erzeugen
    if ( givenSource instanceof File &&
         neededSource.isAssignableFrom( FileInputStream.class ) )
      return new FileInputStream( (File)givenSource );

    // Stream-Array aus Datei-Array erzeugen
    if ( givenSource instanceof File[] &&
         neededSource.isAssignableFrom( FileInputStream[].class ) ) {
      FileInputStream[] stream = new FileInputStream[ ((File[])givenSource).length ];
      for (int i=0; i<stream.length; i++)
        stream[i] = new FileInputStream( ((File[])givenSource)[i] );
      return stream;
    }

    throw new UnsupportedOperationException("Given import source "+givenSource.getClass().getSimpleName()+" can not be converted to needed input "+neededSource.getSimpleName());
  }

  /**
   * Fuegt eine Anzahl an Objekten in den Xulu-Datenpool ein.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param object einzufuegende Objekte
   * @param pool   Datenpool in den die Objekte eingefuegt werden sollen
   * @see #insertObjectToDataPool(Component,Object,DataPool,boolean,boolean)
   */
  public static void insertObjectsToDataPool(Component parent, Object[] object, DataPool pool) {
    // Waehrend der Einfuegung von mehreren Objekten kann der Anwender im
    // Konfliktfall auswaehlen, wie automatisch mit folgenden Konflikten
    // umgegangen werden soll
    // --> Entsprechend werden Einstellungen am Datenpool vorgenommen
    // --> Nach der kollektiven Einfuegung sollen die alten Einstellungen
    //     wieder hergestellt werden
    boolean oldOverwriteOption = pool.getAutoOverwrideDuplicateNames();
    boolean oldDuplicateOption = pool.getDuplicateNamesAllowed();

    boolean autoSkipConflicts = false;
    for (int i = 0; i < object.length; i++)
      try {
        insertObjectToDataPool(parent, object[i], pool, object.length > 1, autoSkipConflicts);
      } catch (OperationCanceledException err) {
        // Einfuegung wurde (ueber Anwender-Dialog) komplett abgebrochen
        // --> Schleife abbrechen
        break;
      } catch ( AlreadyHandledException err ) {
        // Einfuegung wurde (ueber Anwender-Dialog) fuer das Objekt uebergangen
        // und soll auch fuer alle alle Folge-Konflikte uebergangen werden
        // --> Schleife fortsetzen, aber mit autoSkip
        autoSkipConflicts = true;
      }

    // alte Datenpool-Einstellungen wieder herstellen
    pool.setAutoOverwriteDuplicateNames(oldOverwriteOption);
    pool.setDuplicateNamesAllowed(oldDuplicateOption);
  }

  /**
   * Fuegt ein Objekt in den Xulu-Datenpool ein. Fehler werden nicht durch
   * eine Exception verarbeitet, sondern werden ueber eine Meldung dem Anwender
   * mitgeteilt. Das Objekt wird in diesem Fall ignoriert und nicht in den
   * Datenpool eingefuegt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param object einzufuegendes Objekt
   * @param pool   Datenpool in den das Objekt eingefuegt werden sollen
   * @param partOfMultipleInsert gibt an, dass die Einfuege-Operation Teil von
   *                             mehreren Einfuegungen ist; in einem etwaigen
   *                             Dialog kann der Anwender dann angeben, dass seine
   *                             Auswahl auch fuer die Folge-Operationen verwendet
   *                             werden soll
   * @param autoSkipDupl gibt an, ob Duplikat-Fehler automatisch uebergangen werden
   *                     sollen (ohne dass eine Fehlermeldung erscheint)
   * @return {@code false}, falls die Einfuegung nicht vorgenommen werden konnte
   * @exception OperationCanceledException, falls der Anwender in einem
   *            Konflikt-Dialog die "Abbrechen"-Auswahl getroffen hat (Folge-Einfuegungen
   *            sollten in diesem Fall nicht mehr durchgefuehrt werden!)
   * @exception AlreadyHandledException, falls der Anwender in einem
   *            Konflikt-Dialog die "Uebergehen"-Auswahl getroffen hat und
   *            diese Auswahl fuer alle Folge-Einfuegungen gemerkt werden soll
   */
  public static boolean insertObjectToDataPool(Component parent, Object object, DataPool pool, boolean partOfMultipleInsert, boolean autoSkipDupl) {
    try {
      pool.addObject( (XuluObject)object );
      return true;
    } catch (ClassCastException err) {
      XuluGUIMessages.showError(parent,"In den Xulu-Datenpool können nur Instanzen von XuluObject eingefuegt werden!! Keine Instanzen von ".concat(object.getClass().getName()));
    } catch (DuplicateXuluObjectNameException err) {
      // keine Duplikate im Datenpool erlaubt

      // kein Dialog wenn Duplikate automatisch uebergangen werden sollen
      if ( autoSkipDupl )
        return false;

      // Fehler-Dialog
      // --> Abfrage Abbrechen / Uebergehen / Ueberschreiben / Duplikat anlegen
      //     + Auswahl merken (falls Einfuegung Teil von mehreren Einfuegungen ist)
      String    cancelOption    = SWING_RES.getString("Cancel");
      String    skipOption      = SWING_RES.getString("Skip");
      String    overwriteOption = SWING_RES.getString("Overwrite");
      String    duplicateOption = SWING_RES.getString("CreateDuplicate");
      JCheckBox rememberOption  = new JCheckBox(SWING_RES.getString("RememberChoice"), false);
      String    mess = XULUERR_RES.getString("DuplXuluObjectNameMessIntro") + " '" + ((XuluObject)object).getDescription() + "' ...";
      Object selection = MultipleOptionPane.showMultipleOptionDialog(
          parent,
          JOptionPane.ERROR_MESSAGE,
          XULUERR_RES.getString("DataError"),
          partOfMultipleInsert ? new Object[] {  mess, rememberOption } : mess,
          new String[] { cancelOption, skipOption, overwriteOption, duplicateOption },
          skipOption
      );

      // Abbrechen
      if ( selection.equals(cancelOption) )
        throw new OperationCanceledException();
      // Immer Uebergehen
      if ( (selection == null || selection.equals(skipOption)) && rememberOption.isSelected() )
        throw new AlreadyHandledException();
      // Ueberschreiben
      if ( selection.equals(overwriteOption) ) {
        boolean oldOverwriteOption = pool.getAutoOverwrideDuplicateNames();
        pool.setAutoOverwriteDuplicateNames( true );
        pool.addObject( (XuluObject)object );
        if ( !rememberOption.isSelected() )
          pool.setAutoOverwriteDuplicateNames( oldOverwriteOption );
        return true;
      }
      // Duplikat erzeugen
      if ( selection.equals(duplicateOption) ) {
        boolean oldDuplicateOption = pool.getDuplicateNamesAllowed();
        pool.setDuplicateNamesAllowed( true );
        pool.addObject( (XuluObject)object );
        if ( !rememberOption.isSelected() )
          pool.setDuplicateNamesAllowed( oldDuplicateOption );
        return true;
      }

    } catch (Exception err) {
      XuluGUIMessages.showError(parent,err);
    }

    return false;
  }

  /**
   * Importiert ein oder mehrere Objekte ueber eine {@link ImportFactory}
   * und fuegt sie in den Xulu-Datenpool ein. Wie die Import-Quelle vom Anwender
   * abgefragt wird, haengt von der Quellenanforderung der Factory ab.
   * Entsprechend wird ein passender Ablauf gewahlt (siehe
   * {@link #importObjects(Component,ImportFactory,XuluModellingPlatform)}).<br>
   * Fehler beim Import werden nicht durch eine Exception verarbeitet, sondern werden
   * ueber eine Meldung dem Anwender mitgeteilt. Das betroffene Objekt wird
   * in diesem Fall ignoriert und nicht in den Datenpool eingefuegt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param fac    fuer den Import zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @see #importObjectsFromFiles(Component,ImportFactory,XuluModellingPlatform)
   * @see #insertObjectsToDataPool(Component,Object[],DataPool)
   * @exception java.lang.XuluDataException falls die Factory keine Objekte
   *            erzeugt, die im Datenpool verwaltet werden koennen
   */
  public static void importObjectsToDataPool(Component parent, ImportFactory fac, XuluModellingPlatform appl) throws Exception {
    if (!XuluObject.class.isAssignableFrom(fac.getImportType()) &&
        !XuluObject[].class.isAssignableFrom(fac.getImportType()))
      throw new XuluDataException("Factory does not create instances of ".concat(XuluObject.class.getName()));

    Object[] newObject = importObjects(parent,fac,appl);
    insertObjectsToDataPool(parent,newObject,appl.getDataPool());
  }

  /**
   * Importiert ein oder mehrere Objekte ueber eine {@link ImportFactory}
   * und fuegt sie in den Xulu-Datenpool ein. Quelle fuer den Import
   * bilden Dateien, die ueber einen Anwender-Dialog abgefragt werden.
   * Fehler beim Import werden nicht durch eine Exception verarbeitet, sondern werden
   * ueber eine Meldung dem Anwender mitgeteilt. Das betroffene Objekt wird
   * in diesem Fall ignoriert und nicht in den Datenpool eingefuegt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param fac    fuer den Import zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @see #importObjectsFromFiles(Component,ImportFactory,XuluModellingPlatform)
   * @see #insertObjectsToDataPool(Component,Object[],DataPool)
   * @exception java.lang.XuluDataException falls die Factory keine Objekte
   *            erzeugt, die im Datenpool verwaltet werden koennen
   */
  public static void importObjectsFromFileToDataPool(Component parent, ImportFactory fac, XuluModellingPlatform appl) throws Exception {
    if (!XuluObject.class.isAssignableFrom(fac.getImportType()) &&
        !XuluObject[].class.isAssignableFrom(fac.getImportType()))
      throw new XuluDataException("Factory does not create instances of ".concat(XuluObject.class.getName()));

    Object[] newObject = importObjectsFromFiles(parent,fac,appl);
    insertObjectsToDataPool(parent,newObject,appl.getDataPool());
  }


  //######################################################################
  //#########################   E X P O R T   ############################
  //######################################################################
  /**
   * Exportiert ein Objekt ueber eine {@link ExportFactory}. Je nachdem welches
   * Ausgabe-Ziel die Factory benoetigt, wird dieses ueber einen passenden
   * Anwender Dialog ermittelt. Zur Zeit werden folgende Verfahren unterstuetzt:<br>
   * <br>
   * <table width="80%" align=center border=2 cellpadding=5><code>
   * <tr><th>Art</th><th>benoetigtes Ziel</th><th>Ablauf</th></tr>
   * <tr><td>Export in Datei(en)</td>
   *     <td><code>File<br>File[]<br>OutputStream<br>OutputStream[]<br>FileOutputStream<br>FileOutputStream[]</code></td>
   *     <td>Es wird ein Dateiauswahl-Dialog angezeigt, ueber den der Anwender
   *         eine Datei (bzw. mehrere, falls von der Factory gefordert)
   *         fuer den Datenexport bestimmt. Ggf. werden die ausgewaehlten
   *         Files noch in {@link FileOutputStream}-Instanzen umgewandelt.
   *         (siehe {@link #exportObjectsToFiles(Component,Object,ExportFactory,XuluModellingPlatform)})</td></tr>
   * </table>
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param object zu exportierendes Objekt
   * @param fac    fuer den Export zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @throws UnsupportedOperationException falls fuer die von der Factory benoetigte Quelle (noch) kein
   *         Ablauf implementiert ist
   * @throws IllegalArgumentException falls die Factory das angegebene Objekt nicht exportieren kann
   * @throws Exception falls ein unerwarteter Fehler beim Export auftritt
   */
  public static void exportObjects(Component parent, Object object, ExportFactory fac, XuluModellingPlatform appl) throws Exception{
    if ( !fac.isExportable(object) )
      throw new IllegalArgumentException(fac.getClass().getName().concat(" can not export instances of ").concat(object.getClass().getName()));

    Class neededDest = fac.getExportDestinationType();
    // Passendes Export-Verfahren fuer die Factory auswaehlen
    if ( neededDest.isAssignableFrom( File.class ) ||
         neededDest.isAssignableFrom( File[].class ) ||
         neededDest.isAssignableFrom( FileOutputStream.class ) ||
         neededDest.isAssignableFrom( FileOutputStream[].class ) ) {
      exportObjectsToFiles(parent, object, fac, appl);
      return;
    }
    // keine passendes Verfahren verfuegbar
    throw new UnsupportedOperationException("Das von der ExportFactory benoetigte Ausgabeziel ".concat(neededDest.getSimpleName()).concat(" kann von der Xulu-Plattform zur Zeit (noch) nicht bereitgestellt werden!!"));

  }

  /**
   * Exportiert ein Objekt ueber eine {@link ExportFactory}. Hierzu werden vom
   * Anwender ueber einen {@linkplain JFileChooser Auswahl-Dialog} Dateien als
   * Export-Ziel ausgewahlt. Je nach von der Factory benoetigtem
   * Ausgabe-Ziel, werden diese in {@link FileOutputStream}s umgewandelt.<br>
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param fac    fuer den Export zu verwendende Factory
   * @param appl   Instanz der Xulu-Anwendung
   * @throws UnsupportedOperationException falls fuer das von der Factory
   *         benoetigte Ziel kein <code>File</code>, <code>File[]</code>,
   *         <code>OutputStream</code> oder <code>OutputStream[]</code> ist.
   * @throws Exception falls ein unerwarteter Fehler beim Export auftritt
   */
  public static void exportObjectsToFiles(final Component parent, final Object object, final ExportFactory fac, final XuluModellingPlatform appl) throws Exception {
    if ( !fac.isExportable(object) )
      throw new IllegalArgumentException(fac.getClass().getName().concat(" can not export instances of ").concat(object.getClass().getName()));

    final Class neededDest = fac.getExportDestinationType();

    if ( !neededDest.isAssignableFrom( File.class ) &&
         !neededDest.isAssignableFrom( File[].class ) &&
         !neededDest.isAssignableFrom( FileOutputStream.class ) &&
         !neededDest.isAssignableFrom( FileOutputStream[].class ) )
      throw new UnsupportedOperationException("Das von der ExportFactory benoetigte Ausgabeziel ".concat(neededDest.getSimpleName()).concat(" kann nicht ueber einen Datei-Dialog bereitgestellt werden!!"));

    // Datei(en) auswaehlen
    final JFileChooser fileChooser = appl.getMainFrame().FILE_CHOOSER;
    fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fileChooser.setMultiSelectionEnabled( Object[].class.isAssignableFrom(neededDest) );
    if (fileChooser.showSaveDialog(parent) != JFileChooser.APPROVE_OPTION)
      return;

    // Arbeit definieren fuer SwingWorker
    SwingWorker.Work exportWork =  new SwingWorker.Work() {
      Object destObject = null;

      public Object execute() throws Exception {
        // Ziel-Objekt(e) erzeugen
        // Factory benoetigt mehrere Objekte als Ziel
        if ( Object[].class.isAssignableFrom( neededDest ) ) {
          File[] destFile = fileChooser.getSelectedFiles();
          // Pruefen, ob Dateien bereits existieren
          for (File file : destFile) {
            XuluGUIMessages.FileOverrideOption opt = XuluGUIMessages.checkFileOverwrite(parent, file, destFile.length > 1);
            if ( opt == XuluGUIMessages.FileOverrideOption.CANCEL )
              return null;
            if ( opt == XuluGUIMessages.FileOverrideOption.OVERWRITE_ALL )
              break;
          }
          destObject = destFile; // Default: Dateien
          // Factory benoetigt mehrere OutputStreams
          if ( neededDest.isAssignableFrom( FileOutputStream[].class ) ) {
            destObject = new OutputStream[destFile.length];
            for (int i=0; i<destFile.length; i++)
              ((OutputStream[])destObject)[i] = new FileOutputStream(destFile[i]);
          }
        } else {
          // Factory benoetigt ein Objekt als Ziel
          File destFile = fileChooser.getSelectedFile();
          // Pruefen, ob Dateien bereits existieren
          if ( XuluGUIMessages.checkFileOverwrite(parent, destFile, false) == XuluGUIMessages.FileOverrideOption.CANCEL )
            return null;
          destObject = destFile; // Default: Datei
          // Factory benoetigt OutputStream
          if ( neededDest.isAssignableFrom( FileOutputStream.class ) )
            destObject = new FileOutputStream(destFile);
        }

        // Objekte exportieren
        fac.exportObject(object,destObject);
        closeDestObject();

        return null;
      }

      public void performError(Throwable err) {
        closeDestObject();
        super.performError(err);
      }

      private void closeDestObject() {
        if ( destObject == null )
          return;

        // Streams schliessen (falls dies von Factory noch nicht getan wurde!)
        if ( destObject instanceof OutputStream[] ) {
          OutputStream[] out = (OutputStream[])destObject;
          for (int i=0; i<out.length; i++)
            IOUtil.closeOutputStream(out[i]);
        }
        if ( destObject instanceof OutputStream )
          IOUtil.closeOutputStream( (OutputStream)destObject );
      }
    };

    // Skript im Thread ausfuehren
    new SwingWorker(exportWork,appl.getMainFrame(),XULUGUI_RES.getString("ExportProgressMess"),0.2,0.2).start();
  }

  //######################################################################
  //###################   N E U E S   M O D E L L   ######################
  //######################################################################
  /**
   * Ermittelt alle registrierten Modelle aus der Xulu-Registry und gibt
   * sie dem Anwender zur Auswahl. Von dem ausgewaehlten Modell-Typ wird
   * eine neue Instanz erzeugt und ein entsprechender {@link ModelControlFrame_Basic}
   * dem ModelControlManager hinzugefuegt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param appl   Instanz der Xulu-Anwendung
   */
  public static void createNewModel(Component parent, XuluModellingPlatform appl) {
    // Auswahl an Modellen aus der Registry ermitteln
    Class[] modelClass = appl.getRegistry().getModelClasses();
    Object[] input     = MultipleOptionPane.showClassAndDescInputDialog(parent,XULUGUI_RES.getString("ModelChoiceMess"),modelClass,XULUGUI_RES.getString("DefaultModelName"));
    if ( input != null )
      createNewModel(parent,(Class)input[0],(String)input[1],appl);
  }
  /**
   * Erzeugt ein neues Model aus einem übergebenen Modellklassennamen
   * und einer Modelbezeichnung
   *
   * @param modelClass Modellklasse
   * @param modelName Modellbezeichnung
   */
  public static void createNewModel(Component parent, Class modelClass, String modelName, XuluModellingPlatform appl) {
      if ( modelClass ==null || modelName==null)
        return;

      ModelControlFrame controlFrame = null;
      try {
        // Modell instanziieren
        XuluModel model = (XuluModel)modelClass.newInstance();
        // Modellfenster erzeugen
        controlFrame = new ModelControlFrame_parallel(appl, model);
        controlFrame.setTitle( modelName );
        appl.getModelControlManager().add(controlFrame);
      } catch (InstantiationException err) {
//        System.err.println("ERROR: Could not create instance of " + modelClass);
        XuluGUIMessages.showInstantiationError(parent,err);
      } catch (IllegalAccessException err) {
//        System.out.println("Access violation while creating Model: " + err.getMessage());
        XuluGUIMessages.showError(parent,err);
      } catch (DuplicateException err) {
        // Modell-Manager enthaelt bereits ein Modell mit dem angegebenen Namen
        // -> Ersetzen-Abfrage
        String cancelOption  = SWING_RES.getString("Cancel");
        String replaceOption = SWING_RES.getString("Replace");
        String mess          = XULUERR_RES.getString("DuplModelNameMessIntro") + " '" + modelName + "' ...";
        Object selection = MultipleOptionPane.showMultipleOptionDialog(
            parent,
            JOptionPane.ERROR_MESSAGE,
            SWING_RES.getString("Error"),
            mess,
            new String[] { cancelOption, replaceOption },
            cancelOption
        );
        if ( selection == replaceOption ) {
          // Alle Modelle mit dem gleichen Namen loeschen
          ModelControlFrame[] frames = appl.getModelControlManager().getAll();
          for ( ModelControlFrame f : frames )
            if ( controlFrame.equals(f) )
              appl.getModelControlManager().remove(f);
          // Modell nochmal einzufuegen
          appl.getModelControlManager().add(controlFrame);
        }

      } catch (AlreadyHandledException err) {
        // nichts machen
      }
    }

  /**
   * Ermittelt alle geladenen Modelle aus dem Xulu-Datenpool und gibt
   * sie dem Anwender zur Auswahl. Von dem ausgewaehlten Modell wird
   * ein entsprechender {@link ModelControlFrame_Basic} dem ModelControlManager
   * hinzugefuegt.
   * @param parent aufrufende Komponente (kann <code>null</code> sein)
   * @param appl   Instanz der Xulu-Anwendung
   */
  public static void createNewDynamicModel(Component parent, XuluModellingPlatform appl) {
    // Auswahl an Modellen aus dem Datenpool ermitteln Registry ermitteln
    XuluObject[] modelObj = appl.getDataPool().getObjects( XuluModel.class );
    // Eingabe-Optionen fuer den Dialog erstellen
    Object[] input = MultipleOptionPane.showMultipleInputDialog(
      parent,
      XULUGUI_RES.getString("ModelChoiceMess"),
      new InputOption[] { new SelectionInputOption.Combo(XULUGUI_RES.getString("DynModelChoiceMess"),true,modelObj,null),
      new ManualInputOption.Text(SWING_RES.getString("Description"),true) }
    );

    if ( input != null ) {
      // Modellfenster erzeugen
      ModelControlFrame controlFrame = new ModelControlFrame_Basic(appl,(XuluModel)input[0]);
      controlFrame.setTitle( (String) input[1]);
      appl.getModelControlManager().add(controlFrame);
    }
  }

  //######################################################################
  //###############   M O D E L L   N E U   L A D E N  ###################
  //######################################################################
  /**
   * Ermittelt alle Modell-Klassen aus der Xulu-Registry und stellt sie
   * dem Anwender zur Auswahl. Die ausgewaehlte Klasse wird neu geladen,
   * so dass fuer alle folgenden Instanzen dieser Klasse die aktualisierte
   * Version verwendet wird.
   * @param appl Xulu-Instanz
   */
  public static void reloadModelClass(XuluModellingPlatform appl) {
    // Modell-Klasse abfragen
    Class<XuluModel> oldModelClass = askModelClass(appl,true);
    // Dialog abgebrochen
    if ( oldModelClass == null )
      return;

//    // GarbageCollection aufrufen
//    Runtime.getRuntime().runFinalization();
//    LangUtil.gcTotal();
    try {
      // Klasse neu laden
//      Class newModelClass = LangUtil.loadClass( oldModelClass.getName(), appl.getDynamicClassRootDirectory().toURL() );
      Class newModelClass = LangUtil.loadPreferredClass( oldModelClass.getName(), appl.getDynamicClassRootDirectory().toURL() );
      // Alte Klasse in Registry durch neue ersetzen
      appl.getRegistry().replaceModelClass(oldModelClass,newModelClass);
      // Meldung geben
      XuluGUIMessages.showInfoMessage(appl.getMainFrame(),XULUGUI_RES.getString("ModelReloadSuccessMess"));
    } catch (ClassNotFoundException err) {
      XuluGUIMessages.showClassNotFoundError(appl.getMainFrame(),err,oldModelClass.getName());
    } catch (Exception err) {
      XuluGUIMessages.showError(appl.getMainFrame(),err);
    }
  }

  //######################################################################
  //#########################   S C R I P T   ############################
  //######################################################################
  /**
   * Fuehrt ein Skript innerhalb der Xulu-Modelling-Platform aus. Hierzu
   * wird das Skript ueber einen Datei-Eingabe-Dialog abgefragt und
   * in einem eigenen {@linkplain SwingWorker Thread} ausgefuehrt.<br>
   * Waehrend der Ausfuehrung wird ein Status-Dialog angezeigt. Wird dieser
   * durch den Anwender abgebrochen, wird die Skript-Ausfuehrung unmittelbar
   * gestoppt.
   * @param appl        Applikation innerhalb der das Skript ausgefuehrt wird
   * @param interpreter Interpreter, der das Skript ausfuehrt
   * @see SwingWorker
   * @see StatusDialog
   */
  public static void performScriptInThread(final XuluModellingPlatform appl, final ScriptInterpreter interpreter) {
    // Datei auswaehlen
    final JFileChooser fileChooser = appl.getMainFrame().FILE_CHOOSER;
    fileChooser.setMultiSelectionEnabled(false);
    if ( fileChooser.showOpenDialog(appl.getMainFrame()) != JFileChooser.APPROVE_OPTION )
      return;

    // Arbeits-Verzeichnis neu setzen
    appl.setWorkingDirectory( fileChooser.getCurrentDirectory()  );

    // Skript-Definition erzeugen
    ScriptInterpreter.ScriptExecutionDefinition scriptDef =
        new ScriptInterpreter.ScriptFileExecutionDefinition(fileChooser.getSelectedFile(),interpreter);
    // Skript ausfuehren
    performScriptInThread(appl,scriptDef);
  }

  /**
   * Fuehrt ein Skript innerhalb der Xulu-Modelling-Platform aus. Hierbei
   * wird das Skript in einem eigenen {@linkplain SwingWorker Thread}
   * ausgefuehrt.<br>
   * Waehrend der Ausfuehrung wird ein Status-Dialog angezeigt. Wird dieser
   * durch den Anwender abgebrochen, wird die Skript-Ausfuehrung unmittelbar
   * gestoppt.
   * @param appl      Applikation innerhalb der das Skript ausgefuehrt wird
   * @param scriptDef Script-Definition
   * @see SwingWorker
   * @see StatusDialog
   */
  public static void performScriptInThread(final XuluModellingPlatform appl, final ScriptInterpreter.ScriptExecutionDefinition scriptDef) {
    // Skript-Definition an den Anfang der Recent Scripts setzen
//    appl.getRegistry().getRecentScripts().remove( scriptDef );
    appl.getRegistry().getRecentScripts().add( scriptDef );

    // "Arbeit" fuer Thread erzeugen
    SwingWorker.Work scriptWork =  new SwingWorker.Work() {
      public Object execute() throws Exception {
        scriptDef.execute(appl);
        return null;
      }
    };

    // Skript im Thread ausfuehren
    new SwingWorker(scriptWork,appl.getMainFrame(),XULUGUI_RES.getString("ScriptProgressMess"),0.2,0.2).start();
  }

  //######################################################################
  //############   Objekte/Objektbezeichnungen ermitteln   ###############
  //######################################################################
  /**
   * Ermittelt (rekursiv) alle Xulu-Objekte und Eigenschaften aus dem Xulu-Datenpool,
   * die zu einer Klasse (oder Ressource) passen.
   * @param type      Objekt-Type (spezifiert durch ein {@link Class}-Objekt oder
   *                  ein {@link ModelResource}-Objekt)
   * @param dataPool  Datenpool, in dem gesucht wird
   * @return 2-dimensionan Array {@code obj[][]}, wobei {@code obj[0]} die ermittelten
   *         Objekte enthaelt und {@code obj[1]} deren Objektnamen.
   * @exception IllegalArgumentException falls {@code type} keine {@link Class}-
   *            oder {@link ModelResource}-Instanz ist
   */
  public static Object[][] determineObjectsFromDataPool(Object type, DataPool dataPool) {
    Vector         objects = new Vector();
    Vector<String> names   = new Vector<String>();
    determineObjectsFromList(type,dataPool.asEnumeration(),objects,names,"");
    return new Object[][] { objects.toArray(), (String[])names.toArray(new String[0]) };
  }

  /**
   * Ermittelt (rekursiv) alle Xulu-Objekte und Eigenschaften aus dem Xulu-Datenpool,
   * die zu einer Klasse (oder Ressource) passen.
   * @param type      Objekt-Type (spezifiert durch ein {@link Class}-Objekt oder
   *                  ein {@link ModelResource}-Objekt)
   * @param dataPool   Liste von Objekten, in denen gesucht wird
   * @param objects   wird mit den gefundenen (passenden) Objekten gefuellt
   * @param names     wird mit den Bezeichnungen zu den <code>objects</code>-Elementen
   *                  befuellt
   * @exception IllegalArgumentException falls {@code type} keine {@link Class}-
   *            oder {@link ModelResource}-Instanz ist
   */
  public static void determineObjectsFromDataPool(Object type, DataPool dataPool, Vector objects, Vector<String> names) {
    determineObjectsFromList(type,dataPool.asEnumeration(),objects,names,"");
  }

  /**
   * Ermittelt (rekursiv) alle Xulu-Objekte und Eigenschaften aus einer
   * Liste, die zu einer Klasse (oder Ressource) passen.
   * @param type      Objekt-Type (spezifiert durch ein {@link Class}-Objekt oder
   *                  ein {@link ModelResource}-Objekt)
   * @param objList   Liste von Objekten, in denen gesucht wird
   * @param objects   wird mit den gefundenen (passenden) Objekten gefuellt
   * @param names     wird mit den Bezeichnungen zu den <code>objects</code>-Elementen
   *                  befuellt
   * @param namePrefix Praefix, die den Bezeichnungen vorangestellt wird, bevor
   *                   sie in <code>names</code> eingefuegt werden (wird fuer Rekursion
   *                   benoetigt, sollte beim Aufruf "" oder {@code null} sein)
   * @exception IllegalArgumentException falls {@code type} keine {@link Class}-
   *            oder {@link ModelResource}-Instanz ist
   */
  public static void determineObjectsFromList(Object type, Enumeration objList, Vector objects, Vector<String> names, String namePrefix) {
    if ( !(type instanceof Class) && !(type instanceof ModelResource) )
      throw new IllegalArgumentException("Instance of Class or ModelResource expected as type specification!");
    if ( namePrefix == null )
      namePrefix = "";

    for (;objList.hasMoreElements();) {
      Object obj = objList.nextElement();
      String name = "";
      if ( obj instanceof XuluObject )
        name = ((XuluObject)obj).getDescription();
      else if ( obj instanceof Property )
        name = ((Property)obj).getName();

      // passt das Objekt zur Ressource, wird es in die Liste aufgenommen
      if ( type instanceof Class && ((Class)type).isInstance(obj) ||
           type instanceof ModelResource && ((ModelResource)type).checkType(obj) ) {
        objects.add(obj);
        names.add( namePrefix.concat(name) );
      } else if ( obj instanceof ScalarProperty ) {
        Object propObj = ((ScalarProperty)obj).getOneTimeReadAccess().getValue();
        if ( type instanceof Class && ((Class)type).isInstance(propObj) ||
             type instanceof ModelResource && ((ModelResource)type).checkType(propObj) ) {
          objects.add(propObj);
          names.add( namePrefix.concat(name) );
        }
      } else if ( obj instanceof ListProperty ) {
        ListPropertyReadAccess listPropAccess = ((ListProperty)obj).getReadAccess(XuluGUIUtil.class);
        for (int i=0; i<listPropAccess.getCount(); i++) {
          Object propObj = listPropAccess.getValue(i);
          if ( type instanceof Class && ((Class)type).isInstance(propObj) ||
               type instanceof ModelResource && ((ModelResource)type).checkType(propObj) ) {
            objects.add(propObj);
            names.add( namePrefix.concat(name+"["+i+"]") );
          }
        }
        listPropAccess.release();
      }

      // wenn das Objekt aus Eigenschaften besteht wird rekursiv
      // weitergesucht
      if (obj instanceof Properties) {
        //=== Property-Array in Liste umwandeln ===
        Property[] prop = ( (Properties) obj).getProperties();
        Vector propVec = new Vector(prop.length);
        for (int i = 0; i < prop.length; i++)
          if ( prop[i] != obj )
            propVec.add(prop[i]); // Property nur in Liste aufnehmen,
                                  // wenn es sich nicht um das Vater-Objekt
                                  // selbst handelt
        //=========================================

        // Rekursion starten
        determineObjectsFromList(type,
                                 propVec.elements(),
                                 objects,
                                 names,
                                 namePrefix.concat(name).concat(".")
        );
      }
    }
  }


}
