/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.gui;

import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ImageIcon;
import java.util.ResourceBundle;
import java.util.Vector;
import java.io.PrintStream;

import schmitzm.lang.ResourceProvider;
import schmitzm.swing.SwingUtil;
import schmitzm.swing.StatusDialog;
import schmitzm.swing.TextAreaPrintStream;
import edu.bonn.xulu.appl.XuluConstants;

// nur fuer Doku
import edu.bonn.xulu.XuluModellingPlatform;

/**
 * Diese Klasse stellt einen Dialog dar, der angezeigt wird, waehrend die
 * Xulu-Applikation gestartet wird. Neben einem Status-Balken wird ein
 * Text-Feld angezeigt, in dem (durch den {@link XuluModellingPlatform}-Konstruktor)
 * die einzelnen Phasen der Initialisierung aufgelistet werden. Hierzu stellt
 * diese Klasse einen {@link #getPrintStream() PrintStream} zur Verfuegung, in
 * den nach und nach die Meldungen hineingeschrieben werden koennen.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class XuluStartingFrame extends StatusDialog {
  private JTextArea           textArea;
  private TextAreaPrintStream stream;
//  // Bemerkung: Es koennen nicht die ResourceBundles aus XuluConstants verwendet
//  //            werden, da zum Zeitpunkt des StartingFrame die CommandLine-Parameters
//  //            noch nicht eingelesen wurden und somit die DefaultLocale noch
//  //            nicht gesetzt wurde. Wird zu diesem Zeitpunkt auf diese ResourceBundles
//  //            zugegriffen, werden diese nicht entsprechend dem CommandLine-Parameter
//  //            initialisiert!
//  private static ResourceBundle SWING_RES   = ResourceBundle.getBundle(SwingResource.class.getName());
//  private static ResourceBundle XULUGUI_RES = ResourceBundle.getBundle(XuluGUIResource.class.getName());
    /**
     * Erzeugt einen neues Start-Fenster.
     */
    public XuluStartingFrame() {
      super(null,XuluConstants.XULUGUI_RES.getString("StartMess")+" v"+XuluConstants.VERSION+" ["+XuluConstants.VERSION_DATE+"]",XuluConstants.SWING_RES.getString("WaitMess"),XuluConstants.XULU_STARTICON,0.5,0.5);
      setResizable(false);
      stream = new TextAreaPrintStream(textArea);
    }

    /**
     * Fuegt zu den GUI-Komponenten von {@link StatusDialog} ein
     * Text-Fenster in die Komponenten-Liste ein.
     * @param list Liste der GUI-Komponenten des Dialogs
     */
    protected void insertInformationComponents(Vector<Component> list) {
      super.insertInformationComponents(list);
      textArea = new JTextArea(10,30);
      textArea.setEditable(false);
      textArea.setFont( new JLabel().getFont() );
      textArea.setBackground( new JLabel().getBackground() );
      list.add( new JScrollPane(textArea) );
    }

    /**
     * Liefert einen Stream, der auf das Text-Fenster umgeleitet wird.
     */
    public PrintStream getPrintStream() {
      return stream;
    }
}
