/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.model;

import schmitzm.lang.WorkingThread;
import edu.bonn.xulu.model.XuluModel;

/**
 * Diese Klasse stellt einen Thread dar, der ein {@link XuluModel} ausfuehrt.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class XuluModelThread extends WorkingThread {
  /** Speichert das Modell, welches ausgefuehrt wird. */
  protected XuluModel model = null;

  /**
   * Erzeugt einen neuen Thread.
   * @param model auszufuehrendes Modell
   */
  public XuluModelThread(XuluModel model) {
    super();
    this.model = model;
  }

  /**
   * Initialisiert den Thread, in dem das Modell initialisiert wird. Loest
   * {@link WorkingThread#fireThreadInitialised()} aus.
   * @see XuluModel#initModel()
   */
  public void performInit() {
    if ( !model.isInitialised() )
      model.initModel();
    fireThreadInitialised();
  }

  /**
   * Implementiert den Arbeitsschritt des Threads, in dem das Modell gestartet wird.
   * @see XuluModel#startModelling()
   */
  public void performWork() {
    model.startModelling();
  }

  /**
   * Macht nichts, da diese Klasse keine Ressourcen gesperrt hat, die vor
   * dem Auslaufen des Threads wieder freigegeben werden muessten.
   * Sub-Klassen muessen diese Methode ggf. ueberschreiben.
   */
  public void performDispose() {
    model.disposeModel();
  }

  /**
   * Beendet das Modell und den Thread.
   * @see XuluModel#stopModelling()
   */
  public void terminate() {
    model.stopModelling();
    super.terminate();
  }

}
