/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.text.DateFormat;
import java.util.GregorianCalendar;
import java.awt.BorderLayout;
import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.JLabel;

import schmitzm.lang.WorkingThread;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluPlugin;

/**
 * Diese Klasse stellt ein Plugin fuer die Xulu-Applikation dar, welches die
 * aktuelle Uhrzeit (und Datum) in einem eigenen (kleinen) Fester anzeigt.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DateTimeWindowPlugin extends JFrame implements XuluPlugin {
  private String      name        = "";
  private ClockThread clockThread = null;
  private JLabel      clockLabel  = new JLabel(" ");

  /**
   * Erzeugt ein neues Plugin.
   */
  public DateTimeWindowPlugin() {
    super("Current Date/Time");
    this.setLayout( new BorderLayout() );
    this.setAlwaysOnTop(true);
    this.setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
    this.setPreferredSize( new Dimension(200,60) );
    this.getContentPane().add( clockLabel );
    this.clockLabel.setHorizontalAlignment(JLabel.CENTER);
    this.pack();

  }

  /**
   * Startet die Ausfuehrung des Plugins.
   * @param appl Instanz der Xulu-Applikation
   */
  public void execute(XuluModellingPlatform appl) {
    this.clockThread = new ClockThread(clockLabel);
    clockThread.start();
    setVisible(true);
  }

  /**
   * Stoppt die Ausfuehrung des Plugins.
   */
  public void stop() {
    setVisible(false);
    clockThread.terminate();
  }

  /**
   * Prueft, ob das Plugin gestartet ist.
   */
  public boolean isStarted() {
    return clockThread != null && clockThread.isRunning();
  }

  /**
   * Liefert eine Beschreibung des Plugins.
   */
  public String getName() {
    return name;
  }

  /**
   * Setzt die Beschreibung des Plugins.
   * @param name neue Bezeichnung
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Diese Klasse stellt den Uhrzeit-Thread dar, der die Uhrzeit in dem
   * Fenster aktualisiert.
   */
  private static class ClockThread extends WorkingThread {
    private JLabel            clockLabel;
    private DateFormat        dateFormat  = DateFormat.getDateTimeInstance();
    private int               lastSec;

    /**
     * Erzeugt einen neuen Uhrzeit-Thread.
     * @param clockLabel Label, in dem die Uhrzeit dargestellt werden soll
     */
    public ClockThread(JLabel clockLabel) {
      this.clockLabel = clockLabel;
    }

    /**
     * Initialisiert den Thread.
     */
    public void performInit() {
      this.lastSec  = -1;
    }

    /**
     * Implementiert die Aktualisierung der Zeit-Anzeige.
     */
    public void performWork() {
      while (true) {
        this.checkBreakingCommands();
        GregorianCalendar calendar = new GregorianCalendar();
        int actSec = calendar.get(GregorianCalendar.SECOND);
        if ( actSec != lastSec )
          clockLabel.setText( dateFormat.format( calendar.getTime() ) );
        lastSec = actSec;
        try {
          this.sleep(100);
        }
        catch (InterruptedException err) {
        }
      }
    }

    /**
     * Macht nichts.
     */
    public void performDispose() {
    }
  }
}
