/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.io.File;
import javax.swing.JTextField;
import org.apache.log4j.LogManager;

import schmitzm.swing.log4j.LoggerFrame;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluPlugin;

// fuer Doku
import org.apache.log4j.Level;

/**
 * Dieses Xulu-Plugin zeigt alle im {@link LogManager} registrierten Log4j-Logger an
 * und bietet die Möglichkeit, deren {@link Level} zur Laufzeit veraendern.
 * Dies gestattet z.B. ein Ein- und Ausschalten von Debug-Informationen.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class LoggerPlugin extends LoggerFrame implements XuluPlugin {
  /** Log4j-Konfigurationsdatei aus der die Logger gelesen werden.
   *  {@code <XuluPluginVz>\LoggerPlugin\log4j.cfg} */
  public static final String CONFIG_FILE_NAME = "LoggerPlugin"+File.separatorChar+"log4j.cfg";

  /** Instanz der Xulu-Applikation innerhalb der das Plugin ausgefuehrt wird. */
  protected XuluModellingPlatform appl = null;
  /** Flag speichert, ob das Plugin gestartet ist. */
  protected boolean started = false;

  /**
   * Erzeugt eine neue Plugin-Instanz.
   */
  public LoggerPlugin() {
    super();
//    ((JTextField)configFile.getInputComponent()).setEditable(false);
  }

  /**
   * Fuehrt das Plugin aus, in dem die Logger neu aus der Konfigurationsdatei
   * geladen werden.
   * @param appl Xulu-Instanz in der das Plugin ausgefuehrt wird
   * @see XuluModellingPlatform#getPluginDirectory()
   * @see #CONFIG_FILE_NAME
   */
  public void execute(XuluModellingPlatform appl) {
    // Pruefen, ob ein log-Verzeichnis unterhalb des Start-Verzeichnisses
    // existiert
    new File( appl.getStartingDirectory(), "log" ).mkdirs();

    this.appl    = appl;
    this.started = true;
    configFileField.setValue( new File(appl.getPluginDirectory(),CONFIG_FILE_NAME) );
    reload();
  }

  /**
   * Prueft, ob das Plugin gestartet ist.
   */
  public boolean isStarted() {
    return started;
  }

  /**
   * Stoppt das Plugin.
   * @see LogManager#resetConfiguration()
   * @see LogManager#shutdown()
   */
  public void stop() {
    started = false;
    this.setVisible(false);
    LogManager.resetConfiguration();
    LogManager.shutdown();
  }
}
