/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.awt.Container;

import schmitzm.io.IOUtil;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.AbstractScriptInterpreter;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.gui.ModelControlFrame;
import edu.bonn.xulu.gui.ModelContentManagerContainer;
import edu.bonn.xulu.gui.XuluGUIUtil;

// nur fuer Doku
import edu.bonn.xulu.plugin.gui.ModelControlFrame_Basic;

/**
 * Dieser Interpreter fuehrt ein Mapping zwischen Datenpool-Objekten und
 * Modell-Ressourcen durch und kann <b>nur in Verbindung mit {@link ModelControlFrame_Basic}
 * (bzw. {@link ModelContentManagerContainer}) verwendet werden!</b><br>
 * Leerzeilen und {@linkplain IOUtil#isCommentLine(String)
 * Kommentarteilen} im Stream werden ignoriert. In jeder Zeile steht der Name eines
 * Objekts, so wie er im jeweiligen Auswahlfeld des {@link ModelContentManagerContainer}-Fensters
 * vorgeblendet ist. Die Reihenfolge der im Stream angegebenen Objekte, muss
 * der Ressourcen-Reihenfolge des Modells (als im Fenster des
 * {@link ModelContentManagerContainer} zeilenweise von links nach rechts!)
 * entsprechen. Soll eine Ressource nicht zugeordnet werden, ist in der
 * jeweiligen Zeile ein Tilde-Zeichen (~) zu setzen.<br>
 * <b>ACHTUNG: Diese Klasse arbeitet sehr rudimentaer (ein wenig Quick-and-Dirty) und befindet sich noch im Test!!</b>
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class ModelResourceMappingScriptInterpreter_Basic extends AbstractScriptInterpreter implements XuluConstants {
  /**
   * Fuehrt das Script aus. Vor dem Einlesen der Ressourcen-Zuordnung wird
   * zunaechst vom Anwender eine Modell-Instanz abgefragt, auf die das Skript
   * angewandt werden soll (nur sofern mehrere Modelle geladen sind!).
   * @param in   Eingabe-Stream in dem das Skript zu finden ist.
   * @param appl Applikation, in der das Skript auszufuehren ist
   * @see XuluGUIUtil#askModelFrame(XuluModellingPlatform, boolean)
   */
  public void execute(InputStream in, XuluModellingPlatform appl) {
    // Modell abfragen, auf das das Skript angewandt werden soll
    ModelControlFrame selectedModelFrame = XuluGUIUtil.askModelFrame(appl,false);
    // Dialog abgebrochen
    if (selectedModelFrame == null)
      return;

    map(in,selectedModelFrame);
    }

      /**
     * Führt als InputStream übergebene Script aus. Dabei wird das gefundene
     * Mapping auf <b>alle</b> geladenen Modelle mit dem angegebenen Namen
     * angewendet.
     *
     * @param in   Eingabe-Stream in dem das Skript zu finden ist.
     * @param appl Applikation, in der das Skript auszufuehren ist
     *
     * @param modelName das Mapping gilt für alle Modelle mit diesem Namen
     */

  public void execute(InputStream in, String modelName, XuluModellingPlatform appl)
  {
     boolean modelFound = false;
     ModelControlFrame[] controlFrames = appl.getModelControlManager().getAll();
     for (ModelControlFrame c : controlFrames)
       if(c != null && c.getTitle().equals(modelName)){
           modelFound = true;
           map(in,c);
       }
     if(!modelFound)
         System.err.println("Mapping failed: No Model with the name \"" + modelName + "\" found!" );

  }
    /** Führt das eigentliche Mapping aus **/
    private void map(InputStream in, ModelControlFrame controlFrame){
    // Der ModelControlFrame muss einen ModelContentManagerContainer enthalten,
    // sonst kann die Zuordnung nicht druchgefuehrt werden kann
    Container contentPane = controlFrame.getContentPane();
    XuluModel model       = controlFrame.getModel();
    ModelContentManagerContainer resourceContainer = null;
    for (int i=0; i<contentPane.getComponentCount() && resourceContainer==null; i++) {
      if ( contentPane.getComponent(i) instanceof ModelContentManagerContainer )
        resourceContainer = (ModelContentManagerContainer)contentPane.getComponent(i);
    }
    if(controlFrame instanceof ModelControlFrame_Basic)
    	resourceContainer = ((ModelControlFrame_Basic) controlFrame).getModelContentManagerContainer();
    if ( resourceContainer == null )
      throw new UnsupportedOperationException("The script can be applied only on ModelControlFrames which contain a ModelContentManagerContainer.");


    // Stream parsen
    BufferedReader input      = new BufferedReader( new InputStreamReader(in) );
    String         line       = null;
    int            lineNo     = 0;
    int            resourceNo = 0;
    System.out.println("\nStart executing script...");
    try {
      // Zeilenweise einlesen
      for(lineNo=1; resourceNo < model.getContentManager().getResourceCount() && (line = input.readLine()) != null; lineNo++) {
        line = line.trim();
        // Leerzeilen und Kommentare ignorieren
        if ( line.equals("") || IOUtil.isCommentLine(line) )
          continue;

        // Zuordnung erstellen
        System.out.print(">>>> Model resource " + (resourceNo+1));
        if ( !line.equals("~") ) {
          System.out.println(" mapped to '" + line + "'");
          resourceContainer.setResourceByName(resourceNo, line);
        } else {
          System.out.println(" ignored");
        }
        resourceNo++;
      }

      // Stream schliessen
      in.close();
    } catch ( Exception err ) {
      System.err.println("Unknown error while parsing line "+lineNo);
    }
    System.out.println("Script finished...\n");
  }
}
