/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.BorderLayout;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.gui.XuluInternalFrame;

// nur fuer Doku
import schmitzm.data.RasterCalculator;
import edu.bonn.xulu.plugin.appl.RasterCalculatorPlugin;

/**
 * Diese Klasse stellt eine GUI zur Vefuegung, mit der eine einfache arithmetische
 * (und boolsche) Berechnung auf Rasterdaten durchgefuehrt werden kann. Zudem
 * koennen in die Formel Filter eingebunden werden.
 * @see RasterCalculator
 * @see RasterCalculatorPlugin
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class RasterCalculatorGUI extends XuluInternalFrame {
  /** Referenz auf den Container des Fensters. */
  protected Container contentPane = null;

  /** Panel mit der Raster-Calculator-GUI. */
  protected RasterCalculatorPanel rasterCalculatorPanel = null;

  /**
   * Erzeugt eine neue GUI fuer den {@link RasterCalculator}.
   * @param appl Instanz der Xulu-Applikation, in der der Rechner
   *             ausgefuehrt wird
   */
  public RasterCalculatorGUI(XuluModellingPlatform appl) {
    super("Raster calculator");
    this.setVisible( false );
    this.setSize( new Dimension(500,400) );
    this.setClosable( true );
    this.setDefaultCloseOperation( HIDE_ON_CLOSE );
    this.getContentPane().setLayout( new BorderLayout() );
    contentPane  = this.getContentPane();
    rasterCalculatorPanel = new RasterCalculatorPanel(appl);
    contentPane.add( rasterCalculatorPanel, BorderLayout.CENTER );
  }

  /**
   * Aktualisiert die Auswahl-Felder auf Basis des Datenpools.
   */
  public void refresh() {
    rasterCalculatorPanel.refresh();
  }
}
