/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.appl;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import schmitzm.data.RasterCalculator;
import schmitzm.data.RasterFilter;
import schmitzm.data.WritableGrid;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.property.ListProperty;
import schmitzm.data.property.ListPropertyReadAccess;
import schmitzm.data.property.MatrixProperty;
import schmitzm.data.property.Properties;
import schmitzm.data.property.Property;
import schmitzm.data.property.PropertyReadAccess;
import schmitzm.data.property.ScalarProperty;
import schmitzm.swing.InputOption;
import schmitzm.swing.ManualInputOption;
import schmitzm.swing.MultipleOptionPane;
import schmitzm.swing.SelectionInputOption;
import schmitzm.swing.table.AbstractMutableTableModel;
import schmitzm.swing.table.MutableTable;
import schmitzm.swing.table.MutableTableModel;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.DataPool;
import edu.bonn.xulu.data.DynamicXuluObject;
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.gui.XuluGUIMessages;
import edu.bonn.xulu.gui.XuluGUIUtil;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.plugin.data.grid.MultiGrid;
import edu.bonn.xulu.plugin.data.grid.SingleGrid;
import schmitzm.swing.OperationTreePanel;
import schmitzm.swing.SwingUtil;

// nur fuer Doku
import schmitzm.data.RasterOperationTree;
import schmitzm.data.RasterOperationTreeParser;
import edu.bonn.xulu.plugin.appl.RasterCalculatorPlugin;
import java.awt.event.MouseAdapter;
import javax.swing.JTextField;
import java.awt.event.MouseEvent;

/**
 * Diese Klasse stellt eine GUI zur Vefuegung, mit der eine einfache arithmetische
 * (und boolsche) Berechnung auf Rasterdaten durchgefuehrt werden kann. Zudem
 * koennen in die Formel Filter eingebunden werden.
 * @see RasterCalculator
 * @see RasterCalculatorPlugin
 * @see RasterOperationTree
 * @see RasterOperationTreeParser
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class RasterCalculatorPanel extends OperationTreePanel {
  private final Component THIS = this;
  /** Name der Datei, in der die Standard-Filter hinterlegt sind. */
  public static final String DEFAULT_FILTER_FILENAME = "default_filter.inp";

  /** Konstante fuer das Label zur Tabelle der Eingabe-Raster.
  *  @see #layoutConstraints */
  public static final String INRASTER_LABEL = "inputRasterLabel";
  /** Konstante fuer die Tabelle der Eingabe-Raster.
  *  @see #layoutConstraints */
  public static final String INRASTER_TABLE = "inputRasterTable";
  /** Konstante fuer das Label zur Tabelle der Eingabe-Filter.
  *  @see #layoutConstraints */
  public static final String INFILTER_LABEL = "inputFilterLabel";
  /** Konstante fuer die Tabelle mit der Eingabe-Filter.
  *  @see #layoutConstraints */
  public static final String INFILTER_TABLE = "inputFilterTable";
  /** Konstante fuer die ComboBox des Ausgabe-Rasters.
  *  @see #layoutConstraints */
  public static final String OUTRASTER_FIELD = "outputRasterField";
  /** Konstante fuer den Button zum Laden der Default-Filter.
  *  @see #layoutConstraints */
  public static final String LOADFILTER_BUTTON = "filterButton";

  /** Speichert eine Referenz auf die Xulu-Applikation, in der der Code-Generator
   *  ausgefuehrt wird. */
  protected XuluModellingPlatform appl = null;
  /** Tabelle in der die Eingabe-Raster angegeben werden. */
  protected MutableTable          inputRasterTable = null;
  private   InputRasterTableModel inputRasterTableModel = null;
  private   JScrollPane           inputRasterScrollPane = null;
  /** Tabelle in der die Eingabe-Filter angegeben werden. */
  protected MutableTable          inputFilterTable = null;
  private   InputFilterTableModel inputFilterTableModel = null;
  private   JScrollPane           inputFilterScrollPane = null;
  /** Auswahlfeld fuer das Ausgabe-Raster. */
  protected SelectionInputOption.Combo outputRaster = null;
  /** Butten zum Laden von Standard-Filtern. */
  protected JButton loadDefaultFilterButton = null;
  /** Speichert die Komponente, von der eine Drag-Aktion ausgeht. */
  private   Component dragSourceComponent = null;

  /**
   * Erzeugt eine neue GUI fuer den {@link RasterCalculator}.
   * @param appl Instanz der Xulu-Applikation, in der der Rechner
   *             ausgefuehrt wird
   */
  public RasterCalculatorPanel(XuluModellingPlatform appl) {
    this(appl,true);
  }

  /**
   * Erzeugt eine neue GUI fuer den {@link RasterCalculator}.
   * @param appl Instanz der Xulu-Applikation, in der der Rechner
   *             ausgefuehrt wird
   * @param initGUI Flag, ob {@link #initGUI()} am Ende des Konstruktor
   *        aufgerufen werden soll (wenn {@code false} muss die explizit
   *        durch die Unterklasse erfolgen!)
   */
  protected RasterCalculatorPanel(XuluModellingPlatform appl, boolean initGUI) {
    super(false);
    this.appl = appl;

    // Zusaetzliche Operatoren und Konstanten
    avOperators.add("X");            avOperatorsDesc.put("X",        "Raster cell X");
    avOperators.add("Y");            avOperatorsDesc.put("Y",        "Raster cell Y");
    avOperators.add("NoData");       avOperatorsDesc.put("NoData",   "NoData value");
    avOperators.add("isNoData");     avOperatorsDesc.put("isNoData", "isNoData(.)");

    // Layout-Anordnung fuer GUI
    layoutConstraints.put( INRASTER_LABEL,   new GridBagConstraints(0,0,2,1,  0,  0,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(5,10,0,10),0,0) );
    layoutConstraints.put( INRASTER_TABLE,   new GridBagConstraints(0,1,2,1,1.0,1.0,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,10,5,10),0,0) );
    layoutConstraints.put( INFILTER_LABEL,   new GridBagConstraints(0,2,2,1,  0,  0,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE, new Insets(5,10,0,10),0,0) );
    layoutConstraints.put( INFILTER_TABLE,   new GridBagConstraints(0,3,2,1,1.0,1.0,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0,10,5,10),0,0) );
    layoutConstraints.put( RULE_TEXTFIELD,   new GridBagConstraints(0,4,1,1,1.0,  0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0,10,5,10),0,0) );
    layoutConstraints.put( OPERATOR_COMBOBOX,new GridBagConstraints(1,4,1,1,0.0,  0,GridBagConstraints.SOUTHEAST, GridBagConstraints.HORIZONTAL, new Insets(0,10,5,10),2,2) );
    layoutConstraints.put( OUTRASTER_FIELD,  new GridBagConstraints(0,5,2,1,1.0,  0,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0,10,5,10),2,2) );
    layoutConstraints.put( START_BUTTON,     new GridBagConstraints(0,6,1,1,0.0,  0,GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,10,5,10),0,0) );
    layoutConstraints.put( LOADFILTER_BUTTON,new GridBagConstraints(1,6,1,1,0.0,  0,GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0,10,5,10),0,0) );

    if ( initGUI ) {
      initGUI();
      refresh();
    }

    // Listener auf den Datenpool setzen, damit Auswahlfelder automatisch
    // aktualisiert werden
    appl.getDataPool().addObjectListener( new ObjectListener() {
      public void performObjectEvent(ObjectEvent e) {
        if ( e instanceof DataPool.DataPoolChangeEvent )
          refresh();
      }
    });
  }

  /**
   * Initalisiert die GUI des Fensters.
   */
  protected void initGUI() {
    super.initGUI();
    // Input-Raster-Tabelle
    inputRasterTableModel = new InputRasterTableModel(appl);
    inputRasterTable      = new InputTable(inputRasterTableModel,"Simply drag raster objects from xulu data pool to this table");
    inputRasterScrollPane = inputRasterTable.createScrollPane();
    // Input-Filter-Tabelle
    inputFilterTableModel = new InputFilterTableModel(appl);
    inputFilterTable      = new InputTable(inputFilterTableModel,"Simply drag matrix properties from xulu data pool to this table");
    inputFilterScrollPane = inputFilterTable.createScrollPane();
    // Output-Raster
    outputRaster = new SelectionInputOption.Combo("Output raster:",false);
    outputRaster.setToolTipText("Leave this fiels empty, to create an new raster as output");
    // Button zum Laden der Standard-Filter
    loadDefaultFilterButton = new JButton("Load default filter...");
    loadDefaultFilterButton.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        loadDefaultFilter();
      }
    });

// PRUEFEN, VON WELCHER KOMPONENTE DIE DRAG-OPERATION AUSGEHT
// LEIDER WIRD DER DragSourceListener ERST NACH DEM DropTargetListener
// AUSGEFUEHRT
//    // Drag & Drop vom Datenpool
//    DragSource.getDefaultDragSource().addDragSourceListener( new DragSourceAdapter() {
//      public void dragMouseMoved(DragSourceDragEvent e) {
//        System.err.println("dragMoved1");
//        dragSourceComponent = e.getDragSourceContext().getComponent();
//        System.err.println( dragSourceComponent );
//      }
//      public void dragEnter(DragSourceDragEvent e) {
//        System.err.println("dragEnter1");
//
//        dragSourceComponent = e.getDragSourceContext().getComponent();
//        System.err.println( dragSourceComponent );
//      }
//    });

    // Drag & Drop fuer Input-Raster
    DropTarget dropTarget = new DropTarget( appl.getMainFrame(), new DropTargetAdapter() {
      public void dragEnter(DropTargetDragEvent e) {
//        // Drop nur Zulassen, wenn die Drag-Aktion vom Datenpool ausgeht
//        if ( !SwingUtil.isChildComponent( dragSourceComponent, appl.getMainFrame().getDataPoolFrame() ) ) {
//          e.rejectDrop();
//          return;
//        }

        // Pruefen, ob mind, ein akzeptables Objekt im Datenpool selektiert ist
        Object[] selObjects = appl.getMainFrame().getDataPoolFrame().getSelectedObjects();
        for (Object obj : selObjects)
          if ( getWritableGridsFromObject(obj) != null )
            return;
        // Drop zurueckweisen, wenn kein WritableGrid gefunden wurde
        e.rejectDrag();
      }

      public void drop(DropTargetDropEvent e) {
        Object[] selObjects = appl.getMainFrame().getDataPoolFrame().getSelectedObjects();
        // Alle WritableGrid-Objekte der selektieren Datenpool-Objekte ermitteln
        for (Object obj : selObjects) {
          WritableGrid[] grids = getWritableGridsFromObject(obj);
          if ( grids == null )
            continue;
          // WritableGrid-Objekte des Datenpool-Objekts in die Liste einfuegen
          for ( WritableGrid grid : grids )
            try {
              inputRasterTableModel.addRaster(grid);
            } catch (Exception err) {
              XuluGUIMessages.showError(THIS,err);
            }
        }
      }
    } );
    inputRasterTable.setDropTarget( dropTarget );
    inputRasterScrollPane.setDropTarget( dropTarget );

    // Drag & Drop fuer Input-Filter
    dropTarget = new DropTarget( appl.getMainFrame(), new DropTargetAdapter() {
      public void dragEnter(DropTargetDragEvent e) {
//        // Drop nur Zulassen, wenn die Drag-Aktion vom Datenpool ausgeht
//        if ( !SwingUtil.isChildComponent( dragSourceComponent, appl.getMainFrame().getDataPoolFrame() ) ) {
//          e.rejectDrop();
//          return;
//        }

        // Pruefen, ob mind, ein akzeptables Objekt im Datenpool selektiert ist
        Object[] selObjects = appl.getMainFrame().getDataPoolFrame().getSelectedObjects();
        for (Object obj : selObjects)
          if ( getMatrixPropertiesFromObject(obj) != null )
            return;
        // Drop zurueckweisen, wenn keine Matrix gefunden wurde
        e.rejectDrag();
      }

      public void drop(DropTargetDropEvent e) {
        Object[] selObjects = appl.getMainFrame().getDataPoolFrame().getSelectedObjects();
        // Alle MatrixProperty-Objekte der selektieren Datenpool-Objekte ermitteln
        for (Object obj : selObjects) {
          MatrixProperty[] matrices = getMatrixPropertiesFromObject(obj);
          if ( matrices == null )
            continue;
          // MatrixProperty-Objekte des Datenpool-Objekts in die Liste einfuegen
          for (MatrixProperty matrix : matrices) {
              try {
                inputFilterTableModel.addFilterMatrix( matrix/*, matrixName*/);
              } catch (Exception err) {
                XuluGUIMessages.showError(SwingUtil.getParentWindowComponent(THIS),err);//,"Matrix "+matrixName+"not suitable for filter!");
              }
          }
        }
      }
    } );
    inputFilterTable.setDropTarget( dropTarget );
    inputFilterScrollPane.setDropTarget( dropTarget );

    // Komponenten in Fenster anordnen
    add( new JLabel("Input-Rasters (use context menu or simply drag rasters from xulu data pool):"),
                                  layoutConstraints.get(INRASTER_LABEL) );
    add( inputRasterScrollPane,   layoutConstraints.get(INRASTER_TABLE) );
    add( new JLabel("Input-Filter (use context menu or simply drag matrixes from xulu data pool):"),
                                  layoutConstraints.get(INFILTER_LABEL) );
    add( inputFilterScrollPane,   layoutConstraints.get(INFILTER_TABLE) );
    add( outputRaster,            layoutConstraints.get(OUTRASTER_FIELD) );
    add( loadDefaultFilterButton, layoutConstraints.get(LOADFILTER_BUTTON) );
  }

  /**
   * Aktualisiert die Auswahl-Felder auf Basis des Datenpools.
   */
  public void refresh() {
    // Aktualisierung der Tabelle
    inputRasterTableModel.refreshInputOption();
    inputRasterTableModel.fireTableDataChanged();

    // Als Ausgabe-Raster werden alle WritableGrids aus dem Datenpool angeboten
    Object oldSelection = outputRaster.getValue();
    Object[][] fitObjects = XuluGUIUtil.determineObjectsFromDataPool(WritableGrid.class, appl.getDataPool());
    outputRaster.setSelectionObjects(fitObjects[0],fitObjects[1]);
    outputRaster.setSelectedItem(oldSelection);
  }

  /**
   * Prueft die in der Maske spezifizierten Angaben auf Korrektheit. Ist eine
   * Angabe nicht korrekt, wird ein entsprechender Fehler-Dialog angezeigt.
   */
  protected void checkInputsAndError() {
    if ( !outputRaster.isInputValid() )
      throw new IllegalArgumentException("Ausgabe-Raster nicht zulaessig!");
    if ( !rule.isInputValid() )
      throw new IllegalArgumentException("Formel nicht zulaessig!");
    RasterCalculator.checkRuleAndError(rule.getValue());
  }


  /**
   * Liefert die Anzahl an geklammerten Parametern, die ein Operator hat.
   * Erweitert die Methode der Oberklasse um den Operator {@code isNoData(.)}
   * und die Filter-Referenz {@code F.}.
   * @param op Operator
   * @return 1 fuer {@code isNoData} und {@code F..}, sonst den Wert der {@code super}-Methode
   */
  protected int getParameterCount(String op) {
    // Funktionen mit einem geklammerten Parameter
    if (op.equalsIgnoreCase("isNoData") || op.charAt(0) == 'F')
      return 1;
    return super.getParameterCount(op);
  }

  /**
   * Laed Standard-Filter aus der Datei {@code <Plugin-Dir>\RasterCalculator\default_filter.inp}
   * in den Datenpool.
   * @see #DEFAULT_FILTER_FILENAME
   */
  private void loadDefaultFilter() {
    try {
      // (eindeutige) Import-Factory fuer Dynamische Xulu-Objekte ermitteln,
      // die aus einem File oder FileInputStream importieren kann
      ImportFactory[] factories = appl.getRegistry().getImportFactories( DynamicXuluObject.class );
      if ( factories.length < 1 )
        throw new UnsupportedOperationException("No import factory for DynamicXuluObject found in registry!");
      if ( factories.length > 1 )
        throw new UnsupportedOperationException("More than one import factories for DynamicXuluObject found in registry!");
      ImportFactory importFac = factories[0];
      if ( !importFac.getImportSourceType().isAssignableFrom(File.class) &&
           !importFac.getImportSourceType().isAssignableFrom(FileInputStream.class) )
        throw new UnsupportedOperationException("Registrated import factories for DynamicXuluObject can neither import from File nor from FileInputStream!");

      // Dynamisches Xulu-Objekt importieren
      File   importFile = new File( appl.getPluginDirectory(), "RasterCalculator"+File.separator+DEFAULT_FILTER_FILENAME );
      Object importObject = null;
      if ( importFac.getImportSourceType().isAssignableFrom(File.class) )
        importObject = importFile;
      if ( importFac.getImportSourceType().isAssignableFrom(FileInputStream.class) )
        importObject = new FileInputStream(importFile);
      DynamicXuluObject dynXuluObj = (DynamicXuluObject)importFac.importObject(importObject,appl.getRegistry());

      // Alle Objekte aus dem Datenpool entfernen, die den gleichen Namen
      // haben, wie das neuen Objekt
      XuluObject[] presentObj = appl.getDataPool().getObjects( dynXuluObj.getDescription() );
      if ( presentObj.length > 0 ) {
        if ( MultipleOptionPane.showConfirmDialog(this,"Override existing data pool object(s) '"+dynXuluObj.getDescription()+"'?","Override...",MultipleOptionPane.OK_CANCEL_OPTION) == MultipleOptionPane.CANCEL_OPTION )
          return;
        for (XuluObject xuluObj : presentObj)
          appl.getDataPool().removeObject(xuluObj);
      }

      // Neues Objekt dem Datenpool hinzufuegen
      appl.getDataPool().addObject( dynXuluObj );
      // Filter direkt in die Filter-Liste einfuegen
      if ( MultipleOptionPane.showConfirmDialog(this,"Add loaded filters automatically to filter list?","Add filter...",MultipleOptionPane.OK_CANCEL_OPTION) == MultipleOptionPane.CANCEL_OPTION )
        return;
      for ( Property matrix : dynXuluObj.getProperties(MatrixProperty.class).getProperties() )
        inputFilterTableModel.addFilterMatrix( (MatrixProperty)matrix );

    } catch (Exception err) {
      XuluGUIMessages.showError(this,err);
    }
  }

  /**
   * Erzeugt einen {@link RasterOperationTree} aus der Formel und wertet diese
   * aus. Das Ergebnis wird in das Ausgabe-Raster geschrieben
   * @return das Ausgabe-Raster, als {@link WritableGrid}-Wert
   * @see RasterOperationTreeParser
   */
  protected Object performCalculation() throws Exception {
    // Eingabe-Raster als WritableGrid-Array
    WritableGrid[] inGrid = inputRasterTableModel.getRasters();
    // Eingabe-Filter als WritableGrid-Array
    RasterFilter[] inFilter = inputFilterTableModel.getFilter();

    // Ausgabe-Raster ermitteln (wenn keines angegeben, wird eines erzeugt)
    WritableGrid outGrid = null;
    Object outGridValue = outputRaster.getValue();
    if (outGridValue == null) {
      // Namen fuer Raster abfragen
      Object[] retObject = MultipleOptionPane.showMultipleInputDialog( SwingUtil.getParentWindowComponent(THIS), "Create new Object...", new InputOption[] {
          new ManualInputOption.Text("Raster name:", true, "RasterCalculator OUT")
      });
      if ( retObject == null )
        return null; // Dialog abgebrochen
      String rasterName = (String)retObject[0];
      SingleGrid singleGrid = null;
      if ( inGrid.length > 0 )
        // Ausgabe-Grid auf Basis des Eingabe-Rasters erzeugen
        singleGrid = ((SingleGrid)appl.getRegistry().getDefaultFactory( SingleGrid.class ).newInstance(false,inGrid[0]));
      else
        // Dimension des Ausgabe-Grid abfragen
        singleGrid = ((SingleGrid)appl.getRegistry().getDefaultFactory( SingleGrid.class ).newInstance(true));
      singleGrid.setDescription( rasterName );
      appl.getDataPool().addObject(singleGrid);
      outGrid = singleGrid.getGrid();
      outputRaster.setValue( outGrid );
    } else
      outGrid = ( outGridValue instanceof SingleGrid ) ? ((SingleGrid)outGridValue).getGrid() : (WritableGrid)outGridValue;
    RasterCalculator.calculate(rule.getValue(), inGrid, inFilter, outGrid);
    return outGrid;
  }

  private static WritableGrid[] getWritableGridsFromObject(Object obj) {
    if ( obj instanceof SingleGrid )
      obj = ((SingleGrid)obj).getProperty( SingleGrid.PROP_GRID );
    if ( obj instanceof MultiGrid )
      obj = ((MultiGrid)obj).getProperty( MultiGrid.PROP_GRIDS );

    if ( obj instanceof WritableGrid )
      return new WritableGrid[] { (WritableGrid)obj };
    if ( obj instanceof ScalarProperty && WritableGrid.class.isAssignableFrom( ((ScalarProperty)obj).getType() ) )
      return new WritableGrid[] { (WritableGrid)((ScalarProperty)obj).getOneTimeReadAccess().getValue() };
    if ( obj instanceof ListProperty && WritableGrid.class.isAssignableFrom( ((ListProperty)obj).getType() ) ) {
      ListPropertyReadAccess a = ((ListProperty)obj).getReadAccess(obj);
      WritableGrid[] grids = new WritableGrid[ a.getCount() ];
      for (int i=0; i<grids.length; i++)
        grids[i] = (WritableGrid)a.getValue(i);
      a.release();
      return grids;
    }
    return null;
  }

  private static MatrixProperty[] getMatrixPropertiesFromObject(Object obj) {
    if ( obj instanceof MatrixProperty )
      return new MatrixProperty[] { (MatrixProperty)obj };
    if ( obj instanceof Properties ) {
      Property[] props = ((Properties)obj).getProperties(MatrixProperty.class).getProperties();
      MatrixProperty[] matrices = new MatrixProperty[props.length];
      for (int i=0; i<props.length; i++ )
        matrices[i] = (MatrixProperty)props[i];
      return matrices;
    }
    return null;
  }

  ///////////////////////////////////////////////////////////////////
  /////////////////   Table fuer die DnD-Tabellen   /////////////////
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt eine Tabelle fuer die Eingabe-Daten dar. Diese hat
   * keinen Header und es wird kein Tabellen-Raster angezeigt. Ueber ein
   * Kontextmenue koennen der Tabelle Daten hinzugefuegt oder entfernt werden.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class InputTable extends MutableTable {
    /**
     * Erzeugt einen neue Tabelle
     * @param tableModel Daten-Modell fuer die Tabelle
     * @param toolTip Hilfe-Text der als ToolTip-Text angezeigt wird
     */
    public InputTable(InputTableModel tableModel, String toolTip) {
      super( tableModel, MutableTable.ITEM_REMOVE | MutableTable.ITEM_ADD );
      this.setToolTipText( toolTip );
      this.setTableHeader(null);
      this.setShowGrid(false);
      this.getColumnModel().getColumn(0).setMaxWidth(30);
      // Bei Doppelklick auf Tabelle wird der Wert aus Spalte 0 in die Formel
      // eingefuegt
      this.addMouseListener( new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          // Nur auf Doppelklick reagieren
          if ( e.getButton() != MouseEvent.BUTTON1 ||
               e.getClickCount() != 2 )
            return;
          // Selektierte Zeile ermitteln
          String selValue = (String)getModel().getValueAt(
              getSelectedRow(),
              0
          );
          // Attribut-Namen in Formel einfuegen
          performOperatorInsert(selValue, (JTextField)rule.getInputComponent());
          // Fokus zurueck auf Formel-Feld
          rule.grabFocus();
        }
      });
    }

    /**
     * Erzeugt einen neue Tabelle
     * @param tableModel Daten-Modell fuer die Tabelle
     */
    public InputTable(InputTableModel tableModel) {
      this(tableModel,"");
    }
  }

  ///////////////////////////////////////////////////////////////////
  //////////////   TableModels fuer die DnD-Tabellen   //////////////
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt das Daten-Modell der Eingabe-Tabellen dar. Dieses
   * stellt immer 2 Spalten zur Verfuegung. In der ersten Spalte wird die
   * Zeilennummer (eingeleitet durch ein {@linkplain #rowIndexPrefix Praefix})
   * dargestellt, in der zweiten Spalte ein Objekt vom Typ {@code <O>}.
   * Durch einen Rechtsklick, koennen der Tabelle Objekte aus dem Datenpool
   * hinzugefuegt oder entfernt werden. Dabei werden die Objekte aus dem
   * Array {@code #fitObjects} (bzw. {@code fitObjectNames}) vorgeblendet.
   * Diese Array passend zu fuellen muss durch die Sub-Klassen in der Methode
   * {@link #determineFitObjectsAndNames()} implementiert werden.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private abstract class InputTableModel<O> extends AbstractMutableTableModel {
    /** Speichert die in der Tabelle gespeicherten Raster */
    protected Vector<O> objects = new Vector<O>();
    /** Speichert die Namen der in der Tabelle gespeicherten Objekte */
    protected Vector<String> objectNames = new Vector<String>();
    /** Speichert die Instanz der Xulu-Applikation. */
    protected XuluModellingPlatform appl = null;
    /** Eingabe-Option fuer ein einzelnes Objekt. */
    protected SelectionInputOption.Combo objectInputOption = null;

    /** Speichert das Zeilennummer-Praefix. */
    protected String rowIndexPrefix = null;


    /** Flag, ob sich der Inhalt des Datenpools geaendert hat und die Arrays
     *  {@link #fitObjects} und {@link #fitObjectNames} aktualisiert werden
     *  muessen */
    protected boolean  dataPoolChanged = true;
    /** Passende Objekte im Datenpool */
    protected Object[] fitObjects = null;
    /** Bezeichnungen der passenden Objekte im Datenpool */
    protected String[] fitObjectNames = null;

    /**
     * Erzeugt ein neues Tabellen-Modell.
     * @param appl Instanz der Xulu-Applikation
     * @param rowIndexPrefix Praefix, das den Zeilen-Indizes vorangestellt wird
     */
    public InputTableModel(XuluModellingPlatform appl, String rowIndexPrefix) {
      super();
      this.appl = appl;
      this.rowIndexPrefix = rowIndexPrefix;
      objectInputOption = new SelectionInputOption.Combo(getColumnName(1),true);
      refreshInputOption();
      // Listener auf den Datenpool setzen, damit das Auswahlfeld aktualisiert
      // werden kann (wenn notwendig)
      appl.getDataPool().addObjectListener( new ObjectListener() {
        public void performObjectEvent(ObjectEvent e) {
          if ( e instanceof DataPool.DataPoolChangeEvent ) {
            dataPoolChanged = true;
          }
        }
      });
    }

    /**
     * Ruft {@link #determineFitObjectsAndNames()}, wenn das {@link #dataPoolChanged}-Flag
     * auf {@code true} steht und setzt das Flag zurueck.
     */
    protected void refreshFitObjects() {
      if ( dataPoolChanged )
        determineFitObjectsAndNames();
      dataPoolChanged = false;
    }

    /**
     * Aktualisiert das Dialog-Auswahlfeld, in dam alle passenden Objekte
     * aus dem Datenpool vorgeblendet werden.
     */
    protected void refreshInputOption() {
      refreshFitObjects();
      objectInputOption.setSelectionObjects(fitObjects,fitObjectNames);
    }

    /**
     * Wird aufgerufen, wenn sich der Inhalt des Xulu-Datenpools aendert.
     * Muss die Arrays {@link #fitObjects} und {@link #fitObjectNames} mit
     * allen passenden Objekten des Datenpools und deren Bezeichnungen
     * befuellen.
     */
    protected abstract void determineFitObjectsAndNames();


    /**
     * Loescht eine Tabellenzeile.
     * @param row Zeilenindex (beginnend bei 0)
     */
    public void performRemoveRow(int row) {
      objects.remove(row);
      objectNames.remove(row);
    }

    /**
     * Macht nichts, da in der Tabelle keine Daten veraendert werden koennen.
     * @param row Zeilenindex (beginnend bei 0)
     * @param col Spaltenindex (beginnend bei 0)
     */
    public void performChangeData(int row, int col) {
    }

    /**
     * Liefert die Anzahl an Zeilen.
     */
    public int getRowCount() {
      return objects.size();
    }

    /**
     * Liefert den Inahlt einer Tabellenzelle.
     * @param row Zeilenindex (beginnend bei 0)
     * @param col Spaltenindex (beginnend bei 0)
     */
    public Object getValueAt(int row, int col) {
      switch ( col ) {
        case 0: return rowIndexPrefix+row;
        case 1: return objectNames.elementAt(row);
      }
      return null;
    }

  }

  /**
   * Diese Klasse stellt das Daten-Modell der Eingaberaster-Tabelle dar.
   * Durch einen Rechtsklick, koennen der Tabelle {@link WritableGrid}-Objekte
   * aus dem Datenpool hinzugefuegt oder entfernt werden.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class InputRasterTableModel extends InputTableModel<WritableGrid> {
    /**
     * Erzeugt ein neues Tabellen-Modell.
     */
    public InputRasterTableModel(XuluModellingPlatform appl) {
      super(appl,"#");
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {"Index","Raster-Object"};
    }
    
    /**
     * Befuellt die Arrays {@link #fitObjects} und {@link #fitObjectNames} mit
     * allen {@link WritableGrid}- und {@link MultiGrid}-Objekten des
     * Datenpools und deren (rekursiven) Bezeichnungen.
     */
    protected void determineFitObjectsAndNames() {
      Vector fitRasterObjects = new Vector();
      Vector fitRasterObjectNames = new Vector();
      XuluGUIUtil.determineObjectsFromDataPool(MultiGrid.class, appl.getDataPool(), fitRasterObjects, fitRasterObjectNames);
      XuluGUIUtil.determineObjectsFromDataPool(WritableGrid.class, appl.getDataPool(), fitRasterObjects, fitRasterObjectNames);
      this.fitObjects     = fitRasterObjects.toArray();
      this.fitObjectNames = (String[])fitRasterObjectNames.toArray(new String[0]);
    }

    /**
     * Liefert die in der Tabelle angezeigten Raster als Array.
     */
    public WritableGrid[] getRasters() {
      return (WritableGrid[])objects.toArray(new WritableGrid[0]);
    }

    /**
     * Zeigt einen Dialog an, in dem alle {@link WritableGrid}- und
     * {@link MultiGrid}-Objekte des Datenpools zur Auswahl stehen.
     */
    public void performAddRow() {
      refreshFitObjects();
      Object[] value = MultipleOptionPane.showMultipleInputDialog(getParent(),"Select raster...",objectInputOption);
      if ( value == null )
        return;

      Object selObject = objectInputOption.getValue();
      String selName   = (String)objectInputOption.getSelectedDisplayItem();
      if ( selObject instanceof WritableGrid)
        addRaster((WritableGrid)selObject,selName);
      if ( selObject instanceof MultiGrid ) {
        MultiGrid multiGrid = (MultiGrid)selObject;
        for (int i=0; i<multiGrid.getGridCount(); i++)
          addRaster(multiGrid.getGrid(i),selName+"["+i+"]");
      }
    }

    /**
     * Fuegt der Tabelle ein Raster hinzu.
     * @param raster Raster
     * @param name   Name fuer das Raster, der in der Tabelle angezeigt wird
     */
    public void addRaster(WritableGrid raster, String name) {
      refreshFitObjects();
      // Pruefen, ob Raster mit den anderen Rastern uebereinstimmt
      if ( this.objects.size() > 0 ) {
        WritableGrid presentGrid = this.objects.elementAt(0);
        if ( raster.getWidth() != presentGrid.getWidth() ||
             raster.getHeight() != presentGrid.getHeight() )
          throw new UnsupportedOperationException("New Raster does not fit to the other rasters in the list!");
      }
      this.objects.add(raster);
      this.objectNames.add(name);
      this.fireTableDataChanged();
    }

    /**
     * Fuegt der Tabelle ein Raster hinzu. Der Name des Objekts wird automatisch
     * aus dem Datenpool ermittelt.
     * @param raster Raster
     */
    public void addRaster(WritableGrid raster) {
      refreshFitObjects();
      String rasterName = "dummy";
      for (int i=0; i<fitObjects.length; i++)
        if ( raster == fitObjects[i] ) {
          rasterName = fitObjectNames[i];
          break;
        }
      addRaster(raster,rasterName);
    }

  }

  /**
   * Diese Klasse stellt das Daten-Modell der Eingabefilter-Tabelle dar.
   * Durch einen Rechtsklick, koennen der Tabelle {@link MatrixProperty}-Objekte
   * aus dem Datenpool hinzugefuegt oder entfernt werden.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class InputFilterTableModel extends InputTableModel<MatrixProperty> {

    /**
     * Erzeugt ein neues Tabellen-Modell.
     */
    public InputFilterTableModel(XuluModellingPlatform appl) {
      super(appl,"F");
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {"Index","Filter-Object"};
    }

    /**
     * Befuellt die Arrays {@link #fitObjects} und {@link #fitObjectNames} mit
     * allen {@link MatrixProperty}-Objekten des Datenpools und deren
     * (rekursiven) Bezeichnungen.
     */
    protected void determineFitObjectsAndNames() {
      Object[][] fitMatrixObjects = XuluGUIUtil.determineObjectsFromDataPool(MatrixProperty.class, appl.getDataPool());
      this.fitObjects     = fitMatrixObjects[0];
      this.fitObjectNames = (String[])fitMatrixObjects[1];
    }

    /**
     * Liefert die in der Tabelle angezeigten Matrix-Objekte Filter-Array.
     */
    public RasterFilter[] getFilter() {
      RasterFilter[] filter = new RasterFilter[objects.size()];
      for (int i=0; i<objects.size(); i++) {
        MatrixProperty matrix = objects.elementAt(i);
        filter[i] = new RasterFilter(matrix.getSize(0), matrix.getSize(1));
        PropertyReadAccess access = matrix.getReadAccess(this);
        for (int y=0; y<filter[i].getFilterSizeY(); y++)
          for (int x=0; x<filter[i].getFilterSizeX(); x++)
            filter[i].setFilterValue(access.getValueAsDouble(x, y), x, y);
        access.release();
      }
      return filter;
    }

    /**
     * Zeigt einen Dialog an, in dem alle {@link MatrixProperty}-Objekte
     * des Datenpools zur Auswahl stehen.
     */
    public void performAddRow() {
      Object[] value = MultipleOptionPane.showMultipleInputDialog(getParent(),"Select matrix...",objectInputOption);
      if ( value == null )
        return;

      Object selObject = objectInputOption.getValue();
      String selName   = (String)objectInputOption.getSelectedDisplayItem();
      if ( selObject instanceof MatrixProperty )
        try {
          addFilterMatrix( (MatrixProperty) selObject, selName);
        } catch (Exception err) {
          XuluGUIMessages.showError(SwingUtil.getParentWindowComponent(THIS),err,"Matrix '"+selName+"' not suitable for filter!");
        }
    }

    /**
     * Fuegt der Tabelle einen Filter hinzu.
     * @param filterMatrix Filter
     * @param name   Name fuer das Raster, der in der Tabelle angezeigt wird
     */
    public void addFilterMatrix(MatrixProperty filterMatrix, String name) {
      refreshFitObjects();
      // Pruefen, ob Matrix als Filter in Frage kommt
      if ( filterMatrix.getDimension() != 2 )
        throw new UnsupportedOperationException("Only a 2-dimensional matrix can be used as filter!");
      if ( filterMatrix.getSize(0)%2 != 1 || filterMatrix.getSize(1)%2 != 1 )
        throw new UnsupportedOperationException("To use a matrix as filter its size must be odd in each dimension!");
      this.objects.add(filterMatrix);
      this.objectNames.add(name);
      this.fireTableDataChanged();
    }

    /**
     * Fuegt der Tabelle einen Filter hinzu. Der Name des Objekts wird automatisch
     * aus dem Datenpool ermittelt.
     * @param filterMatrix Filter
     */
    public void addFilterMatrix(MatrixProperty filterMatrix) {
      refreshFitObjects();
      String filterMatrixName = "dummy";
      for (int i=0; i<fitObjects.length; i++)
        if ( filterMatrix == fitObjects[i] ) {
          filterMatrixName = fitObjectNames[i];
          break;
        }
      addFilterMatrix(filterMatrix,filterMatrixName);
    }

  }

}
