/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.data.feature;

import org.geotools.feature.FeatureCollection;

import schmitzm.data.property.ScalarProperty;
import edu.bonn.xulu.data.AbstractXuluObject;

/**
 * Dieser Datentyp stellt eine Sammlung von Features dar. Er baut auf der
 * Klasse {@link FeatureCollection org.geotools.feature.FeatureCollection}
 * auf und hat folgende Eigenschaften:<br>
 * <ol>
 * <li>{@linkplain #PROP_FEATURES Features}: die {@link FeatureCollection} (ScalarProperty[FeatureCollection])</li>
 * <li>{@linkplain #PROP_STYLE Style}: Darstellungs-Style fuer die Features (ScalarProperty[Object])</li>
 * </ol>
 * <br>
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class SingleFeatureCollection extends AbstractXuluObject {
  /** Name der Eigenschaft, die die {@link FeatureCollection} beinhaltet. */
  public static final String PROP_FEATURES = "FEATURES";

  /**
   * Erzeugt eine neue <code>SingleFeatureCollection</code>.
   * Die Eigenschaft FEATURES ist noch <b>nicht</b> mit einer konkreten
   * {@link FeatureCollection} belegt.
   */
  public SingleFeatureCollection() {
    // Beim Freigeben der Property soll auch das Grid zerstoert werden
    propSet.addProperty(new ScalarProperty(PROP_FEATURES, FeatureCollection.class) {
      public void dispose() {
        if ( value != null )
          ((FeatureCollection)value).clear();
        super.dispose();
      }
    });
    // Property fuer den Style
    propSet.addProperty( new ScalarProperty(PROP_STYLE,Object.class) );
  }

  /**
   * Erzeugt eine neue <code>SingleFeatureCollection</code>.
   * @param fc Wert fuer die Eigenschaft FEATUERES
   */
  public SingleFeatureCollection(FeatureCollection fc) {
    this();
    setFeatureCollection(fc);
  }

  /**
   * Liefert eine Referenz auf die {@link FeatureCollection}, die in der
   * Eigenschaft FEATURES gespeichert ist.
   */
  public FeatureCollection getFeatureCollection() {
    return (FeatureCollection) ((ScalarProperty)getProperty(PROP_FEATURES)).getOneTimeReadAccess().getValue();
  }

  /**
   * Setzt die {@link FeatureCollection}, die in der Eigenschaft FEATURES
   * gespeichert ist.
   */
  public void setFeatureCollection(FeatureCollection fc) {
    ( (ScalarProperty) getProperty(PROP_FEATURES)).getOneTimeWriteAccess().setValue(fc);
  }
}
