/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JScrollPane;
import javax.swing.JButton;
//import javax.swing.JTable;
//import javax.swing.table.AbstractTableModel;

import schmitzm.swing.table.AbstractMutableTableModel;
import schmitzm.swing.table.ComponentRenderer;
import schmitzm.swing.table.TableComponentMouseListener;
import schmitzm.swing.table.MutableTable;
import schmitzm.swing.table.MutableTableModel;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.ObjectEvent;

import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.appl.AbstractManager;
import edu.bonn.xulu.appl.AbstractManager.ChangeEvent;
import edu.bonn.xulu.gui.AbstractManagerFrame;
import edu.bonn.xulu.gui.XuluGUIMessages;

/**
 * Diese Klasse stellt eine GUI-Komponente fuer den {@link AbstractManager}
 * dar. In dieser werden die darin registrierten Komponenten tabellarisch
 * aufgelistet:
 * <ul>
 * <li>Spalte 1: Titel der Komponente</li>
 * <li>Spalte 2: Klassenname der Komponente</li>
 * <li>Spalte 3: Button zum Anzeigen der Komponente</li>
 * </ul>
 * Mit einem Doppelklick auf einen Eintrag wird das jeweilige Fenster angezeigt.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class AbstractManagerFrame_BasicTable extends AbstractManagerFrame {
  private MutableTable compTable = null;

  /**
   * Erzeugt einen neue Instanz des grafischen Manager-Fensters.
   * Die Groesse ist auf 300x350 Pixel voreingestellt.
   * @param title   Bezeichnung fuer das Fenster
   * @param manager Manager der im Fenster dargestellt wird
   */
  public AbstractManagerFrame_BasicTable(String title, AbstractManager manager) {
    this(title, new AbstractManagerTableModel(manager));
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Manager-Fensters.
   * @param title   Bezeichnung fuer das Fenster
   * @param manager Manager der im Fenster dargestellt wird
   * @param size    Groesse des Fensters.
   */
  public AbstractManagerFrame_BasicTable(String title, AbstractManager manager, Dimension size) {
    this(title,new AbstractManagerTableModel(manager),size);
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Manager-Fensters.
   * Die Groesse ist auf 300x350 Pixel voreingestellt.
   * @param title             Bezeichnung fuer das Fenster
   * @param managerTableModel Datenmodell fuer den Manager der im Fenster dargestellt wird
   */
  public AbstractManagerFrame_BasicTable(String title, AbstractManagerTableModel managerTableModel) {
    this(title, managerTableModel, new Dimension(300,350));
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Manager-Fensters.
   * @param title             Bezeichnung fuer das Fenster
   * @param managerTableModel Datenmodell fuer den Manager der im Fenster dargestellt wird
   * @param size              Groesse des Fensters.
   */
  public AbstractManagerFrame_BasicTable(String title, AbstractManagerTableModel managerTableModel, Dimension size) {
    this(title,managerTableModel,size,MutableTable.ITEM_REMOVE);
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Manager-Fensters.
   * @param title             Bezeichnung fuer das Fenster
   * @param managerTableModel Datenmodell fuer den Manager der im Fenster dargestellt wird
   * @param size              Groesse des Fensters.
   * @param tableMask         Kontextmenue-Optionen, die zur manuellen Modifikation der Tabelle zur
   *                          Verfuegung stehen (vgl. {@link MutableTable})
   */
  protected AbstractManagerFrame_BasicTable(String title, AbstractManagerTableModel managerTableModel, Dimension size, int tableMask) {
    super(title,managerTableModel.manager);
    this.setSize(size);
    this.setLayout( new BorderLayout() );
    // MutableTable implementiert "Aendern" fuer einen Doppelklick.
    // Fuer den AbstractManager soll jedoch das jeweilige Fenster
    // geoeffnet werden -> performDoubleClick(..) ueberschreiben
    this.compTable = new MutableTable( managerTableModel, tableMask ) {
      public void performDoubleClick() {
        AbstractManager.ContentType[] selComp = getSelectedObjects();
        if ( selComp != null ) selComp[0].setVisible(true);
      }
    };
    this.compTable.addMouseListener( new TableComponentMouseListener(compTable) );
    this.compTable.setDefaultRenderer(compTable.getModel().getColumnClass(2),new ComponentRenderer());
    this.compTable.getColumnModel().getColumn(2).setPreferredWidth(70);
    this.compTable.getColumnModel().getColumn(2).setMaxWidth(70);
    this.getContentPane().add( compTable.createScrollPane() );
  }

  /**
   * Aktualisiert die Darstellung des Managers, indem ein
   * <code>fireTableDataChanged()</code>-Event auf das TableModel der
   * Manager-Tabelle ausgeloest wird.
   */
  public void refresh() {
    ((MutableTableModel)compTable.getModel()).fireTableDataChanged();
  }

  /**
   * Liefert die aktuell in der Tabelle selektierte Komponente.
   * In der Tabelle kann immer nur ein Eintrag selektiert werden. Diese
   * Methode liefert also immer einen Array, der nur ein Element enthaelt.
   * @return <code>null</code> falls kein Eintrag selektiert ist
   */
  public AbstractManager.ContentType[] getSelectedObjects() {
    int selCompIdx = compTable.getSelectedRow();
    return selCompIdx >=0 ? new AbstractManager.ContentType[] { manager.get( selCompIdx ) } : null;
  }

  ///////////////////////////////////////////////////////////////////
  // TableModel fuer die Manager-Tabelle
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein TableModel fuer den {@link AbstractManager} dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  public static class AbstractManagerTableModel extends AbstractMutableTableModel implements ObjectListener {
    /** Speichert die Spaltentypen der Tabelle */
    protected Class[] COLUMN_TYPE = {
        String.class,
        String.class,
        JButton.class
    };
    /** Speichert die Datenbasis fuer die Tabelle */
    protected AbstractManager manager = null;

    /**
     * Erzeugt ein neues Daten-Modell fuer einen <code>AbstractManager</code>.
     * @param manager Instanz eines <code>AbstractManager</code>
     */
    public AbstractManagerTableModel(AbstractManager manager) {
      super();
      this.manager = manager;
      manager.addObjectListener( this );
    }

    /**
     * Wird aufgerufen, wenn sich der Inhalt des Managers aendert.
     * Handelt es sich um ein {@link AbstractManager.ChangeEvent}
     * wird ein <code>fireTableDataChanged()</code> ausgeloest, damit der
     * Tabelleninhalt aktualisiert wird.
     */
    public void performObjectEvent(ObjectEvent e) {
      if ( e instanceof AbstractManager.ChangeEvent )
        fireTableDataChanged();
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {
          XuluConstants.XULUGUI_RES.getString("Name"),
          XuluConstants.XULUGUI_RES.getString("Type"),
          ""
      };      
    }
    
    /**
     * Liefert die Anzahl an Tabellenzeilen. Entspricht der Anzahl der
     * Komponenten, die im Manager registriert sind.
     */
    public int getRowCount() {
      return manager.getCount();
    }

    /**
     * Liefert einen Wert der Tabelle. Fuer Spalte 0 wird der jeweilige Titel
     * der Komponente geliefert, fuer Spalte 1 der Klassenname.
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public Object getValueAt(int row, int col) {
      switch( col ) {
        case 0: // erste Spalte: Titel der Komponente
                return manager.get(row).getTitle();
        case 1: // zweite Spalte: Typ der Komponente
                return manager.get(row).getClass().getName();
        case 2: // dritte Spalte: Oeffnen-Button
                JButton openButton = new JButton( XuluConstants.SWING_RES.getString("Open") );
                final int r = row;
                openButton.addMouseListener( new MouseAdapter() {
                  public void mouseClicked(MouseEvent e) {
                    manager.get(r).setVisible(true);
                  }
                });
                return openButton;
      }
      return null;
    }

    /**
     * Prueft, ob eine Tabellen-Zelle editierbar ist. Dies ist fuer alle
     * Zellen der Spalte 0 ("Titel") der Fall.
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public boolean isCellEditable(int row, int col) {
      return col == 0;
    }

    /**
     * Setzt den Titel der jeweiligen Komponente neu,
     * wenn als Spaltennummer 0 angegeben wird. Ansonsten macht diese Methode nichts.
     * @param obj neuer Titel (<code>String</code>)
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public void setValueAt(Object obj, int row, int col) {
      // nur die Spalte mit der Objekt-Beschreibung ist editierbar
      if ( isCellEditable(row,col) )
        manager.get(row).setTitle(obj.toString());
    }


    /**
     * Entfernt einen Eintrag aus dem <code>AbstractManager</code> und
     * somit aus der Tabelle. Wird aufgerufen, wenn der Menuepunkt
     * "Entfernen" aus dem Kontextmenue gewaehlt wird.
     * @param row Tabellenzeile
     * @see AbstractManager#remove(int)
     */
    public void performRemoveRow(int row) {
      manager.remove(row);
    }

    /**
     * Macht nichts, da das Menu fuer den <code>AbstractManager</code>
     * den Aendern-Menuepunkt nicht enthaelt.
     * @param row Tabellenzeile
     * @param col Tabellenspalte
     */
    public void performChangeData(int row, int col) {
    }

    /**
     * Macht nichts, da das Menu fuer den <code>AbstractManager</code>
     * den Hinzufuegen-Menuepunkt nicht enthaelt.
     */
    public void performAddRow() {}

  }
}
