/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JFileChooser;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import java.util.Vector;
import java.util.Enumeration;
import java.io.File;
//import java.io.FileInputStream;
//import java.io.FileOutputStream;

import schmitzm.swing.MultiSplitPane;

import schmitzm.swing.event.PopupMenuListener;

import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.property.Property;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.DataPool;

import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.gui.DataPoolFrame;
import edu.bonn.xulu.gui.DataPoolMenu;
import edu.bonn.xulu.plugin.gui.DisplayContainer_Properties;

/**
 * Dieses Fenster stellt die grafische Implementierung des Xulu-Datenpools
 * dar. Es visualiert den Datenpool tabellarisch durch zwei Komponenten:
 * <ul>
 * <li>Tabelle mit den Objekten des Datenpools (oberer Bereich)</li>
 * <li>Tabelle mit den Properties des aktuell selektieren Objekts (unterer Bereich)</li>
 * </ul>
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DataPoolFrame_BasicTable extends DataPoolFrame {
  private XuluModellingPlatform       appl              = null;
  private MultiSplitPane              dataPoolSplitPane = new MultiSplitPane(2,javax.swing.JSplitPane.VERTICAL_SPLIT);
  private Container                   contentPane       = null;
  private JTable                      dataPoolTable     = null;
  private DisplayContainer_Properties propCont          = null;
  private Object[]                    selectedObjects   = new Object[0];
  private DataPoolMenu                dataPoolMenu      = null;

  /**
   * Erzeugt einen neue Instanz des grafischen Datenpools. Die Groesse ist
   * auf 300x350 Pixel voreingestellt.
   * @param appl Xulu-Applikation fuer die der Datenpool-Frame instanziiert wird
   */
  public DataPoolFrame_BasicTable(XuluModellingPlatform appl) {
    this(appl, new Dimension(300,350));
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Datenpools.
   * @param appl   Xulu-Applikation fuer die der Datenpool-Frame instanziiert wird
   * @param size   Groesse des Fensters.
   */
  public DataPoolFrame_BasicTable(XuluModellingPlatform appl, Dimension size) {
    super();
    this.appl = appl;
    this.setSize(size);
    this.contentPane = this.getContentPane();
    contentPane.setLayout( new BorderLayout() );

    // Tabelle mit Properties
    this.propCont = new DisplayContainer_Properties();
    // Tabelle mit Xulu-Objekten
    this.dataPoolTable = new JTable( new DataPoolTableModel(appl.getDataPool()) );
    this.dataPoolTable.addMouseListener( new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        int index = dataPoolTable.getSelectedRow();
        XuluObject xuluObject  = index < 0 ? null : ((DataPoolTableModel)dataPoolTable.getModel()).getXuluData(index);
        if ( propCont.canDisplay(xuluObject) )
          propCont.setObject(xuluObject);
      }
    });

    // SplitPane organisieren
    dataPoolSplitPane.setResizeWeigth( new double[] {0.5,0.5} );
    dataPoolSplitPane.setDividerSize(2);
    dataPoolSplitPane.setContainer(0,new JScrollPane(dataPoolTable));
    dataPoolSplitPane.setContainer(1,propCont);
    contentPane.add(dataPoolSplitPane);

    // Popup-Menu anbinden
    dataPoolMenu = new DataPoolMenu(appl, this);
    PopupMenuListener listener = new PopupMenuListener(dataPoolMenu.getPopupMenu()) {
      public void mouseReleased(MouseEvent e) {
        // Wenn noch keine Zeile in Tabelle selektiert war, wird automatisch
        // die Zeile selektiert, auf die geklickt wurde
        if ( dataPoolTable.getSelectedRowCount() <= 1 ) {
          int clickIdx = dataPoolTable.rowAtPoint( e.getPoint() );
          dataPoolTable.getSelectionModel().addSelectionInterval(
              clickIdx,
              clickIdx
          );
        }
        super.mouseReleased(e);
      }
    };
    dataPoolTable.addMouseListener( listener );
    dataPoolSplitPane.getContainer(0).addMouseListener( listener );
  }

  /**
   * Liefert alle in der Datentabelle <b>und</b> in der Eigenschaftstabelle
   * selektierten Objekte. Dies koennen sowohl <code>XuluObject</code>s sein,
   * als auch einzelne <code>Property</code>s.<br>
   * <b>Bemerke:</b><br>
   * Zur Zeit werden nur die Xulu-Objekte zurueckgegeben!!
   * @return einen leeren Array wenn keine Zeilen selektiert sein.
   */
  public Object[] getSelectedObjects() {
    int[]      selXuluData = dataPoolTable.getSelectedRows();
    Property[] selProperty = propCont.getSelectedElements();

    Object[] selectedObjects = new Object[ selXuluData.length + selProperty.length ];
    for (int i=0; i<selXuluData.length; i++)
      selectedObjects[i] = ((DataPoolTableModel)dataPoolTable.getModel()).getXuluData(selXuluData[i]);
    for (int i=0; i<selProperty.length; i++)
      selectedObjects[i+selXuluData.length] = selProperty[i];
    return selectedObjects;
  }

  /**
   * Liefert das in der Datentabelle selektierte XuluObjekt.
   * @return <code>null</code> wenn keine Zeile selektiert ist
   */
  public XuluObject getSelectedObject() {
    int selXuluData = dataPoolTable.getSelectedRow();
    if ( selXuluData < 0 )
      return null;
    return ((DataPoolTableModel)dataPoolTable.getModel()).getXuluData(selXuluData);
  }

  /**
   * Aktualisiert die Visualisierung des Datenpools, indem ein
   * <code>fireTableDataChanged()</code>-Event auf das TableModel der
   * Datenpool-Tabelle ausgeloest wird.
   */
  public void refresh() {
    ((AbstractTableModel)dataPoolTable.getModel()).fireTableDataChanged();
  };


  ///////////////////////////////////////////////////////////////////
  // TableModel fuer die Datenpool-Tabelle
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt ein TableModel fuer den {@link schmitzm.data.DataPool} dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class DataPoolTableModel extends AbstractTableModel {
    /** Speichert die Spaltennamen der Tabelle */
    protected final String[] COLUMN_NAME = {"Xulu-Objekt","Datentyp"};

    /** Fuer wahlfreien Zugriff, wird der DataPool-Inhalt protokolliert. */
    private Vector content = new Vector();

    /**
     * Erzeugt ein neues Daten-Modell fuer einen Datenpool.
     * @param dp Instanz eines Xulu-Datenpools
     */
    public DataPoolTableModel(DataPool dp) {
      // Inhalt des Datenpools aufnehmen
      for (Enumeration e=dp.asEnumeration(); e.hasMoreElements(); )
        content.add( e.nextElement() );

      // Listener, der neue DP-Inhalte protokolliert
      dp.addObjectListener( new ObjectListener() {
        public void performObjectEvent(ObjectEvent e) {
          DataPool.DataPoolChangeEvent dpEvent = (DataPool.DataPoolChangeEvent)e;
          switch (dpEvent.getType()) {
            // Objekt wurde hinzugefuegt
            case DataPool.DataPoolChangeEvent.OBJECT_ADDED:
              content.add(dpEvent.getSource().getRoot());
              break;
            // Objekt wurde entfernt
            case DataPool.DataPoolChangeEvent.OBJECT_REMOVED:
              content.remove(dpEvent.getSource().getRoot());
              break;
          }
          // Aenderungen an Table(s) weitergeben
          fireTableDataChanged();
        }
      });

    }

    /**
     * Liefert ein Xulu-Objekt.
     * @param row Zeile des Datenmodells
     */
    public XuluObject getXuluData(int row) {
      return (XuluObject)content.elementAt(row);
    }

    /**
     * Liefert einen Eintrag der Tabelle. Fuer Spalte 0 wird die Objekt-Beschreibung
     * zurueckgegeben, fuer Spalte 1 der Datentyp des jeweiligen (durch <code>row</code>
     * spezifizierten) Objekts.
     * @param row Tabellenzeile
     * @param col Tabellenspalte
     */
    public Object getValueAt(int row, int col) {
      switch( col ) {
        case 0: // Objekt-Beschreibung
                return ((XuluObject)content.elementAt(row)).getDescription();
        case 1: // Objekt-Typ
                return ((XuluObject)content.elementAt(row)).getClass();
      }
      return null;
    }

    /**
     * Liefert die Anzahl an Tabellen-Zeilen. Diese entspricht der Anzahl an
     * Daten-Objekten im Datenpool.
     */
    public int getRowCount() {
      return this.content.size();
    }

    /**
     * Liefert die Anzahl an Tabellen-Spalten. Diese ist konstant 2 (eine
     * Spalte fuer die Objekt-Bezeichnung und eine fuer den Datentyp).
     */
    public int getColumnCount() {
      return COLUMN_NAME.length;
    }

    /**
     * Liefert die Spaltennamen.
     * @see #COLUMN_NAME
     */
    public String getColumnName(int col) {
      return COLUMN_NAME[col];
    }

    /**
     * Prueft, ob eine Tabellen-Zelle editierbar ist. Dies ist fuer alle
     * Zellen der Spalte 0 ("Objekt-Bezeichnung") der Fall.
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public boolean isCellEditable(int row, int col) {
      return col == 0;
    }

    /**
     * Setzt die jeweilige Objekt-Beschreibung neu, wenn als Spaltennummer 0
     * angegeben wird. Ansonsten macht diese Methode nichts.
     * @param obj neue Objektbeschreibung (<code>String</code>)
     * @param row Tabellen-Zeile
     * @param col Tabellen-Spalte
     */
    public void setValueAt(Object obj, int row, int col) {
      // nur die Spalte mit der Objekt-Beschreibung ist editierbar
      if ( isCellEditable(row,col) )
        ((XuluObject)content.elementAt(row)).setDescription(obj.toString());
    }
  }
}
