/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.BorderLayout;
import java.awt.Color;

import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.feature.FeatureCollection;
import org.geotools.styling.ColorMap;
import org.geotools.styling.RasterSymbolizer;
import org.geotools.styling.Style;
import org.geotools.styling.StyleBuilder;
import org.opengis.geometry.Envelope;

import schmitzm.geotools.feature.FeatureUtil;
import schmitzm.geotools.gui.JMapPane;
import schmitzm.swing.ObjectDisplayContainer;

/**
 * Diese Klasse stellt eine GUI-Komponente dar, in der immer eine der folgenden
 * Instanzen grafisch angezeigt werden kann:
 * <ul>
 * <li>{@link GridCoverage2D}</li>
 * <li>{@link FeatureCollection}</li>
 * </ul>
 * @see JMapPane
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class DisplayContainer_GeomAndGrid extends ObjectDisplayContainer {
  private JMapPane map = null;

  /**
   * Erzeugt eine neue Darstellungskomponente.
   */
  public DisplayContainer_GeomAndGrid() {
    this(null);
  }

  /**
   * Erzeugt eine neue Darstellungskomponente und zeigt sofort ein
   * Objekt an.
   * @param data anzuzeigendes Objekt
   */
  public DisplayContainer_GeomAndGrid(Object data) {
    super();
    this.setLayout(new BorderLayout());
    this.map = new JMapPane();
    this.map.setSize(100,100);
    this.add(map, BorderLayout.CENTER);
    this.doLayout();
    setObject(data);
  }

  /**
   * Prueft, ob ein Objekt darstellbar ist.
   * @return <code>true</code> gdw. das angegebene Objekt eine Instanz von
   *         {@link GridCoverage2D} oder {@link FeatureCollection} ist.
   */
  public boolean canDisplay(Object data) {
    return data instanceof GridCoverage2D
        || data instanceof FeatureCollection
    ;
  }

  /**
   * Aktualisiert die Darstellung des aktuell angezeigten Objekts.
   */
  public void refresh() {
    if (map != null) {
      // aktuelle Darstellung (Position/Zoom) merken, um nachher den
      // das neue Layer in diesen Einstellungen anzuzeigen
      com.vividsolutions.jts.geom.Envelope oldArea = ( map.getContext().getLayerCount() > 0 ) ? map.getMapArea() : null;
      com.vividsolutions.jts.geom.Envelope newArea = null;
      // alte Layer entfernen
      map.getContext().clearLayerList();

      // neues Layer einfuegen
      if ( data instanceof GridCoverage2D ) {
        // ###### RASTER ######
        StyleBuilder builder = new StyleBuilder();
        ColorMap     colMap  = builder.createColorMap(
            new String[]  {    "Blue",     "Green",     "Yellow",    "Red"},
            new double[]  {         0,           1,          2,          3},
            new Color[]   {Color.BLUE, Color.GREEN, Color.YELLOW, Color.RED},
            ColorMap.TYPE_VALUES
        );
        RasterSymbolizer rasterSymb = builder.createRasterSymbolizer();
        rasterSymb.setColorMap( colMap );

        Style rasterStyle = builder.createStyle( rasterSymb );
        map.getContext().addLayer( (GridCoverage2D) data, rasterStyle );

// Versuch, die Farbe nachtraeglich zu aktualisieren:
//        colMap  = builder.createColorMap(
//            new String[]  {    "Blue",     "Green",     "Yellow",    "Red"},
//            new double[]  {         0,           1,          2,          3},
//            new Color[]   {Color.BLUE, Color.GREEN, Color.YELLOW, Color.RED},
//            ColorMap.TYPE_VALUES
//        );
//        rasterSymb = builder.createRasterSymbolizer(colMap,1.0);
//        rasterSymb.setColorMap( colMap );
//        map.getContext().getLayer(0).setStyle(rasterStyle);

        // neuer Anzeigebereich: Das komplette Raster
        Envelope e = ((GridCoverage2D) data).getEnvelope();
        newArea = new com.vividsolutions.jts.geom.Envelope(
              e.getUpperCorner().getOrdinate(0),  // X1
              e.getLowerCorner().getOrdinate(0),  // X2
              e.getUpperCorner().getOrdinate(1),  // Y1
              e.getLowerCorner().getOrdinate(1)); // Y2
      } else
      if ( data instanceof FeatureCollection ) {
        // ###### FEATURECOLLECTION ######
        FeatureCollection fc      = (FeatureCollection) data;
        Style             fcStyle = FeatureUtil.createDefaultStyle(fc);
        map.getContext().addLayer( fc, fcStyle );
        // neuer Anzeigebereich: Die komplette FeatureCollection
        newArea = (fc.getBounds());
      }

      map.setMapArea( newArea );
      map.setReset(true);
      map.repaint();

    }
  }
}
