/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.BorderLayout;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Container;
import java.awt.Component;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JMenu;

import schmitzm.lang.WorkingThread;
import schmitzm.lang.WorkingThreadAdapter;
import schmitzm.lang.LangUtil;
import schmitzm.swing.SwingUtil;
import schmitzm.swing.TextAreaPrintStream;
import schmitzm.swing.event.PopupMenuListener;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluConstants;
import edu.bonn.xulu.appl.EventHandler;
import edu.bonn.xulu.appl.AbstractManager;
import edu.bonn.xulu.appl.EventManager;

import edu.bonn.xulu.gui.ModelControlFrame;
import edu.bonn.xulu.gui.XuluGUIMessages;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.gui.ModelControlContainer;
import edu.bonn.xulu.gui.ModelContentManagerContainer;
import edu.bonn.xulu.data.XuluDataException;

// nur fuer Doku
import edu.bonn.xulu.model.ModelContentManager;
import appl.plugin.multimodelcontrol.MultiModelControlPlugin;

/**
 * Diese Klasse stellt ein Fenster, ueber welches ein Modell gesteuert werden
 * kann. Es besteht im wesentlichen aus 3 Bereichen:
 * <ol>
 * <li><b>Kontroll-Bereich ({@link ModelControlContainer})</b><br>
 *     Dient der Steuerung des Modellablaufs (Start/Stop/Init/...)</li>
 * <li><b>Daten-Bereich ({@link ModelContentManagerContainer})</b><br>
 *     Dient der Zuordnung von Datenpool-Objekten zu den benoetigten
 *     Modell-Ressourcen</li>
 * <li><b>Status-Bereich ({@link JTextArea})</b><br>
 *     Auf diesen Text-Bereich werden saemtlich Status-Ausgaben des Modells
 *     umgelenkt (siehe {@link XuluModel#setStatusOut(PrintStream)} und
 *     {@link TextAreaPrintStream})</li>
 * <li><b>Reload-Button</b><br>
 *     Dient dazu die dem Fenster zugrunde liegende Modell-Klasse neu von der
 *     Festplatte zu laden. Wurde die Klasse seit dem Instanziieren des Modells
 *     veraendert und neu kompiliert, werden diese Aenderungen uebernommen.</li>
 * </ol>
 *
 * <hr>
 *<br><b>1.04.07, Dominik Appl</b><br>
 *  reorganzied, so that subclasses can reuse more code
 *<br><b>29.04.07, Dominik Appl</b><br>
 *  it is now possible to access the underlying control container and the setResources() method
 *  from outside (for controlling the model execution from the {@link MultiModelControlPlugin})
 *
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.1
 */
public class ModelControlFrame_Basic extends ModelControlFrame implements XuluConstants {
	protected ModelContentManagerContainer contentContainer     = null;
	protected ModelControlContainer        controlContainer     = null;
	protected JButton                      reloadButton         = new JButton("Modell neu laden");
	private   JTextArea                    statusArea           = null;
	private   TextAreaPrintStream          statusStream         = null;
	protected Container                    contentPane          = null;
	private   JFrame                       modelSpecificFrame   = null;
	private   JMenu                        statusMenu           = new JMenu("Bearbeiten");
	private   JMenuItem                    statusMenuItem_clear = new JMenuItem("Löschen");
	protected XuluModellingPlatform        appl                 = null;
  /**
   * Erzeugt ein neues Kontrollfenster
   * @param appl  Applikation in der das Modell laeuft
   * @param model Modell, das durch das Fenster kontrolliert werden soll
   */
  public ModelControlFrame_Basic(XuluModellingPlatform appl, XuluModel model) {
    super(model);
    this.appl = appl;

    contentPane = this.getContentPane();
    contentPane.setLayout( new GridBagLayout() );

    initControlContainer();
    initReloadButton();
    initContentPane();
    initStatusArea();

    // Fenster auf optimale Breite bringen
    this.pack();

    // ####### optionale Modell-GUI #######
    createModelSpecificFrame(model);
  }

  	/**
	 * inits the status Area
	 */
	protected void initStatusArea() {
		// ####### Status-Komponente #######
		statusArea = new JTextArea();
		statusArea.setEditable(false);
		statusArea.setRows(10); // initiale Groesse 10 Zeilen
		contentPane.add(new JLabel("Status-Ausgaben:"), new GridBagConstraints(
				0, 2, 2, 1, 0.0, 0.0, GridBagConstraints.SOUTHWEST,
				GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));
		contentPane.add(new JScrollPane(statusArea), new GridBagConstraints(0,
				3, 2, 1, 1.0, 1.0, GridBagConstraints.CENTER,
				GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));
		// Model-Ausgabe auf Status-Komponente umleiten
		statusStream = new TextAreaPrintStream(statusArea);
		model.setStatusOut(statusStream);
		// PopupMenue um Feld zu loeschen
		statusMenuItem_clear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				statusArea.setText("");
			}
		});
		statusMenu.add(statusMenuItem_clear);
		statusArea.addMouseListener(new PopupMenuListener(this.statusMenu
				.getPopupMenu()));

	}

	/**
	 * inits the content pane
	 */
	protected void initContentPane() { // ####### Daten-Komponente #######
		contentContainer = new ModelContentManagerContainer(model
				.getContentManager(), appl.getDataPool());
		contentPane.add(contentContainer, new GridBagConstraints(0, 1, 2, 1,
				1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));
	}

	/**
	 * inits the reload button
	 */
	protected void initReloadButton() {
		// ####### Reload-Button ######
		reloadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				reloadModel();
			}
		});
		contentPane.add(reloadButton, new GridBagConstraints(1, 0, 1, 1, 1.0,
				0.0, GridBagConstraints.EAST, GridBagConstraints.NONE,
				new Insets(5, 5, 5, 5), 0, 0));

	}

	/**
	 * inits the control container
	 */
	protected void initControlContainer() {
		// ####### Kontroll-Komponente #######
		controlContainer = new ModelControlContainer(model);
		// Bei Init-Aktion muessen zunaechst die Modell-Ressourcen mit den
		// ausgewaehlten
		// Datenpool-Objekten verknuepft werden; bevor das Modell initialisiert
		// wird
		controlContainer.addButtonActionListener(
				ModelControlContainer.BUTTON_INIT, new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setModelResources();
					}
				});
		// Waehrend das Modell laeuft, soll nichts an den Ressourcen veraendet
		// werden
		controlContainer.addThreadListener(new WorkingThreadAdapter() {
			public void threadStarted(WorkingThread thread) {
				contentContainer.setEnabled(false);
				reloadButton.setEnabled(false);
			}

			public void threadStopped(WorkingThread thread) {
				contentContainer.setEnabled(true);
				reloadButton.setEnabled(true);
			}
		});
		contentPane.add(controlContainer, new GridBagConstraints(0, 0, 1, 1,
				1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(5, 5, 5, 5), 0, 0));
	}


/**
 * Erzeugt das Fenster fuer die modellspezifische GUI, falls eine solche vom
 * Modell angeboten wird
 */
  private void createModelSpecificFrame(XuluModel model) {
    if ( modelSpecificFrame != null )
      modelSpecificFrame.dispose();

    Component modelGUI = model.getModelSpecificGUI();
    if ( modelGUI == null )
      this.modelSpecificFrame = null;
    else {
      this.modelSpecificFrame = new JFrame();
      this.modelSpecificFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      this.modelSpecificFrame.getContentPane().setLayout(new BorderLayout());
      this.modelSpecificFrame.getContentPane().add(modelGUI, BorderLayout.CENTER);
      this.modelSpecificFrame.pack();
      SwingUtil.setRelativeFramePosition(modelSpecificFrame,this,SwingUtil.BOUNDS_OUTER,SwingUtil.NORTHEAST);
    }
  }

  /**
   * Zeigt das Kontroll-Fenster an oder verbirgt es. Zudem wird (falls
   * vorhanden) das modellspezifische Fenster angezeigt/verborgen.
   */
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    if ( modelSpecificFrame != null)
      modelSpecificFrame.setVisible(visible);
  }

  /**
   * Setzt die Bezeichnung fuer das Kontroll-Fenster. Zudem wird (falls
   * vorhanden) die entsprechende Bezeichnung fuer das modellspezifische Fenster
   * gesetzt.
   */
  public void setTitle(String title) {
    super.setTitle(title);
    model.setName(title);
    if ( modelSpecificFrame != null)
      modelSpecificFrame.setTitle(getTitle()+" [Modellspezifisch]");
  }

  /**
   * Belegt die Ressourcen des Modells mit den ausgewaehlten Objekten des
   * Datenpools. Hierbei wird durch den {@link ModelContentManager} die
   * Korrektheit geprueft.
   * @exception XuluDataException falls eine Ressource nicht korrekt belegt wurde
   */
  public void setModelResources() {
    Object[] data = contentContainer.getSelection();

    if ( model.isInitialised() )
      model.disposeModel();

    try {
      // erst die Ressoucen zuordnen (ohne Check, da sie aufeinander
      // aufbauen koennen!)
      for (int i = 0; i < data.length; i++)
        model.getContentManager().getResource(i).setData(data[i],false);
      // wenn alle Ressourcen zugeordnet sind, checken
      model.getContentManager().checkAndError();
    } catch (XuluDataException err) {
      XuluGUIMessages.showDataError(this,err);
    } catch (Exception err) {
      XuluGUIMessages.showError(this,err,"Fehler bei Ressourcenzuteilung");
    }
  }

  /**
   * Schliesst die Komponente komplett. Das Modell wird angehalten und
   * alle Ressourcen freigegeben.
   */
  public void close() {
    model.disposeModel();
    setVisible(false);
    contentContainer.dispose();
    if ( modelSpecificFrame != null)
      modelSpecificFrame.dispose();
    super.dispose();
  }

  /**
   * Aktualisiert die grafische Darstellung des internen Fensters, in dem
   * die Auswahlfelder des ModelContentManagers neu organisiert werden.
   * Die aktuelle Auswahl geht dabei verloren!
   */
  public void refresh() {
    this.contentContainer.reorganize();
  }

  public ModelContentManagerContainer getModelContentManagerContainer(){
	  return contentContainer;
  }
  public ModelControlContainer getControlContainer(){
	  return controlContainer;
  }

  protected ModelControlFrame_Basic newInstance(XuluModel newModel){
	  return new ModelControlFrame_Basic(appl,newModel);
  }

  private void reloadModel() {
    try {
      // Klasse neu laden
      Class newModelClass = LangUtil.loadPreferredClass(model.getClass().getName(), appl.getDynamicClassRootDirectory().toURL() );
      // Alte Klasse in Registry durch neue ersetzen
      appl.getRegistry().replaceModelClass( model.getClass(), newModelClass );
      // Modell neu erzeugen
      XuluModel newModel = (XuluModel)newModelClass.newInstance();
      // Modell-Fenste neu erzeugen (und Groesse/Position des alten uebernehmen)
      ModelControlFrame_Basic newModelFrame = newInstance(newModel);
      newModelFrame.setTitle( getTitle() );
      newModelFrame.setExtendedState( getExtendedState() );
      // Groesse und Position nur setzen, wenn Fenster nicht minimiert ist.
      // Ansonsten wird "normale" Groesse/Position auf maximierte Groesse gesetzt!
      if ( (getExtendedState() & JFrame.MAXIMIZED_BOTH) == 0 ) {
        newModelFrame.setSize(getSize());
        newModelFrame.setLocation(getLocation());
      }
      // Position des Modell-spezifischen Fensters setzen
      if ( modelSpecificFrame != null && newModelFrame.modelSpecificFrame != null )
        newModelFrame.modelSpecificFrame.setLocation( modelSpecificFrame.getLocation() );

      // Ressourcen-Zurordnung von altem Fenster uebernehmen
      Object[] selection = contentContainer.getSelection();
      for (int i=0; i<selection.length && i<newModel.getContentManager().getResourceCount(); i++)
        newModelFrame.contentContainer.setResourceByObject(i,selection[i]);

      // alle Events vom alten Modell merken
      Vector<EventHandler> events = new Vector<EventHandler>();
      EventManager em = appl.getEventManager();
      for (int i=0; i<em.getCount(); i++) {
        EventHandler eh = (EventHandler)em.get(i);
        if ( eh.getObject() == model )
          events.add( new EventHandler(newModel,eh.getEventType(),eh.getHandler(),appl) );
      }

      // Altes Modell-Fenster loeschen (EventHandler vom geloeschten Modell werden
      // automatisch auch geloescht!!)
      appl.getModelControlManager().remove(this);
      // Neues Modell-Fenster hinzufuegen
      appl.getModelControlManager().add(newModelFrame,true);
      for (int i=0; i<events.size(); i++)
        em.add( events.elementAt(i) );

      // Meldung geben, dass Aktualisierung erfolgreich
      XuluGUIMessages.showInfoMessage(newModelFrame,XULUGUI_RES.getString("ModelReloadSuccessMess"));

    } catch (ClassNotFoundException err) {
      XuluGUIMessages.showClassNotFoundError(this,err,model.getClass().getName());
    } catch (Exception err) {
      XuluGUIMessages.showError(this,err);
    }

  }

}
