/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.JButton;

import schmitzm.swing.StoplightContainer;
import schmitzm.swing.table.AbstractMutableTableModel;
import schmitzm.swing.table.MutableTable;
import schmitzm.swing.table.MutableTableModel;
import schmitzm.swing.table.ComponentRenderer;
import schmitzm.swing.table.TableComponentMouseListener;

import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.appl.XuluPlugin;
import edu.bonn.xulu.gui.XuluInternalFrame;
import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.ObjectEvent;
import schmitzm.data.event.GeneralObjectChangeEvent;

/**
 * Diese Klasse stellt das Xulu-internes Fenster dar, in dem alle aktiven
 * (in der Xulu-Registry registrierten) {@linkplain XuluPlugin Xulu-Plugins}
 * tabellarisch aufgelistet werden.
 * Bei einem Doppelklick auf eine Tabellenzeile wird (sofern vorhanden) die
 * Konfigurations-GUI des Plugins angezeigt.<br>
 * Fuer jedes Plugin wird eine {@linkplain StoplightContainer "Ampel"} dargestellt,
 * die anzeigt, ob das Plugin gestartet ist (GRUEN) oder nicht (ROT). Bei einem
 * Doppelklick auf die Ampel wird das Plugin gestartet, bzw. gestoppt.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class PluginManagerFrame_BasicTable extends XuluInternalFrame {
  private XuluModellingPlatform appl        = null;
  private MutableTable          pluginTable = null;

  /**
   * Erzeugt ein neues Xulu-internes Fenster.
   * @param name Titel des Fensters
   * @param appl Instanz der Xulu-Applikation
   */
  public PluginManagerFrame_BasicTable(String name, XuluModellingPlatform appl) {
    this(name, new Dimension(300,350), appl);
  }

  /**
   * Erzeugt ein neues Xulu-internes Fenster.
   * @param name Titel des Fensters
   * @param size initiale Groesse des Fensters
   * @param appl Instanz der Xulu-Applikation
   */
  public PluginManagerFrame_BasicTable(String name, Dimension size, XuluModellingPlatform appl) {
    super(name);
    this.appl = appl;
    this.pluginTable = new MutableTable( new PluginTableModel(appl.getRegistry()), MutableTable.ITEM_CHANGE );
    this.pluginTable.addMouseListener( new TableComponentMouseListener(pluginTable) );
    this.pluginTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    this.pluginTable.setDefaultRenderer(pluginTable.getModel().getColumnClass(2),new ComponentRenderer());
    this.pluginTable.getColumnModel().getColumn(2).setPreferredWidth(40);
    this.pluginTable.getColumnModel().getColumn(2).setMaxWidth(40);
    this.setSize(size);
    this.setLayout( new BorderLayout() );
    this.getContentPane().add( new JScrollPane(pluginTable) );
  }

  /**
   * Aktualisiert die grafische Darstellung des Plugin-Fensters.
   */
  public void refresh() {
    if ( pluginTable != null && pluginTable.getModel() != null )
      ((PluginTableModel)this.pluginTable.getModel()).fireTableDataChanged();
  }

  /**
   * Diese Klasse bildet die Datenbasis der Plugin-Tabelle.
   */
  private class PluginTableModel extends AbstractMutableTableModel {
    private final Class[]  colClass = {String.class, String.class, StoplightContainer.class};

    private XuluRegistry registry = null;

    /**
     * Erzeugt ein neues Tabellen-Modell. Die dargestellten Plugins bezieht
     * es aus der Xulu-Registry.
     * @param registry Instanz der Xulu-Registry
     */
    public PluginTableModel(XuluRegistry registry) {
      this.registry = registry;
      registry.addObjectListener( new ObjectListener() {
        public void performObjectEvent(ObjectEvent e) {
          if ( e instanceof XuluRegistry.ChangeEvent ) {
            XuluRegistry.ChangeEvent regEvent = (XuluRegistry.ChangeEvent)e;
            if ( regEvent.getOldValue() instanceof XuluPlugin || // Plugin geloescht
                 regEvent.getNewValue() instanceof XuluPlugin || // Plugin hinzugefuegt
                 regEvent.getOldValue() == null && regEvent.getNewValue() == null ) // Generelles Aenderungs-Ereignis
              fireTableDataChanged();
          }
        }
      });
    }

    /**
     * Liefert die Spaltennamen der Tabelle.
     */
    @Override
    public String[] createColumnNames() {
      return new String[] {"Plugin", "Beschreibung", ""};
    }
    
    /**
     * Liefert einen Wert fuer die Tabelle. Fuer Spalte 0 wird der Klassenname
     * des Plugins geliefert (String), fuer Spalte 1 die Plugin-Beschreibung (String)
     * und fuer Spalte 3 die "Ampel", welche den Plugin-Status anzeigt ({@link StoplightContainer}).
     * @param row Zeilennummer
     * @param col Spaltennummer
     */
    public Object getValueAt(int row, int col) {
      XuluPlugin plugin = registry.getPlugins()[row];
      switch ( col ) {
          case 0: // Plugin-Name
                  return plugin.getClass().getSimpleName();
          case 1: // Plugin-Bezeichnung
                  return plugin.getName();
          case 2: // Button zum Starten/Stoppen des Plugins
                  return new PluginControlButton(plugin,
                                                 appl,
                                                 pluginTable.getColumnModel().getColumn(2).getWidth(),
                                                 15
                  );
      }
      return "";
    }

    /**
     * Liefert den Typ einer Tabellen-Spalte.
     * @param col Spaltennummer (beginnend bei 0)
     */
    public Class getColumnClass(int col) {
      return colClass[col];
    }

    /**
     * Liefert die Anzahl an Tabellen-Zeilen. Entspricht der Anzahl der in der
     * Xulu-Registry registrierten Plugins.
     */
    public int getRowCount() {
      return registry.getPlugins().length;
    }

    /**
     * Macht nichts, da die Plugin-Tabelle die Option "Loeschen" nicht
     * unterstuetzt.
     * @param row Zeilennummer
     */
    public void performRemoveRow(int row) {
    }

    /**
     * Macht nichts, da die Plugin-Tabelle die Option "Hinzufuegen" nicht
     * unterstuetzt.
     */
    public void performAddRow() {
    }

    /**
     * Wird bei einem Doppelklick auf die Tabellenzeile ausgefuehrt.
     * Dabei wird die GUI zur Konfiguration des jeweilige Plugins geoeffnet.
     * @param row Zeilennummer
     * @param col Spalzennummer (nicht relevant)
     */
    public void performChangeData(int row, int col) {
      XuluPlugin plugin = registry.getPlugins()[row];
      // nicht beim Doppelklick auf die Start/Stop-Button
      if ( col != 2 )
        plugin.setVisible(true);
    }
  }

  /**
   * Diese Klasse stellt den Ampel-Button dar, der den Status eines Plugins anzeigt.
   * Bei einem Doppelklick auf die "Ampel" wird das Plugin gestartet, bzw.
   * gestoppt.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class PluginControlButton extends StoplightContainer {
    private XuluPlugin            plugin;

    /**
     * Erzeugt eine neue Ampel.
     * @param plugin Plugin, das durch den Button gesteuert wird
     * @param appl   Instanz der Xulu-Applikation
     * @param width  Breite des Buttons
     * @param height Hoehe des Buttons
     */
    public PluginControlButton(final XuluPlugin plugin, final XuluModellingPlatform appl, int width, int height) {
      super(StoplightContainer.RED,width,height,false);
      this.plugin = plugin;
      actualiseCaption();
      this.addMouseListener( new MouseAdapter() {
        public void mouseClicked(MouseEvent e) {
          if ( e.getClickCount() != 2 )
            return;

          if ( plugin.isStarted() )
            plugin.stop();
          else
            plugin.execute(appl);
          actualiseCaption();
        }
      });
    }

    /**
     * Aktualisiert die Darstellung der Ampel entsprechend des Plugin-Status.
     */
    public void actualiseCaption() {
      if ( plugin.isStarted() )
        this.setMode(GREEN);
      else
        this.setMode(RED);
    }
  }
}
