/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import java.awt.BorderLayout;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import java.util.EventObject;

import schmitzm.data.event.ObjectListener;
import schmitzm.data.event.ObjectEvent;
import schmitzm.lang.NamedObject;
import schmitzm.lang.AlreadyHandledException;
import schmitzm.swing.tree.EditableNode;
import schmitzm.swing.tree.EmptyNode;
import schmitzm.swing.tree.EmptyInnerNode;
import schmitzm.swing.tree.ContentNode;

import edu.bonn.xulu.io.Factory;
import edu.bonn.xulu.io.InstantiationFactory;
import edu.bonn.xulu.io.ImportFactory;
import edu.bonn.xulu.io.ExportFactory;
import edu.bonn.xulu.io.TypeMapping;
import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.appl.ScriptInterpreter;
import edu.bonn.xulu.appl.XuluPlugin;
import edu.bonn.xulu.appl.VisualisationTool;
import edu.bonn.xulu.gui.XuluGUIMessages;
import edu.bonn.xulu.gui.RegistryFrame;

/**
 * Dieses Fenster stellt die Visualisierung der {@link edu.bonn.xulu.appl.XuluRegistry}
 * dar. Es stellt deren gesamten Inhalt in einer Baumstruktur dar.
 * @todo Veraenderungen an Registry mittels PopupMenu
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class RegistryFrame_Tree extends RegistryFrame {
  private XuluRegistry registry     = null;
  private JTree        registryTree = null;

  /**
   * Erzeugt ein neues Registry-Fenster.
   * @param registry Datenbasis
   */
  public RegistryFrame_Tree(XuluRegistry registry) {
    super();
    this.registry = registry;
    this.setLayout( new BorderLayout() );
    this.registryTree = new JTree();
    registryTree.setModel( new RegistryTreeModel(registry) );
    registryTree.setRootVisible(false);
    registryTree.setEditable(true);
    registryTree.getSelectionModel().setSelectionMode( TreeSelectionModel.SINGLE_TREE_SELECTION );
    // nicht alle Zellen im Baum duerfen veraendet werden
    registryTree.setCellEditor( new DefaultTreeCellEditor(registryTree, (DefaultTreeCellRenderer)registryTree.getCellRenderer()) {
      private Object lastSelected = null;
      // Diese Methode wird aufgerufen, um zu entscheiden, ob ein
      // Knoten editierbar ist
      public boolean isCellEditable(EventObject event) {
        return super.isCellEditable(event) &&
               lastSelected != null &&
               lastSelected instanceof EditableNode &&
               ((EditableNode)lastSelected).isEditable();
      }
      // Diese Methode wird aufgerufen, sobald ein neuer Knoten im Baum
      // selektiert wurde --> merken, welcher das ist
      public void valueChanged(TreeSelectionEvent e) {
        if ( e.getNewLeadSelectionPath() != null )
          lastSelected = e.getNewLeadSelectionPath().getLastPathComponent();
        else
          lastSelected = null;
        super.valueChanged(e);
      }
    });
    // UpdateListener auf Registry
    registry.addObjectListener( new ObjectListener() {
      public void performObjectEvent(ObjectEvent e) {
        if ( e instanceof XuluRegistry.ChangeEvent ) {
          XuluRegistry.ChangeEvent regEvent = (XuluRegistry.ChangeEvent)e;
          // Wenn der alte oder neue Wert null ist, wurde ein Eintrag
          // hinzugefuegt oder entfernt --> Update noetig
          // Bei blosser Aenderung muss der Baum nicht reorganisiert werden!
          if (regEvent.getOldValue() == null || regEvent.getNewValue() == null)
            refresh();
        }
      }
    });
    this.getContentPane().add( new JScrollPane( registryTree ) );
  }

  /**
   * Baut den Baum der Registry komplett neu auf.
   */
  public void refresh() {
    ((RegistryTreeModel)registryTree.getModel()).reorganize();
  }

  ///////////////////////////////////////////////////////////////////
  // Datenmodell fuer den Registry-Baum
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese klasse stellt das Datenmodell fuer den Regstry-Baum dar.
   * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
   * @version 1.0
   */
  private class RegistryTreeModel extends DefaultTreeModel {
    /**
     * Erzeugt ein neues Datenmodell
     * @param registry darzustellende XuluRegistry
     */
    public RegistryTreeModel(XuluRegistry registry) {
      super(new RegistryRootNode(registry));
    }

    /**
     * Regorganisiert das Datenmodell der Registry komplett.
     */
    public void reorganize() {
      ((RegistryRootNode)this.getRoot()).reorganize();
      nodeStructureChanged((RegistryRootNode)this.getRoot());
    }
  }

  ///////////////////////////////////////////////////////////////////
  // Knoten des Registry-Baums
  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt Knoten dar, der den Klassennamen des UserObjects
   * anzeigt. Handelt es sich beim UserObject bereits um eine
   * <code>Class</code>-Instanz, wird deren Name angezeigt.
   */
  private abstract class ClassDisplayNode extends ContentNode {
    /**
     * Ereugt einen neuen Knoten.
     */
    public ClassDisplayNode(Object userObject, boolean editable) {
      super(userObject,editable);
    }

    /**
     * Liefert den Klassennamen des UserObject (oder falls es sich dabei
     * bereits um eine <code>Class</code>-Instanz handelt, deren Namen).
     */
    public String toString() {
      Object o = getUserObject();

// UEBERDENKEN: Aenderung eines Objekts problematisch, wenn Name angezeigt wird!
//              --> Klasse soll aus String erzeugt werden!!
//      // benannte Objekte
//      if ( o instanceof NamedObject ) {
//        NamedObject n = (NamedObject)o;
//        if ( n.getName() != null && !n.getName().equals("") )
//          return n.getName().concat(" (").concat(n.getClass().getName()).concat(")");
//      }

      // Klassen
      if ( o instanceof Class )
        return ((Class)o).getName();

      // alle anderen Objekte
      return o.getClass().getName();
    }

    /**
     * Wird aufgerufen, wenn der angezeigte Klassenname veraendert wird und
     * daraus eine neue Klasse instanziiert werden konnte.
     * Die Implementierung muss dann entscheiden, wie weiter mit dieser
     * Klasse verfahren wird (ob daraus eine neue Instanz erzeugt wird,
     * oder die Klasse selbst gespeichert wird, ...).
     * @see #setUserObject(Object)
     * @return aus der Klasse generiertes Object, welches als UserObject
     *         gespeichert wird
     */
    public abstract Object createUserObjectFromClass(Class newClass) throws Exception;

    /**
     * Wird aufgerufen, wenn der angezeigte Klassenname veraendert wird.
     * Es wird versucht, daraus ein neues <code>Class</code>-Objekt zu
     * instanziieren.
     * Nachdem die Implementierung daraus erfolgreich ein neues UserObject
     * erzeugt hat, wird diese gespeichert.
     * @see #createUserObjectFromClass(Class)
     */
    public void setUserObject(Object object) {
      try {
        Class newValue = Class.forName( (String) object);
        // Registry entsprechend updaten
        createUserObjectFromClass(newValue);
        // nachdem Registry erfolgreich aktualisiert wurde
        // kann auch der Baum aktualisiert werden
        super.setUserObject(newValue);
      }
      catch (AlreadyHandledException err) {
        // Fehler wurde bereits verarbeitet
      }
      catch (ClassNotFoundException err) {
        XuluGUIMessages.showClassNotFoundError(GUI_COMPONENT, err, (String)object);
//        throw new AlreadyHandledException();
      }
      catch (Exception err) {
        XuluGUIMessages.showError(GUI_COMPONENT, err);
//        throw new AlreadyHandledException();
      }
    }
  }

  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse stellt den Wurzelknoten des Registry-Baums dar. Er wird
   * mit "ROOT" benannt und enthaelt Kind-Knoten fuer:
   * <ul>
   * <li>die registrierten Datentypen</li>
   * <li>die registrierten Default-Factorys</li>
   * <li>die registrierten Import-Factorys</li>
   * <li>die registrierten Export-Factorys</li>
   * <li>die registrierten TypeMappings</li>
   * <li>die registrierten Modell-Klassen</li>
   * <li>die registrierten Script-Interpreter</li>
   * <li>die registrierten Plugins</li>
   * <li>die registrierten Autostart-Plugins</li>
   * </ul>
   */
  private class RegistryRootNode extends EmptyNode {
    private XuluRegistry registry = null;

    /**
     * Erzeugt einen neuen Wurzel-Knoten.
     * @param registry Datenbasis-XuluRegistry
     */
    public RegistryRootNode(XuluRegistry registry) {
      super("ROOT");
      this.registry = registry;
      reorganize();
    }
    /**
     * Reorganisiert den (Unter)Baum der Registry, in dem der komplette
     * Baum neu aufgebaut wird.
     */
    public void reorganize() {
      this.removeAllChildren();
      // Kind fuer Datentypen
      this.add(new ClassesRoot("Daten-Typen", registry.getDataTypes()) {
        public void replaceClass(Class oldType, Class newType) {
          registry.replaceDataType(oldType, newType);
        }
      });
      // Kind fuer Default-Factories
      this.add(new ToolsRoot("Default-Factorys", registry.getDefaultFactories()) {
        public void replaceTool(Object oldFac, Object newFac) {
          registry.replaceDefaultFactory((InstantiationFactory)oldFac, (InstantiationFactory)newFac);
        }
      });
      // Kind fuer Import-Factories
      this.add(new ToolsRoot("Import-Factorys", registry.getImportFactories()) {
        public void replaceTool(Object oldFac, Object newFac) {
          registry.replaceImportFactory((ImportFactory)oldFac, (ImportFactory)newFac);
        }
      });
      // Kind fuer Export-Factories
      this.add(new ToolsRoot("Export-Factorys", registry.getExportFactories()) {
        public void replaceTool(Object oldFac, Object newFac) {
          registry.replaceExportFactory((ExportFactory)oldFac, (ExportFactory)newFac);
        }
      });
      // Kind fuer TypeMappings
      this.add(new TypeMappingRoot("Type-Mappings", registry));
      // Kind fuer Visualisationtools
      this.add(new ClassesRoot("Visualisierungstools", registry.getVisualisationTools()) {
        public void replaceClass(Class oldVis, Class newVis) {
          registry.replaceVisualisationTool(oldVis, newVis);
        }
      });
      // Kind fuer Modellklassen
      this.add(new ClassesRoot("Modellklassen", registry.getModelClasses()) {
        public void replaceClass(Class oldVis, Class newVis) {
          registry.replaceModelClass(oldVis, newVis);
        }
      });
      // Kind fuer Skript-Interpreter
      this.add(new ToolsRoot("Script-Interpreter", registry.getScriptInterpreter()) {
        public void replaceTool(Object oldInt, Object newInt) {
          registry.replaceScriptInterpreter((ScriptInterpreter)oldInt, (ScriptInterpreter)newInt);
        }
      });
      // Kind fuer Plugins
      this.add(new ToolsRoot("Xulu-Plugins", registry.getPlugins()) {
        public void replaceTool(Object oldPlugin, Object newPlugin) {
          registry.replacePlugin((XuluPlugin)oldPlugin, (XuluPlugin)newPlugin);
        }
      });
      // Kind fuer Autostart-Plugins
      this.add(new ToolsRoot("Autostart-Plugins", registry.getAutostartPlugins()) {
        public void replaceTool(Object oldPlugin, Object newPlugin) {
          registry.replaceAutostartPlugin((XuluPlugin)oldPlugin, (XuluPlugin)newPlugin);
        }
      });
    }
  }

  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse bildet die Wurzel fuer Registry-Unterbaeume, die
   * Klassen (z.B. Datentypen oder Visualisationtools) darstellen.
   */
  private abstract class ClassesRoot extends EmptyInnerNode {
    /**
     * Erzeugt einen neuen Knoten.
     * @param desc    Beschreibung
     * @param classes Klassen fuer den Teilbaum
     */
    public ClassesRoot(String desc, final Class[] classes) {
      super(desc);
      for (int i = 0; classes != null && i < classes.length; i++)
        add(classes[i]);
    }

    /**
     * Fuegt einen neuen Datentyp-Kindknoten ein.
     * @param aChild muss eine Instanz von <code>ClassDisplayNode</code> sein
     */
    public void add(MutableTreeNode aChild) {
      if ( !(aChild instanceof ClassDisplayNode) )
        throw new RuntimeException("Only instances of ClassDisplayNode are allowed as childs for ClassesRoot");
      super.add(aChild);
    }

    /**
     * Fuegt einen neuen Datentyp-Kindknoten ein.
     * @param type ein Datentyp
     */
    public void add(Class type) {
      add( new ClassDisplayNode(type, true) {
        public Object createUserObjectFromClass(Class newClass) throws Exception {
          try {
            replaceClass( (Class)getUserObject(), newClass);
          }
          catch (Exception err) {
            XuluGUIMessages.showError(GUI_COMPONENT, err);
            throw new AlreadyHandledException();
          }
          return newClass;
        }
      });
    }

    /**
     * Ersetzt in der Datenbasis eine Klasse durch ein anderes.
     * An welcher Stelle in der Registry dies geschiet, wird durch die Implementierung
     * bestimmt.
     * @param oldClass zu ersetzende Klasse
     * @param newClass neue Klasse
     */
    public abstract void replaceClass(Class oldClass, Class newClass) throws Exception;

  }

  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse bildet eine allgemeine Wurzel fuer alle Registry-Unterbaeume,
   * die "Tools" (z.B. Factorys) darstellen.
   * Mit "Tools" sind quasi-statische Klassen gemeint, die durch ihren
   * Klassenamen repraesentiert werden und durch einen Standard-Konstruktor
   * instanziiert werden.<br>
   * Die Implementierung dieser abstrakten Klasse bestimmt lediglich an welcher
   * Stelle die Registry zu aktualisieren ist, wenn sich ein Eintrag im Baum
   * aendert.
   * @see #replaceTool(Object,Object)
   */
  private abstract class ToolsRoot extends EmptyInnerNode {
    /**
     * Erzeugt einen neuen Knoten. Dieser enthaelt noch keine Kinder
     * (ausser dem Pseudo-Nachfolger des {@link schmitzm.swing.tree.EmptyInnerNode}).
     * @param desc Beschreibung fuer den Knoten, welche angezeigt wird
     */
    public ToolsRoot(String desc) {
      this(desc, new Object[0]);
    }

    /**
     * Erzeugt einen neuen Knoten.
     * @param desc Beschreibung fuer den Knoten, welche angezeigt wird
     * @param tool Tools, deren Klassennamen aufgelistet werden.
     */
    public ToolsRoot(String desc, Object[] tool) {
      super(desc);
      for (int i = 0; tool != null && i < tool.length; i++)
        add(tool[i]);
    }

    /**
     * Fuegt einen neuen Tool-Kindknoten ein.
     * @param aChild muss eine Instanz von <code>ClassDisplayNode</code> sein
     */
    public void add(MutableTreeNode aChild) {
      if ( !(aChild instanceof ClassDisplayNode) )
        throw new RuntimeException("Only instances of ClassDisplayNode are allowed as childs for ToolsRoot");
      super.add(aChild);
    }

    /**
     * Fuegt einen neuen Tool-Kindknoten ein.
     * @param tool eine "Tool"-Instanz (z.B. Factory)
     */
    public void add(Object tool) {
      add( new ClassDisplayNode(tool,true) {
        public Object createUserObjectFromClass(Class newToolType) throws Exception {
          try {
            Object newTool = newToolType.newInstance();
            replaceTool( getUserObject(), newTool );
            return newTool;
          }
          catch (InstantiationException err) {
            XuluGUIMessages.showInstantiationError(GUI_COMPONENT, err);
            throw new AlreadyHandledException();
          }
          catch (ClassCastException err) {
            XuluGUIMessages.showClassCastError(GUI_COMPONENT, err, getUserObject().getClass());
            throw new AlreadyHandledException();
          }
          catch (Exception err) {
            XuluGUIMessages.showError(GUI_COMPONENT, err);
            throw new AlreadyHandledException();
          }
        }
      });
    }

    /**
     * Ersetzt in der Datenbasis ein Tool durch ein anderes.
     * An welcher Stelle in der Registry dies geschiet, wird durch die Implementierung
     * bestimmt.
     * @param oldTool zu ersetzende Tool-Instanz
     * @param newTool neue Tool-Instanz
     */
    public abstract void replaceTool(Object oldTool, Object newTool) throws Exception;
  }

  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse bildet die Wurzel fuer den Registry-Unterbaum, der die
   * Type-Mappings darstellt.
   */
  private class TypeMappingRoot extends EmptyInnerNode {
    private XuluRegistry registry = null;

    /**
     * Erzeugt einen neuen Knoten.
     * @param desc     Beschreibung fuer den Knoten
     * @param registry Datenbasis fuer die TypeMappings
     */
    public TypeMappingRoot(String desc, XuluRegistry registry) {
      super(desc);
      this.registry = registry;
      TypeMapping[] mapping = registry.getTypeMappings();
      for (int i = 0; mapping != null && i < mapping.length; i++)
        add(mapping[i]);
    }

    /**
     * Fuegt einen neuen TypeMapping-Kindknoten ein.
     * @param aChild muss eine Instanz von <code>ClassDisplayNode</code> sein
     */
    public void add(MutableTreeNode aChild) {
      if ( !(aChild instanceof TypeMappingNode) )
        throw new RuntimeException("Only instances of TypeMappingNode are allowed as childs for TypeMappingRoot");
      super.add(aChild);
    }

    /**
     * Fuegt einen neuen TypeMapping-Kindknoten ein.
     * @param mapping ein TypeMapping
     */
    public void add(TypeMapping mapping) {
      add(new TypeMappingNode(mapping));
    }
  }


  ///////////////////////////////////////////////////////////////////
  /**
   * Diese Klasse bildet die Wurzel fuer den Registry-Unterbaum, der ein
   * Type-Mapping darstellt. Dieser selbst ist nicht editierbar, da der
   * Datentyp eines TypeMappings nicht veraenderbar ist.
   */
  private class TypeMappingNode extends ClassDisplayNode {
    public TypeMappingNode(final TypeMapping mapping) {
      super(mapping.getType(), false);

      // Kind fuer die Default-Factorys
      add(new ToolsRoot("Default-Factory", new Factory[] {mapping.getDefaultFactory()}) {
        // ersetzt die DefaultFactory des Mappings durch eine andere
        public void replaceTool(Object oldFac, Object newFac) throws Exception {
          try {
            mapping.setDefaultFactory( (InstantiationFactory) newFac);
          }
          catch (ClassCastException err) {
            XuluGUIMessages.showClassCastError(GUI_COMPONENT, err,InstantiationFactory.class);
            throw new AlreadyHandledException();
          }
        }
      });

      // Kind fuer die Import-Factorys
      add(new ToolsRoot("Import-Factorys", mapping.getImportFactories()) {
        // ersetzt eine ImportFactory des Mappings durch eine andere
        public void replaceTool(Object oldFac, Object newFac) throws Exception {
          try {
            mapping.replaceImportFactory( (ImportFactory)oldFac, (ImportFactory)newFac );
          }
          catch (ClassCastException err) {
            XuluGUIMessages.showClassCastError(GUI_COMPONENT, err,ImportFactory.class);
            throw new AlreadyHandledException();
          }
        }
      });

      // Kind fuer die Export-Factorys
      add(new ToolsRoot("Export-Factorys", mapping.getExportFactories()) {
        // ersetzt eine ExportFactory des Mappings durch eine andere
        public void replaceTool(Object oldFac, Object newFac) throws Exception {
          try {
            mapping.replaceExportFactory( (ExportFactory)oldFac, (ExportFactory)newFac );
          }
          catch (ClassCastException err) {
            XuluGUIMessages.showClassCastError(GUI_COMPONENT, err,ExportFactory.class);
            throw new AlreadyHandledException();
          }
        }
      });
    }

    /**
     * Macht nichts, da der Knoten nicht editierbar ist.
     * @return <code>newClass</code>
     */
    public Object createUserObjectFromClass(Class newClass) {
      // macht nichts, da der Knoten nicht editierbar ist
      return newClass;
    }
  }
}
