/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.gui;

import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.OutputStream;
import java.io.PrintStream;

import schmitzm.swing.event.PopupMenuListener;
import edu.bonn.xulu.XuluModellingPlatform;
import edu.bonn.xulu.gui.StatusFrame;

/**
 * Diese Klasse stellt ein Fenster fuer die Xulu-Modelling-Platform dar, in
 * dem beliebige Statusmeldungen ausgegeben werden koennen. Dies kann ueber
 * einzelne Zeilen geschehen ({@link #print(String)}, {@link #println(String)})
 * oder ueber einen <code>PrintStream</code>. So ist es z.B. auch moeglich
 * mittels {@link System#setErr(PrintStream)} die Standard-Fehlerausgabe auf
 * dieses Fenster umzulenken.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class StatusFrame_BasicTextArea extends StatusFrame {

  private XuluModellingPlatform appl = null;
  private JTextArea             textArea = null;
  private JMenu                 menu           = new JMenu("Bearbeiten");
  private JMenuItem             menuItem_clear = new JMenuItem("Löschen");

  /**
   * Erzeugt einen neue Instanz des grafischen Statusfenster. Die Groesse ist
   * auf 300x350 Pixel voreingestellt.
   * @param appl Xulu-Applikation fuer die das Status-Fenster instanziiert wird
   */
  public StatusFrame_BasicTextArea(XuluModellingPlatform appl) {
    this(appl, new Dimension(300,350));
  }

  /**
   * Erzeugt einen neue Instanz des grafischen Statusfenster. Die Groesse ist
   * auf 300x350 Pixel voreingestellt.
   * @param appl Xulu-Applikation fuer die das Status-Fenster instanziiert wird
   * @param size initiale Groesse des Fensters
   */
  public StatusFrame_BasicTextArea(XuluModellingPlatform appl, Dimension size) {
    super();
    this.appl = appl;
    this.setSize(size);
    this.getContentPane().setLayout( new BorderLayout() );
    this.textArea = new JTextArea();
    this.textArea.setEditable(false);
    this.getContentPane().add( new JScrollPane( textArea ) );
    // PopupMenue
    this.menuItem_clear.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        textArea.setText("");
      }
    });
    this.menu.add(menuItem_clear);
    this.textArea.addMouseListener( new PopupMenuListener(this.menu.getPopupMenu()) );
  }

  /**
   * Macht nichts, da die Ausgaben im Statusfenster nicht von einer bestimmten
   * anderen Komponente abhaengt, welche sich geaendert haben koennte.
   */
  public void refresh() {
  }

  /**
   * Schreibt einen Text in das Statusfenster (ohne anschliessenden Zeilenumbruch!).
   */
  public void print(String text) {
    textArea.append(text);
    // Position ans Ende des Textes setzen
    textArea.setCaretPosition( textArea.getText().lastIndexOf("\n")+1 );
  }
}
