/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.io.grid.gt;

import java.awt.geom.Rectangle2D;
import org.geotools.geometry.Envelope2D;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import schmitzm.data.WritableGrid;
import schmitzm.geotools.grid.WritableGridCoverage;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
// nur fuer Doku
import edu.bonn.xulu.data.XuluObject;
import edu.bonn.xulu.plugin.data.grid.SingleGrid;

/**
 * Diese Factory erzeugt Standard-Instanzen des Datentyps {@link WritableGridCoverage}.<br>
 * <b>Bemerke:</b><br>
 * Diese stellen <b>keine</b> {@linkplain XuluObject Xulu-Objekte} dar, koennen also
 * nicht direkt im Xulu-Datenpool gespeichert werden. Hierzu ist der Datentyp
 * {@link SingleGrid} (bzw. eine entsprechende Factory) zu verwenden!
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class GridCoverageFactory extends WritableGridFactory {
  /**
   * Liefert den Datentyp, den die Factory erzeugt.
   * @return immer {@link WritableGridCoverage WritableGridCoverage.class}
   */
  public Class getInstanceType() {
    return WritableGridCoverage.class;
  }

  /**
   * Erzeugt ein neues Grid.
   * @param type    Datentyp der gespeicherten Objekte
   * @param widthc  Breite in Zellen
   * @param heightc Hoehe in Zellen
   * @param minX    Index der ersten Zelle in X-Richtung (<b>wird nicht verwendet! immer 0!</b>)
   * @param minY    Index der ersten Zelle in Y-Richtung (<b>wird nicht verwendet! immer 0!</b>).
   * @param x       Georeferenz Longitute (Westliche/Linke Kante)
   * @param y       Georeferenz Latitude (Suedliche/Untere Kante)
   * @param width   Breite
   * @param height  Hoehe
   * @param crs     Referenzsystem fuer das Raster
   */
  public WritableGridCoverage newInstance(int type, int widthc, int heightc, int minX, int minY, double x, double y, double width, double height, CoordinateReferenceSystem crs) {
    // Envelope erzeugen
    Envelope2D e = new Envelope2D(crs,new Rectangle2D.Double(x,y,width,height));
    // Grid erzeugen
    return WritableGridCoverage.create("",type,widthc,heightc,e);
  }

}
