/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package edu.bonn.xulu.plugin.io.grid.gt;

import java.io.File;

import schmitzm.geotools.io.GeoImportUtil;
import schmitzm.geotools.grid.WritableGridCoverage;
import schmitzm.data.WritableGrid;

import edu.bonn.xulu.appl.XuluRegistry;
import edu.bonn.xulu.io.AbstractFactory;

import edu.bonn.xulu.plugin.data.grid.MultiGrid;
import edu.bonn.xulu.plugin.data.grid.GridList;
import edu.bonn.xulu.plugin.io.grid.WritableGridFactory;
import edu.bonn.xulu.plugin.io.grid.gt.GridListFactory_ArcInfoAsciiGrid;

// nur fuer Doku
import org.geotools.coverage.grid.GridCoverage2D;
import edu.bonn.xulu.io.InstantiationFactory;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

/**
 * Diese Factory importiert und exportiert Instanzen des Datentyps
 * {@link MultiGrid} aus/in Dateien im ArcInfo-ASCII-Grid-Format.
 * Beim Import liefert die Factory eine auf GeoTools basierendes Raster-Liste
 * ({@link GridCoverage2D}).
 * Entsprechend koennen beim Export nur {@link MultiGrid MultiGrids}
 * verarbeitet werden, die {@link GridCoverage2D}-Instanzen in ihrer
 * Raster-Property besitzen.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class MultiGridFactory_ArcInfoAsciiGrid extends GridListFactory_ArcInfoAsciiGrid {
  ////////////////////////////////////////////////////////////////////////
  ///////////////     Implementierung von ImportFactory     //////////////
  ////////////////////////////////////////////////////////////////////////
  /**
   * Liefert den Datentyp, den die Factory erzeugt.
   * @return immer {@link MultiGrid MultiGrid.class}
   */
  public Class getImportType() {
    return MultiGrid.class;
  }

  /**
   * Importiert Raster aus Dateien im ArcInfoAsciiGrid-Format
   * und fuegt diese in einem {@link MultiGrid} zusammen.
   * @param input Eingabe-Quelle (muss ein {@link File File[]} sein!)
   * @param reg   Instanz der Xulu-Registry, ueber die eine Factory ermittelt
   *              wird, die eine Standard-Instanz von {@link MultiGrid} erzeugt, in
   *              die das Objekt importiert wird
   * @exception UnsupportedOperationException falls als Eingabe-Quelle keine
   *            Dateien angegeben werden
   * @see GeoImportUtil#readGridFromArcInfoASCII(File,CoordinateReferenceSystem)
   */
  public MultiGrid importObject(Object input, XuluRegistry reg) throws Exception {
    AbstractFactory.checkImportSourceObject(this,input,getImportSourceType(),true);

    // MultiGrid erst erzeugen, wenn ein Beispiel-Raster vorliegt
    MultiGrid mg    = null;
    // Raster importieren und in GridList einfuegen
    for (int i=0; i<((Object[])input).length; i++) {
      WritableGrid grid = WritableGridCoverage.create( GeoImportUtil.readGridFromArcInfoASCII(((File[])input)[i]), 0);
      // das erste Grid dient dem MultiGrid als Vorlage
      if ( mg == null )
        mg = new MultiGrid(grid,getWritableGridFactory());
      mg.addGrid(grid);
      // Style vom ersten Raster uebernehmen
      if ( edu.bonn.xulu.plugin.io.IOUtil.getStyleFromXuluObject( mg ) == null )
        edu.bonn.xulu.plugin.io.IOUtil.importRasterStyle( mg, ((File[])input)[i].toURI().toURL() );
    }

   return mg;
  }

  /**
   * Liefert eine Instanz von {@link GridCoverageFactory}, die auf GeoTools
   * basierende Instanzen von {@link WritableGrid} erzeugt. Mit diesen
   * Instanzen wird das SingleGrid gefuellt.
   */
  protected WritableGridFactory getWritableGridFactory() {
    return new GridCoverageFactory();
  }

}
