package edu.bonn.xulu.plugin.model.parallel;

import schmitzm.data.WritableGrid;
import schmitzm.data.property.ScalarProperty;
import schmitzm.data.property.ListProperty;
import schmitzm.data.property.ListPropertyReadAccess;
import schmitzm.data.property.MatrixProperty;

import schmitzm.temp.BaseTypeUtil;

import edu.bonn.xulu.model.ModelResource;
import edu.bonn.xulu.model.AbstractModelResource;
import edu.bonn.xulu.model.ValuePropertyResource;
import edu.bonn.xulu.model.AbstractModelContentManager;
import edu.bonn.xulu.data.XuluDataException;

import edu.bonn.xulu.plugin.data.grid.MultiGrid;

/**
 * Diese Klasse verwaltet die Ressourcen fuer das {@linkplain ClueModelParallel Clue-Modell}
 * und prueft diese auf Korrektheit.<br>
 * Die Art der benoetigten Ressourcen ist der {@linkplain ClueModelParallel Model-Doku}
 * zu entnehmen.
 * @author <a href="mailto:martin.schmitz@koeln.de">Martin Schmitz</a> (University of Bonn/Germany)
 * @version 1.0
 */
public class ClueModelContentManager extends AbstractModelContentManager {
    /** Ressource: Simulation step count */
    public  static final int    MODELSTEPS  = 0; // Nummer der Ressource
    private static final int    MODELSTEPS_MIN  = 1;
    private static final int    MODELSTEPS_MAX  = 50;
    private static final String MODELSTEPS_DESC = "Number of time steps (".concat(String.valueOf(MODELSTEPS_MIN)).concat("-").concat(String.valueOf(MODELSTEPS_MAX)).concat(")");
    /** Ressource: Type of tolerance expression for demand compliance */
    public  static final int    TOLERANCETYPE      = 1; // Nummer der Ressource
    private static final String TOLERANCETYPE_DESC = "Type of tolerance expressions ["+ClueModelParallel.TOLTYPE_PCT+" = perc.; "+ClueModelParallel.TOLTYPE_ABS+" = abs.] (integer)";
    /** Ressource: Tolerance for demand compliance for each lu type */
    public  static final int    TOLERANCESINGLE      = 2; // Nummer der Ressource
    private static final String TOLERANCESINGLE_DESC = "Demand tolerance for each single land use type (double)";
    /** Ressource: Average tolerance for demand compliance */
    public  static final int    TOLERANCEAVERAGE      = 3; // Nummer der Ressource
    private static final String TOLERANCEAVERAGE_DESC = "Average tolerance for land use demand (double)";
    /** Ressource: Toleranz fuer Bedarfe */
    public  static final int    CONVFACTQM      = 4; // Nummer der Ressource
    private static final String CONVFACTQM_DESC = "Conversion factor: demand unit to qm (double)";
    /** Ressource: Base scenario */
    public  static final int    BASESCENARIO      = 5; // Nummer der Ressource
    private static final String BASESCENARIO_DESC = "Base scenatio (float-Grid)";
    /** Ressource: Land use history of base scenario */
    public  static final int    LUHISTORY      = 6; // Nummer der Ressource
    private static final String LUHISTORY_DESC = "Land use history (float-Grid)";
    /** Ressource: LU-Conversion-Matrix */
    public  static final int    LUCONV      = 7; // Nummer der Ressource
    private static final String LUCONV_DESC = "Land use conversion matrix (integer-Matrix)";
    /** Ressource: Conversion elasticity */
    public  static final int    ELASTICITY      = 8; // Nummer der Ressource
    private static final String ELASTICITY_DESC = "Conversion elasticity (double-List)";
    // Ressource: Area restrictions
    public  static final int    AREARESTR      = 9; // Nummer der Ressource
    private static final String AREARESTR_DESC = "Area restrictions (float-Grid)";
    /** Ressource: Demand scenario */
    public  static final int    DEMAND      = 10; // Nummer der Ressource
    private static final String DEMAND_DESC = "Demand scenario (integer-Matrix)";
    /** Ressource: Static driving forces */
    public  static final int    STATDF      = 11; // Nummer der Ressource
    private static final String STATDF_DESC = "Static driving forces (float-MultiGrid)";
    /** Ressource: List of the dynamic driving force types */
    public  static final int    DYNDFTYPES      = 12; // Nummer der Ressource
    private static final String DYNDFTYPES_DESC = "Dynamic driving force types (int-List)";
    /** Ressource: Dynamic driving forces */
    public  static final int    DYNDF      = 13; // Nummer der Ressource
    private static final String DYNDF_DESC = "Dynamic driving forces (float-MultiGrid)";
    /** Ressource: Regression results */
    public  static final int    REGRESSION      = 14; // Nummer der Ressource
    private static final String REGRESSION_DESC = "Regression results (double-Matrix)";
    /** Ressource: Gewichtung der Neighborhood-Regression results */
    public  static final int    NBHWEIGHTS     = 15; // Nummer der Ressource
    private static final String NBHWEIGHTS_DESC = "Weight of neighborhood (double-List)";
    /** Ressource: Neighborhood-Settings */
    public  static final int    NBHSETTINGS      = 16; // Nummer der Ressource
    private static final String NBHSETTINGS_DESC = "Neighborhood settings (double-Matrix)";
    /** Ressource: Neighborhood-Regression results */
    public  static final int    NBHREGRESSION      = 17; // Nummer der Ressource
    private static final String NBHREGRESSION_DESC = "Regression results for neighborhood (double-Matrix)";

    /** Ressource: Temp-Raster for actual LU History */
    public  static final int    ACTLUHISTORY      = 18; // Nummer der Ressource
    private static final String ACTLUHISTORY_DESC = "Temp-Raster for actual land use history (float-Grid)";
    /** Ressource: Temp-Raster for LU Probabilities */
    public  static final int    LUPROB      = 19; // Nummer der Ressource
    private static final String LUPROB_DESC = "Temp-Raster for land use probabilities (float-MultiGrid)";
    /** Ressource: Temp-Raster for LU Probabilities */
    public  static final int    NBHPROB      = 20; // Nummer der Ressource
    private static final String NBHPROB_DESC = "Temp-Raster for neighborhood probabilities (float-MultiGrid)";
    /** Ressource: Temp-List for iteration variables */
    public  static final int    ITERVAR      = 21; // Nummer der Ressource
//    private static final String ITERVAR_DESC = "Temp-List for iteration variables (double-List)";
    private static final String ITERVAR_DESC = "Temp-List for iteration variables (float-List)";

    /** Ressource: Temp-Raster for actual LU */
    public  static final int    ACTLU      = 22; // Nummer der Ressource
    private static final String ACTLU_DESC = "Temp-Raster for actual land use (float-Grid)";
    /** Ressource: Out-Rasters for time step results */
    public  static final int    STEPRESULTS      = 23; // Nummer der Ressource
    private static final String STEPRESULTS_DESC = "Out-Rasters time step results (float-MultiGrid)";

  /**
   * Erzeugt einen neuen ContentManager fuer das {@linkplain ClueModelParallel ClueModelParallel}.
   */
  public ClueModelContentManager() {
    super(24);
    // Ressourcen festlegen
    resource[MODELSTEPS]       = new ModelStepResource(ModelResource.CATEGORY_INPUT,MODELSTEPS_DESC,MODELSTEPS_MIN,MODELSTEPS_MAX);
    resource[TOLERANCETYPE]    = new ToleranceTypeResource(ModelResource.CATEGORY_INPUT,TOLERANCETYPE_DESC);
    resource[TOLERANCESINGLE]  = new SingleToleranceResource(ModelResource.CATEGORY_INPUT,TOLERANCESINGLE_DESC);
    resource[TOLERANCEAVERAGE] = new AverageToleranceResource(ModelResource.CATEGORY_INPUT,TOLERANCEAVERAGE_DESC);
    resource[CONVFACTQM]       = new ConversionFactorQMResource(ModelResource.CATEGORY_INPUT,CONVFACTQM_DESC);
    resource[BASESCENARIO]     = new InitialScenarioResource(ModelResource.CATEGORY_INPUT,BASESCENARIO_DESC);
    resource[LUHISTORY]        = new LUHistoryResource(ModelResource.CATEGORY_INPUT,LUHISTORY_DESC);
    resource[LUCONV]           = new LUConversionMatrixResource(ModelResource.CATEGORY_INPUT,LUCONV_DESC);
    resource[ELASTICITY]       = new ConversionElasticityResource(ModelResource.CATEGORY_INPUT,ELASTICITY_DESC);
    resource[AREARESTR]        = new AreaRestrictionsResource(ModelResource.CATEGORY_INPUT,AREARESTR_DESC);
    resource[DEMAND]           = new DemandScenarioResource(ModelResource.CATEGORY_INPUT,DEMAND_DESC);
    resource[STATDF]           = new StaticDrivingForcesResource(ModelResource.CATEGORY_INPUT,STATDF_DESC);
    resource[DYNDFTYPES]       = new DynamicDrivingForceTypesResource(ModelResource.CATEGORY_INPUT,DYNDFTYPES_DESC);
    resource[DYNDF]            = new DynamicDrivingForcesResource(ModelResource.CATEGORY_INPUT,DYNDF_DESC);
    resource[REGRESSION]       = new RegressionResultsResource(ModelResource.CATEGORY_INPUT,REGRESSION_DESC);
    resource[NBHWEIGHTS]       = new NeighborhoodWeightsResource(ModelResource.CATEGORY_INPUT,NBHWEIGHTS_DESC);
    resource[NBHSETTINGS]      = new NeighborhoodSettingsResource(ModelResource.CATEGORY_INPUT,NBHSETTINGS_DESC);
    resource[NBHREGRESSION]    = new NeighborhoodRegressionResultsResource(ModelResource.CATEGORY_INPUT,NBHREGRESSION_DESC);
    resource[ACTLU]            = new ActualLUResource(ModelResource.CATEGORY_OUTPUT,ACTLU_DESC);
    resource[ACTLUHISTORY]     = new ActualLUHistoryResource(ModelResource.CATEGORY_TEMP,ACTLUHISTORY_DESC);
    resource[LUPROB]           = new LUProbabilitiesResource(ModelResource.CATEGORY_TEMP,LUPROB_DESC);
    resource[NBHPROB]          = new NeighborhoodProbabilitiesResource(ModelResource.CATEGORY_TEMP,NBHPROB_DESC);
    resource[ITERVAR]          = new IterationVariablesResource(ModelResource.CATEGORY_TEMP,ITERVAR_DESC);
    resource[STEPRESULTS]      = new StepResultsResource(ModelResource.CATEGORY_OUTPUT,STEPRESULTS_DESC);
  }

  /**
   * Prueft alle Modell-Ressourcen auf Korrektheit.
   * Neben den lokalen Bedingungen an die Ressourcen (die bereits durch
   * die Methode der Oberklasse gechecked werden), prueft diese Methode, ob
   * <ul>
   * <li>ob die Raster alle dieselbe Groesse haben, wie das Base Scenario</li>
   * <li>die MultiGrids genug Raster enthalten</li>
   * <li>die Matrizen die passenden Groessen haben</li>
   * </ul>
   */
  public void checkAndError() {
    super.checkAndError();

    // Parameter ermitteln, die fuer die Konsistenz relevant sind
    int n = getLUConversionMatrix().getSize(0);
    int m = getStaticDrivingForces().getGridCount();
    int t = getStepCount().getOneTimeReadAccess().getValueAsInt();
    WritableGrid baseGrid = (WritableGrid)getBaseScenario().getOneTimeReadAccess().getValue();
    WritableGrid testGrid = null;

    // Conversion Elasticity muss n Eintraege haben
    int ceCount = getConversionElasticity().getOneTimeReadAccess().getCount();
    if ( ceCount < n )
      throw new XuluDataException("List for conversion elasticity must have at least "+n+" entries (one for each land use types)...");
    // Area Restriction Raster muss dem Base Scenario entsprechen
    testGrid = (WritableGrid)getAreaRestrictions().getOneTimeReadAccess().getValue();
    checkRasterDim(baseGrid,BASESCENARIO_DESC,testGrid,AREARESTR_DESC);

    // Demand Scenario muss (n,t)-Matrix sein
    int[] dsSize = getDemandScenario().getSize();
    if ( dsSize[0] < n )
      throw new XuluDataException("Matrix for demand scenario must have "+n+" column (one for each land use type)...");
    if ( dsSize[1] < t )
      throw new XuluDataException("Matrix for demand scenario must have "+t+" rows (one for each time step)...");

    // Driving Forces muessen m Raster, die dem Base Scenario entsprechen,
    MultiGrid sdfGrid = getStaticDrivingForces();
    if ( sdfGrid.getGridCount() < m )
      throw new XuluDataException("MultiGrid for static driving forces must contain at leat "+m+" entries (one for each driving force)...");
    // erstes befuelltes Raster checken
    checkRasterDim(baseGrid,BASESCENARIO_DESC,sdfGrid.getFirstNotNullGrid(),STATDF_DESC);

    // Liste der Dynamischen DF darf nur m Eintraege haben und Werte zwischen 0 und m enthalten
    int ddfCount = 0;
    if ( getDynamicDrivingForceTypes() != null ) {
      ListPropertyReadAccess ddf = getDynamicDrivingForceTypes().getReadAccess(this);
      ddfCount = ddf.getCount();
      if ( ddfCount > m )
        throw new XuluDataException("List of dynamic driving forces must not have more than "+m+" entries...");
      for (int i=0; i<ddfCount; i++)
        if ( ddf.getValueAsInt(i) < 0 || ddf.getValueAsInt(i) >= m )
          throw new XuluDataException("Only values between 0 and "+(m-1)+" are allowed in list of dynamic driving forces...");
      ddf.release();
    }

    // Werden dyn. DF verwendet, muessen auch Raster angegeben werden
    MultiGrid ddfGrids = getDynamicDrivingForces();
    if ( ddfCount > 0 && ddfGrids == null )
      throw new XuluDataException("Dynamic Driving Forces are uses! Resource '"+DYNDF_DESC+"' is not allowed to be null...");
    // Es muessen mind. t Raster fuer jeden dyn. DF angegeben werden
    if ( ddfCount > 0 && ddfGrids.getGridCount() < ddfCount*t )
      throw new XuluDataException("MultiGrid for dynamic driving forces must contain at leat "+(ddfCount*t)+" entries (one for each driving force at every time step)...");
    // Fuer jeden DF muss dieselbe Anzahl an Grids enthalten sein
    if ( ddfCount > 0 && ddfGrids.getGridCount() % ddfCount != 0 )
      throw new XuluDataException("MultiGrid for dynamic driving forces must contain the same number of grids for all driving forces...");

    // Regression Results muss (m+1,n)-Matrix sein
    int[] rrSize = getRegressionResults().getSize();
    if ( rrSize[0] < m+1 )
      throw new XuluDataException("Matrix for regression results must have "+(m+1)+" columns (one for the constant and one for each driving force)...");
    if ( rrSize[1] < n )
      throw new XuluDataException("Matrix for regression results must have "+n+" rows (one for each land use type)...");

    // Wenn angegeben, muss die Liste der Neighborhood Weights mind. n Eintraege haben
    ListProperty nbhWeights = getNeighborhoodWeights();
    if ( nbhWeights != null ) {
      if ( nbhWeights.getOneTimeReadAccess().getCount() < n )
        throw new XuluDataException("List of neighborhood weights must have at least "+n+" entries (one for each land use type)...");
    }

    // Wenn angegeben, muss die Matrix der Neighborhood Settings in der 3. Dimension
    // die Groessen n haben
    if ( nbhWeights != null ) {
      if ( getNeighborhoodSettings() == null )
        throw new XuluDataException("Neighborhood settings must be specified...");
      int[] nbSize = getNeighborhoodSettings().getSize();
      if ( nbSize[0]%2 == 0 || nbSize[1]%2 == 0 )
        throw new XuluDataException("Neighborhood settings must be specified in matrix of uneven size in dimension 1 and 2 (the center is the considered cell)...");
      if ( getNeighborhoodSettings().getSize(2) < n )
        throw new XuluDataException(n+" neighborhood settings must be specified in matrix (one for each land use type)...");

    }

    // Wenn angegeben, muss die Matrix der Neighborhood Regression eine
    // (n+1,n)-Matrix sein
    if ( nbhWeights != null ) {
      if ( getNeighborhoodRegression() == null )
        throw new XuluDataException("Neighborhood regression must be specified...");
      int[] nbSize = getNeighborhoodRegression().getSize();
      if ( nbSize[0] < n+1 )
        throw new XuluDataException("Matrix for neighborhood regression results must have "+(n+1)+" columns (one for the constant and one for each land use type)...");
      if ( nbSize[1] < n )
        throw new XuluDataException("Matrix for neighborhood regression results must have "+n+" rows (one for each land use type)...");
    }

    // Raster fuer die aktuelle LU muss dem Base Scenario entsprechen
    testGrid = (WritableGrid)getActLUResult().getOneTimeReadAccess().getValue();
    checkRasterDim(baseGrid,BASESCENARIO_DESC,testGrid,ACTLU_DESC);

    // Raster fuer die aktuelle LU History muss dem Base Scenario entsprechen
    testGrid = (WritableGrid)getActLUHistory().getOneTimeReadAccess().getValue();
    checkRasterDim(baseGrid,BASESCENARIO_DESC,testGrid,ACTLUHISTORY_DESC);

//    // MultiGrid fuer die LU-Wahrscheinlichkeiten muss n Raster enthalten,
//    // die dem Base Scenario entsprechen
//    MultiGrid luProbGrids = this.getLUProbabilities();
//    if ( luProbGrids.getGridCount() < n )
//      throw new XuluDataException("MultiGrid for land use probabilities must contain at leat "+n+" grids (one for each land use type)...");
//    checkRasterDim(baseGrid,BASESCENARIO_DESC,luProbGrids.getGrid(0),LUPROB_DESC);
    // MultiGrid fuer die LU-Wahrscheinlichkeiten muss dem Base Scenario entsprechen
    // Die n Inhalt-Grids werden bei der Modell-Initialisierung dynamisch erzeugt!
    MultiGrid luProbGrids = this.getLUProbabilities();
    checkRasterDim(baseGrid,BASESCENARIO_DESC,luProbGrids.getTemplateGrid(),LUPROB_DESC);

//    // MultiGrid fuer die Nachbarschafts-Wahrscheinlichkeiten muss n Raster enthalten,
//    // die dem Base Scenario entsprechen
//    if ( nbhWeights != null ) {
//      MultiGrid nbhProbGrids = this.getNeighborhoodProbabilities();
//      if ( nbhProbGrids == null )
//        throw new XuluDataException("Temp. raster for neighborhood probabilities must be specified...");
//      if ( nbhProbGrids.getGridCount() < n )
//        throw new XuluDataException("MultiGrid for neighborhood probabilities must contain at leat "+n+" grids (one for each land use type)...");
//      checkRasterDim(baseGrid,BASESCENARIO_DESC,nbhProbGrids.getGrid(0),LUPROB_DESC);
//    }
    // MultiGrid fuer die Nachbarschafts-Wahrscheinlichkeiten muss dem Base Scenario entsprechen
    // Die n Inhalt-Grids werden bei der Modell-Initialisierung dynamisch erzeugt!
    if ( nbhWeights != null ) {
      MultiGrid nbhProbGrids = this.getNeighborhoodProbabilities();
      if ( nbhProbGrids == null )
        throw new XuluDataException("Temp. raster for neighborhood probabilities must be specified...");
      checkRasterDim(baseGrid,BASESCENARIO_DESC,nbhProbGrids.getTemplateGrid(),LUPROB_DESC);
    }

//    // Wenn angegeben, muss das MultiGrid fuer die Zeitschritt-Ergebnisse
//    // t Raster enthalten, die dem Base Scenario entsprechen
//    MultiGrid resultGrids = this.getStepLUResults();
//    if (resultGrids != null) {
//      if ( resultGrids.getGridCount() < t )
//        throw new XuluDataException("MultiGrid for time step results must contain at leat "+t+" grids (one for each time step)...");
//      checkRasterDim(baseGrid, BASESCENARIO_DESC, resultGrids.getGrid(0),STEPRESULTS_DESC);
//    }
    // Das MultiGrid fuer die Zeitschritt-Ergebnisse dem Base Scenario entsprechen
    // Die t Inhalt-Grids werden waehrend des Modellablaufs dynamisch erzeugt!
    MultiGrid resultGrids = this.getStepLUResults();
    checkRasterDim(baseGrid, BASESCENARIO_DESC, resultGrids.getTemplateGrid(),STEPRESULTS_DESC);
  }

  /**
   * Vergleicht zwei Raster auf gleiche Struktur (Hoehe, Breite, Zell-Hoehe,
   * Zell-Breite, Sample-Type).
   */
  private static void checkRasterDim(WritableGrid grid1, String desc1, WritableGrid grid2, String desc2) {
    if ( grid1.getWidth() != grid2.getWidth() )
      throw new XuluDataException(createCompareMessage("Grid width",desc1,desc2));
    if ( grid1.getHeight() != grid2.getHeight() )
      throw new XuluDataException(createCompareMessage("Grid height",desc1,desc2));
    if ( grid1.getCellWidth() != grid2.getCellWidth() )
      throw new XuluDataException(createCompareMessage("Cell width",desc1,desc2));
    if ( grid1.getCellHeight() != grid2.getCellHeight() )
      throw new XuluDataException(createCompareMessage("Cell height",desc1,desc2));
    if ( grid1.getSampleType() != grid2.getSampleType() )
      throw new XuluDataException(createCompareMessage("Grid sample-type",desc1,desc2));
  }

  /**
   * Erzeugt eine Fehlermeldung fuer eine bestimmte inkompatible Raster-Struktur.
   */
  private static String createCompareMessage(String compType, String objDesc1, String objDesc2) {
    return compType.concat(" of '").concat(objDesc1).concat("' is incompatible to '").concat(objDesc2).concat("'...");
  }

  //////////////////////////////////////////////////////////////////////
  ///////   vereinfachende Zugriffsmethoden auf Ressourcen   ///////////
  //////////////////////////////////////////////////////////////////////
  /** Liefert Zugriff auf die Ressource "Time Steps". */
  public ScalarProperty getStepCount() {
    return (ScalarProperty)getResourceData(this.MODELSTEPS);
  }

  /** Liefert Zugriff auf die Ressource "Art der Abweichungsangaben". */
  public ScalarProperty getDemandToleranceType() {
    return (ScalarProperty)getResourceData(this.TOLERANCETYPE);
  }

  /** Liefert Zugriff auf die Ressource "Einzel-Abweichung von Bedarfen". */
  public ScalarProperty getSingleDemandTolerance() {
    return (ScalarProperty)getResourceData(this.TOLERANCESINGLE);
  }

  /** Liefert Zugriff auf die Ressource "Durchschnitts-Abweichung von Bedarfen". */
  public ScalarProperty getAverageDemandTolerance() {
    return (ScalarProperty)getResourceData(this.TOLERANCEAVERAGE);
  }


  /** Liefert Zugriff auf die Ressource "Umrechnungsfaktor Bedarfseinheit -> qm". */
  public ScalarProperty getConversionFactor() {
    return (ScalarProperty)getResourceData(this.CONVFACTQM);
  }

  /** Liefert Zugriff auf die Ressource "Base Scenario". */
  public ScalarProperty getBaseScenario() {
    return (ScalarProperty)getResourceData(this.BASESCENARIO);
  }

  /** Liefert Zugriff auf die Ressource "Land use History". */
  public ScalarProperty getLUHistory() {
    return (ScalarProperty)getResourceData(this.LUHISTORY);
  }

  /** Liefert Zugriff auf die Ressource "LU Conversion Matrix". */
  public MatrixProperty getLUConversionMatrix() {
    return (MatrixProperty)getResourceData(this.LUCONV);
  }

  /** Liefert Zugriff auf die Ressource "Conversion Elasticity". */
 public ListProperty getConversionElasticity() {
    return (ListProperty)getResourceData(this.ELASTICITY);
  }

  /** Liefert Zugriff auf die Ressource "Area Restrictions". */
  public ScalarProperty getAreaRestrictions() {
    return (ScalarProperty)getResourceData(this.AREARESTR);
  }

  /** Liefert Zugriff auf die Ressource "Demand Scenario". */
  public MatrixProperty getDemandScenario() {
    return (MatrixProperty)getResourceData(this.DEMAND);
  }

  /** Liefert Zugriff auf die Ressource "Static Driving Forces". */
  public MultiGrid getStaticDrivingForces() {
    return (MultiGrid)getResourceData(this.STATDF);
  }

  /** Liefert Zugriff auf die Ressource "Dynamic Driving Force Types". */
 public ListProperty getDynamicDrivingForceTypes() {
    return (ListProperty)getResourceData(this.DYNDFTYPES);
  }

  /** Liefert Zugriff auf die Ressource "Dynamic Driving Forces". */
 public MultiGrid getDynamicDrivingForces() {
    return (MultiGrid)getResourceData(this.DYNDF);
  }

  /** Liefert Zugriff auf die Ressource "Regression Results". */
  public MatrixProperty getRegressionResults() {
    return (MatrixProperty)getResourceData(this.REGRESSION);
  }

  /** Liefert Zugriff auf die Ressource "Neighborhood Weights". */
  public ListProperty getNeighborhoodWeights() {
    return (ListProperty)getResourceData(this.NBHWEIGHTS);
  }

  /** Liefert Zugriff auf die Ressource "Neighborhood Settings". */
  public MatrixProperty getNeighborhoodSettings() {
    return (MatrixProperty)getResourceData(this.NBHSETTINGS);
  }

  /** Liefert Zugriff auf die Ressource "Neighborhood Regression Results". */
  public MatrixProperty getNeighborhoodRegression() {
    return (MatrixProperty)getResourceData(this.NBHREGRESSION);
  }

  /** Liefert Zugriff auf die Ausgabe-Ressource "Aktuelle Landnutzung". */
  public ScalarProperty getActLUResult() {
    return (ScalarProperty)getResourceData(this.ACTLU);
  }

  /** Liefert Zugriff auf die Ausgabe-Ressource "LU-Wahrscheinlichkeiten". */
  public MultiGrid getLUProbabilities() {
    return (MultiGrid)getResourceData(this.LUPROB);
  }

  /** Liefert Zugriff auf die Ausgabe-Ressource "Nachbarschafts-Wahrscheinlichkeiten". */
  public MultiGrid getNeighborhoodProbabilities() {
    return (MultiGrid)getResourceData(this.NBHPROB);
  }

  /** Liefert Zugriff auf die Ausgabe-Ressource "Iterationsvariablen". */
  public ListProperty getIterationVariables() {
    return (ListProperty)getResourceData(this.ITERVAR);
  }

  /** Liefert Zugriff auf die Ressource "Aktuelle Land use History". */
  public ScalarProperty getActLUHistory() {
    return (ScalarProperty)getResourceData(this.ACTLUHISTORY);
  }

  /** Liefert Zugriff auf die Ausgabe-Ressource "LU zu jedem Zeitschritt". */
  public MultiGrid getStepLUResults() {
    return (MultiGrid)getResourceData(this.STEPRESULTS);
  }

  //////////////////////////////////////////////////////////////////////
  ///////////   Klassen fuer die benoetigten Ressourcen   //////////////
  //////////////////////////////////////////////////////////////////////
  /**
   * Ressource fuer Simulationsschritte.
   */
  private class ModelStepResource extends ValuePropertyResource {
    private int stepMin = 0;
    private int stepMax = 0;

    public ModelStepResource(int cat,String desc, int stepMin, int stepMax) {
      super(cat,desc,ScalarProperty.class,Number.class,false);
      this.stepMin = stepMin;
      this.stepMax = stepMax;
    }

    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // muss als Datentyp Long oder Integer haben
      if ( !BaseTypeUtil.isNumeric(((ScalarProperty)data).getType()) ||
            BaseTypeUtil.isDecimal(((ScalarProperty)data).getType()) )
        throwDataException("Long- or Integer-Property");
    }

    public void checkAndError(Object data) {
      super.checkAndError(data);
      // muss zwischen 1 und 20 liegen
      checkRangeAndError(this,((ScalarProperty)data).getOneTimeReadAccess().getValueAsLong(),stepMin,stepMax);
    }
  }

  /**
   * Ressource fuer Art der Toleranz-Angaben (prozentual, absolut).
   */
  private class ToleranceTypeResource extends ValuePropertyResource {
    public ToleranceTypeResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,Integer.class,false);
    }
    public void checkAndError(Object data) {
      super.checkAndError(data);
      // Flag muss 0 (prozentual) oder 1 (absolut) sein
      checkRangeAndError(this,((ScalarProperty)data).getOneTimeReadAccess().getValueAsInt(),0,1);
    }
  }

  /**
   * Ressource fuer Toleranz der einzelnen Bedarfe.
   */
  private class SingleToleranceResource extends ValuePropertyResource {
    public SingleToleranceResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,Number.class,false);
    }
    public void checkAndError(Object data) {
      super.checkAndError(data);
      // muss groesser 0 sein
      checkRangeAndError(this,((ScalarProperty)data).getOneTimeReadAccess().getValueAsDouble(),0,Double.MAX_VALUE);
    }
  }

  /**
   * Ressource fuer Durchschnitts-Toleranz der einzelnen Bedarfe.
   */
  private class AverageToleranceResource extends ValuePropertyResource {
    public AverageToleranceResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,Number.class,false);
    }
    public void checkAndError(Object data) {
      super.checkAndError(data);
      // muss groesser 0 sein
      checkRangeAndError(this,((ScalarProperty)data).getOneTimeReadAccess().getValueAsDouble(),0,Double.MAX_VALUE);
    }
  }

  /**
   * Ressource fuer Umrechnungsfaktor Bedarfseinheit >> Quadratmeter.
   */
  private class ConversionFactorQMResource extends ValuePropertyResource {
    public ConversionFactorQMResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,Double.class,false);
    }
  }

  /**
   * Ressource fuer Ausgangsszenario.
   */
  private class InitialScenarioResource extends ValuePropertyResource {
    public InitialScenarioResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,WritableGrid.class,false);
    }
  }

  /**
   * Ressource fuer LU-History.
   */
  private class LUHistoryResource extends ValuePropertyResource {
    public LUHistoryResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,WritableGrid.class,false);
    }
  }

  /**
   * Ressource fuer LU-Conversion-Matrix.
   */
  private class LUConversionMatrixResource extends ValuePropertyResource {
    public LUConversionMatrixResource(int cat,String desc) {
      super(cat,desc,MatrixProperty.class,Integer.class,false);
    }
    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // Matrix muss mindestens 2-dimensional sein
      if ( ((MatrixProperty)data).getDimension() < 2 )
        throwDataException("At least 2-dimensional Matrix");
    }
  }

  /**
   * Ressource fuer Conversion Elasticity.
   */
  private class ConversionElasticityResource extends ValuePropertyResource {
    public ConversionElasticityResource(int cat,String desc) {
      super(cat,desc,ListProperty.class,Double.class,false);
    }

    public void checkAndError(Object data) {
      super.checkAndError(data);
      // Alle Werte der Liste muessen zwischen 0 und 1 liegen
      ListPropertyReadAccess a = ((ListProperty)data).getReadAccess(this);
      for (int i=0; i<a.getCount(); i++) {
        double v = a.getValueAsDouble(i);
        if ( v < 0 || 1 < v ) {
          a.release();
          throwDataException("ListProperty with values between 0 and 1");
        }
      }
      a.release();
    }
  }

  /**
   * Ressource fuer Area Restrictions.
   */
  private class AreaRestrictionsResource extends ValuePropertyResource {
    public AreaRestrictionsResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,WritableGrid.class,false);
    }
  }

  /**
   * Ressource fuer Demand Scenario.
   */
  private class DemandScenarioResource extends ValuePropertyResource {
    public DemandScenarioResource(int cat,String desc) {
      super(cat,desc,MatrixProperty.class,Integer.class,false);
    }

    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // Matrix muss mindestens 2-dimensional sein
      if ( ((MatrixProperty)data).getDimension() < 2 )
        throwDataException("At least 2-dimensional Matrix");
    }
  }

  /**
   * Ressource fuer Static Driving Forces.
   */
  private class StaticDrivingForcesResource extends AbstractModelResource {
    public StaticDrivingForcesResource(int cat,String desc) {
      super(cat,desc,MultiGrid.class,false);
    }

    public void checkTypeAndError(Object data) {
      if ( data!=null && !(data instanceof MultiGrid) ||
           data==null && !nullable)
        throwDataException("MultiGrid");
    }

    public void checkAndError(Object data) {
      checkTypeAndError(data);
    }
  }

  /**
   * Ressource fuer Dynamic Driving Force Types.
   */
  private class DynamicDrivingForceTypesResource extends ValuePropertyResource {
    public DynamicDrivingForceTypesResource(int cat,String desc) {
      super(cat,desc,ListProperty.class,Integer.class,true);
    }
  }

  /**
   * Ressource fuer Dynamic Driving Forces.
   */
  private class DynamicDrivingForcesResource extends AbstractModelResource {
    public DynamicDrivingForcesResource(int cat,String desc) {
      super(cat,desc,MultiGrid.class,true);
    }

    public void checkTypeAndError(Object data) {
      if ( data != null && !(data instanceof MultiGrid) ||
           data == null && !nullable)
        throwDataException("MultiGrid");
    }

    public void checkAndError(Object data) {
      checkTypeAndError(data);
    }
  }

  /**
   * Ressource fuer Regression Results.
   */
  private class RegressionResultsResource extends ValuePropertyResource {
    public RegressionResultsResource(int cat,String desc) {
      super(cat,desc,MatrixProperty.class,Double.class,false);
    }
    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // Matrix muss mindestens 2-dimensional sein
      if ( ((MatrixProperty)data).getDimension() < 2 )
        throwDataException("At least 2-dimensional Matrix");
    }
  }

  /**
   * Ressource fuer Neighborhood Weight.
   */
  private class NeighborhoodWeightsResource extends ValuePropertyResource {
    public NeighborhoodWeightsResource(int cat,String desc) {
      super(cat,desc,ListProperty.class,Double.class,true);
    }
  }

  /**
   * Ressource fuer Neighborhood Settings.
   */
  private class NeighborhoodSettingsResource extends ValuePropertyResource {
    public NeighborhoodSettingsResource(int cat,String desc) {
      super(cat,desc,MatrixProperty.class,Double.class,true);
    }
    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // Matrix muss mindestens 3-dimensional sein
      if ( data!=null && ((MatrixProperty)data).getDimension() < 3 )
        throwDataException("At least 3-dimensional Matrix");
    }
  }

  /**
   * Ressource fuer Neighborhood Regression Results.
   */
  private class NeighborhoodRegressionResultsResource extends ValuePropertyResource {
    public NeighborhoodRegressionResultsResource(int cat,String desc) {
      super(cat,desc,MatrixProperty.class,Double.class,true);
    }
    public void checkTypeAndError(Object data) {
      super.checkTypeAndError(data);
      // Matrix muss mindestens 2-dimensional sein
      if ( data!=null && ((MatrixProperty)data).getDimension() < 2 )
        throwDataException("At least 2-dimensional Matrix");
    }
  }

  /**
   * Ressource fuer die simulierte LU.
   */
  private class ActualLUResource extends ValuePropertyResource {
    public ActualLUResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,WritableGrid.class,false);
    }
  }

  /**
   * Ressource fuer aktuelle LU-History.
   */
  private class ActualLUHistoryResource extends ValuePropertyResource {
    public ActualLUHistoryResource(int cat,String desc) {
      super(cat,desc,ScalarProperty.class,WritableGrid.class,false);
    }
  }

  /**
   * Ressource fuer die errechneten LU-Wahrscheinlichkeiten.
   */
  private class LUProbabilitiesResource extends AbstractModelResource {
    public LUProbabilitiesResource(int cat,String desc) {
      super(cat,desc,MultiGrid.class,false);
    }
    public void checkTypeAndError(Object data) {
      if ( data!=null && !(data instanceof MultiGrid) ||
           data==null && !nullable )
        throwDataException("MultiGrid");
    }
    public void checkAndError(Object object) {
      checkTypeAndError(object);
    }
  }

  /**
   * Ressource fuer die errechneten Nachbarschafts-Wahrscheinlichkeiten.
   */
  private class NeighborhoodProbabilitiesResource extends AbstractModelResource {
    public NeighborhoodProbabilitiesResource(int cat,String desc) {
      super(cat,desc,MultiGrid.class,true);
    }
    public void checkTypeAndError(Object data) {
      if ( data!=null && !(data instanceof MultiGrid) ||
           data==null && !nullable )
        throwDataException("MultiGrid");
    }
    public void checkAndError(Object object) {
      checkTypeAndError(object);
    }
  }

  /**
   * Ressource fuer Iterationsvariablen.
   */
  private class IterationVariablesResource extends ValuePropertyResource {
    public IterationVariablesResource(int cat,String desc) {
//      super(cat,desc,ListProperty.class,Double.class,false);
      super(cat,desc,ListProperty.class,Float.class,false);    }
  }

  /**
   * Ressource fuer die einzelnen Zeit-Schritt-Ergebnisse.
   */
  private class StepResultsResource extends AbstractModelResource {
    public StepResultsResource(int cat,String desc) {
      super(cat,desc,MultiGrid.class,false);
    }

    public void checkTypeAndError(Object data) {
      if ( data!=null && !(data instanceof MultiGrid) ||
           data==null && !nullable )
        throwDataException("MultiGrid");
    }

    public void checkAndError(Object object) {
      checkTypeAndError(object);
    }
  }

  /**
   * Prueft, ob ein Wert in einem bestimmten Bereich liegt. Wenn nicht, wird
   * fuer die angegebene Ressource eine Exception geworfen.
   */
  private static void checkRangeAndError(ModelResource r, double value, double min, double max) {
    if ( value < min ||  max < value )
     throw new XuluDataException("'".concat(r.getDesc()).concat("' nicht zul�ssig!"));
  }

}
