package edu.bonn.xulu.plugin.model.sleuth;

import java.awt.Rectangle;
import java.util.HashMap;

import edu.bonn.xulu.model.AbstractStepModel;
import edu.bonn.xulu.model.XuluModel;
import edu.bonn.xulu.plugin.data.grid.MultiGrid;

import schmitzm.data.WritableGrid;
import schmitzm.data.property.PropertyReadAccess;
import schmitzm.data.property.PropertyWriteAccess;
import schmitzm.data.property.ScalarProperty;

/**
 *  This very simple model looks at every grid cell and takes the average 
 *  over a the surrounding cells in a specified neighborhood. It was implemented
 *  as demonstration of the Xulu / V functionality. You can see the parallel version
 *  of this model in {@link AverageNeighborhoodParallelDemoModel} and also a tuned
 *  version using late-merging support in {@link AVNTuned}. 
 * <ol>
 * <li><b>Input Grid ({@code inputGrid}):</b> A grid with input values </li>
 * <li><b>Output Grid ({@code outputGrid}):</b> A grid where the output is
 * stored </li>
 * <li><b>Number of steps ({@code steps}):</b> the number of steps to
 * compute. </li>
 * </ol>
 * 
 * @see AverageNeighborhoodContentManager
 * @author Dominik Appl
 * @version 1.0
 */
public class UrbanGrowthModelSelfModifying extends AbstractStepModel {

	protected UrbanGrowthModelSelfModifyingContentManager contManager;

	// ********** Lese/Schreibrechte, die fuer den gesamten ***********
	// ********** Modellanlauf gehalten werden muessen ***********
	private PropertyReadAccess RA_inputGrid = null; // Input Grid
	private PropertyWriteAccess WA_outputGrid = null; // Output Grid
	private PropertyReadAccess RA_steps = null; // Number of steps
	private PropertyReadAccess RA_areaRestr = null; //Area Restrictions
	private PropertyReadAccess RA_spread = null;	//Spread-Koeffizient
	private PropertyReadAccess RA_disp = null;		//Dispersions-Koeffizient
	private PropertyReadAccess RA_breed = null;		//Breed-Koeffizient
	private PropertyReadAccess RA_roadGravity = null;	//Road-Gravity-Koeffizient
	private PropertyReadAccess RA_slope = null;	//Slope-Koeffizient
	private PropertyReadAccess RA_criticalSlope = null;	//Critical Slope Value
	private PropertyReadAccess RA_roadGrid = null;	//Road Grid
	private PropertyReadAccess RA_slopeGrid = null;	//Slope Grid
	private PropertyWriteAccess WA_outStep = null;	// Step Results
	private PropertyReadAccess RA_roadGravSens = null;	//RoadGravity Sensitivity
	private PropertyReadAccess RA_slopeSens = null;	//Slope Sensitivity
	private PropertyReadAccess RA_criticalLow = null;	//Critical Low
	private PropertyReadAccess RA_criticalHigh = null;	//Critical High
	private PropertyReadAccess RA_Boom = null;	//Boom
	private PropertyReadAccess RA_Bust = null;	//Bust

	// **************** Variablen mit denen gearbeitet wird *******************
	private WritableGrid inputGrid = null; // Input Grid
	private WritableGrid outputGrid = null; // Output Grid
	private PropertyReadAccess steps = null; // Number of steps
	private WritableGrid areaRestr = null; //Area Restrictions
	protected double spread = 0;	//Spread-Koeffizient
	protected double disp = 0;	//Dispersions-Koeffizient
	protected double breed = 0;	//Breed-Koeffizient
	protected double roadGravity = 0;	//Road-Gravity-Koeffizient
	protected double slope = 0;	//Slope-Koeffizient
	protected double criticalSlope = 0;	//Critical Slope Value
	private WritableGrid roadGrid = null;	//Road Grid
	private WritableGrid slopeGrid = null;	//Slope Grid
	private MultiGrid outStep = null;	//Step Results
	protected double roadGravSens = 0;	//RoadGravity Sensitivity
	protected double slopeSens = 0;	//Slope Sensitivity
	protected double criticalLow = 0;	//critical Low
	protected double criticalHigh = 0;	//critical High
	protected double boom = 0;	//Boom
	protected double bust = 0;	//Bust
		
	private  static final int[][] NEIGHBOR = new int[][] {
	      {-1,-1}, {0,-1}, {1,-1},
	      {-1, 0},         {1, 0},
	      {-1, 1}, {0, 1}, {1, 1}
	  };
	
	protected  HashMap<Number,Number> slopeLUT;
	private int slopeMax = 0;
	private int totalPixels = 0;
	private int excldCount = 0;
	
	/**
	 * Erzeugt eine neue Instanz des Modells.
	 */
	public UrbanGrowthModelSelfModifying() {
		super(new UrbanGrowthModelSelfModifyingContentManager());
		this.contManager = (UrbanGrowthModelSelfModifyingContentManager) super.contManager;
	}

	
	/**
	 * Initializes the model. Like in the init method of every {@link XuluModel}
	 * the resources are initalized. 
	 */
	public void performModelInit() {
		if (contManager == null)
			return;
		// Zugriffsrechte aus Ressourcen/Propertys holen
		RA_inputGrid = null;
		if (contManager.getResource(0).getData() != null)
			RA_inputGrid = ((ScalarProperty) contManager.getResource(0)
					.getData()).getReadAccess(this);
		WA_outputGrid = null;
		if (contManager.getResource(1).getData() != null)
			WA_outputGrid = ((ScalarProperty) contManager.getResource(1)
					.getData()).getWriteAccess(this);
		RA_steps = null;
		if (contManager.getResource(2).getData() != null)
			RA_steps = ((ScalarProperty) contManager.getResource(2).getData())
					.getReadAccess(this);
		RA_areaRestr = null;
		if (contManager.getResource(3).getData() != null)
			RA_areaRestr = ((ScalarProperty) contManager.getResource(3)
					.getData()).getReadAccess(this);
		RA_spread = null;
		if (contManager.getResource(4).getData() != null)
			RA_spread = ((ScalarProperty) contManager.getResource(4).getData())
					.getReadAccess(this);
		RA_disp = null;
		if (contManager.getResource(5).getData() != null)
			RA_disp = ((ScalarProperty) contManager.getResource(5).getData())
					.getReadAccess(this);
		RA_breed = null;
		if (contManager.getResource(6).getData() != null)
			RA_breed = ((ScalarProperty) contManager.getResource(6).getData())
					.getReadAccess(this);
		RA_roadGravity = null;
		if (contManager.getResource(7).getData() != null)
			RA_roadGravity = ((ScalarProperty) contManager.getResource(7).getData())
					.getReadAccess(this);
		RA_slope = null;
		if (contManager.getResource(8).getData() != null)
			RA_slope = ((ScalarProperty) contManager.getResource(8).getData())
					.getReadAccess(this);
		RA_criticalSlope = null;
		if (contManager.getResource(9).getData() != null)
			RA_criticalSlope = ((ScalarProperty) contManager.getResource(9).getData())
					.getReadAccess(this);
		RA_roadGrid = null;
		if (contManager.getResource(10).getData() != null)
			RA_roadGrid = ((ScalarProperty) contManager.getResource(10)
					.getData()).getReadAccess(this);
		RA_slopeGrid = null;
		if (contManager.getResource(11).getData() != null)
			RA_slopeGrid = ((ScalarProperty) contManager.getResource(11)
					.getData()).getReadAccess(this);
		RA_roadGravSens = null;
		if (contManager.getResource(13).getData() != null)
			RA_roadGravSens = ((ScalarProperty) contManager.getResource(13).getData())
					.getReadAccess(this);
		RA_slopeSens = null;
		if (contManager.getResource(14).getData() != null)
			RA_slopeSens = ((ScalarProperty) contManager.getResource(14).getData())
					.getReadAccess(this);
		RA_criticalLow = null;
		if (contManager.getResource(15).getData() != null)
			RA_criticalLow = ((ScalarProperty) contManager.getResource(15).getData())
					.getReadAccess(this);
		RA_criticalHigh = null;
		if (contManager.getResource(16).getData() != null)
			RA_criticalHigh = ((ScalarProperty) contManager.getResource(16).getData())
					.getReadAccess(this);
		RA_Boom = null;
		if (contManager.getResource(17).getData() != null)
			RA_Boom = ((ScalarProperty) contManager.getResource(17).getData())
					.getReadAccess(this);
		RA_Bust = null;
		if (contManager.getResource(18).getData() != null)
			RA_Bust = ((ScalarProperty) contManager.getResource(18).getData())
					.getReadAccess(this);

		// Variablen belegen mit denen gearbeitet wird
		inputGrid = (WritableGrid) RA_inputGrid.getValue();
		outputGrid = (WritableGrid) WA_outputGrid.getValue();
		steps = RA_steps;
		areaRestr = (WritableGrid) RA_areaRestr.getValue();
		
		this.stepCount = steps.getValueAsInt();
	   // this.spread = RA_spread.getValueAsFloat();
	    spread  = RA_spread.getValueAsDouble();
	    disp	= RA_disp.getValueAsDouble();
	    breed	= RA_breed.getValueAsDouble();
	    roadGravity = RA_roadGravity.getValueAsDouble();
	    slope	= RA_slope.getValueAsDouble();
	    criticalSlope = RA_criticalSlope.getValueAsDouble();
	    roadGrid = (WritableGrid) RA_roadGrid.getValue();
	    slopeGrid = (WritableGrid) RA_slopeGrid.getValue();
	    outStep = (MultiGrid)contManager.getResource(12).getData();
	    roadGravSens = RA_roadGravSens.getValueAsDouble();
	    slopeSens = RA_slopeSens.getValueAsDouble();
	    criticalLow = RA_criticalLow.getValueAsDouble();
	    criticalHigh = RA_criticalHigh.getValueAsDouble();
	    boom = RA_Boom.getValueAsDouble();
	    bust = RA_Bust.getValueAsDouble();

		int startAnz = 0;
		for (int i = 0; i < inputGrid.getWidth();i++ ) {
			for (int ii = 0; ii < inputGrid.getHeight(); ii++ ) {
				//if (inputGrid.getRasterSampleAsFloat(i,ii) == 1f);
				float val = inputGrid.getRasterSampleAsFloat(i,ii);
				if (val == 1.0 ) {
					val = 1f;
					startAnz++;
				}
					else val = 0f;
				
				inputGrid.setRasterSample(val, inputGrid.getMinX() + i, inputGrid.getMinY() + ii);
				outputGrid.setRasterSample(val, inputGrid.getMinX() + i, inputGrid.getMinY() + ii);
			}
		}
		for (int x = 0; x < inputGrid.getWidth();x++ ) 
			for (int y = 0; y < inputGrid.getHeight(); y++ ) {
				if ( x < inputGrid.getMinX() || y < inputGrid.getMinY() || x > inputGrid.getMinX()+inputGrid.getWidth()-1 || y > inputGrid.getMinY()+inputGrid.getHeight()-1 ||
			            Float.isNaN(outputGrid.getRasterSampleAsFloat(x,y)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
			             continue;
				totalPixels++;
				if (areaRestr.getRasterSampleAsFloat(x,y) != 0 && !Float.isNaN(areaRestr.getRasterSampleAsFloat(x,y)))
		        	//continue;	//wenn in Area Restriction oder wenn schon Siedlung da ist, mach weiter
				//if (inputGrid.getRasterSampleAsFloat(i,ii) == 1f);
					excldCount++;
			}
		
		statusOut.println("Anzahl der startenden Individuen: "+startAnz);
		statusOut.println(NEIGHBOR.length);
		statusOut.println("Total: "+totalPixels+", Excluded: "+excldCount);
		
		
		for (int s = 0; s < inputGrid.getWidth();s++){
			for(int ss = 0; ss < inputGrid.getHeight();ss++){
				float slopeVal = slopeGrid.getRasterSampleAsFloat(s,ss);
				if (slopeVal > slopeMax){
					slopeMax = (int) slopeVal;
				} 
			}
		}
		statusOut.println("Maximale Hangneigung: "+slopeMax);
		
	}

	/**
	* like in every model: frees the resources
	 */
	public void performModelDispose() {

		// Ressourcen wieder freigeben
		releaseAccess(RA_inputGrid);
		releaseAccess(WA_outputGrid);
		releaseAccess(RA_steps);
		releaseAccess(RA_areaRestr);
		releaseAccess(RA_spread);
		releaseAccess(RA_disp);
		releaseAccess(RA_breed);
		releaseAccess(RA_roadGravity);
		releaseAccess(RA_slope);
		releaseAccess(RA_criticalSlope);
		releaseAccess(RA_roadGrid);
		releaseAccess(RA_slopeGrid);
		releaseAccess(WA_outStep);
		releaseAccess(RA_roadGravSens);
		releaseAccess(RA_slopeSens);
		releaseAccess(RA_criticalLow);
		releaseAccess(RA_criticalHigh);
		releaseAccess(RA_Boom);
		releaseAccess(RA_Bust);
	}

	/**
	 * This very simple model looks at every grid cell and takes the average
	 * over a the surrounding cells in a specified neighborhood.
	 * 
	 * @param stepNo
	 *            zu modellierender Schritt (beginnend bei 1!)
	 */
	public void performModelStep(int stepNo) {
		
		statusOut.println("Starting step " + stepNo + "...");
		
		//Erstellt einen Lookup-Table
		slopeLUT = new HashMap<Number,Number>();
		
		int MaxSlopeResistance = 0;
		double exp = 0.0;
		MaxSlopeResistance = 100;
		exp = (slope / (MaxSlopeResistance / 2));
		double[] lookup = new double[slopeMax+1];
		Number key = 0;
		
		for (int i = 0;i <= slopeMax;i++){
			if (i <= criticalSlope){
				double val = (criticalSlope - i) / criticalSlope;
				lookup[i] = 1 - Math.pow(val,exp);
				key = i;
			} else{
				lookup[i] = 1;
				key = i;
			}
			slopeLUT.put(key,lookup[i]);
			statusOut.println(slopeLUT.get(key));
		}	
		
		Number number = slopeLUT.get(79);
//		statusOut.println(number);

		int areaMinX = inputGrid.getMinX();								//Dimensionen des Rasters
		int areaMinY = inputGrid.getMinY();
		int areaMaxX = inputGrid.getMinX() + inputGrid.getWidth() - 1;
		int areaMaxY = inputGrid.getMinY() + inputGrid.getHeight() - 1;
		int areaWidth = inputGrid.getWidth();
		int areaHeight = inputGrid.getHeight();
		
		long localStartTime = System.currentTimeMillis();
		boolean anyBodyAlive = false;
		
		boolean[][] tmpGrid = new boolean[inputGrid.getWidth()][inputGrid.getHeight()];
		Float actUrb = null;
		
		int nbX = 0; // X-Koordinate einer Nachbarzelle
	    int nbY = 0; // Y-Koordinate einer Nachbarzelle
	    int[] nbCell = null;
	    
		int anzSpreadTreffer = 0;
		int anzDispTreffer = 0;
		int anzBreedTreffer = 0;
		int anzRoadTreffer = 0;
		int anzGesamtTreffer = 0;
		double growthRate = 0;
		int thisYear = 0;
		int roadCount = 0;
		
		
		//DISPERSION
		double disp_value = Math.round((disp*0.005)* Math.sqrt(Math.pow(inputGrid.getWidth(), 2)+(Math.pow(inputGrid.getHeight(), 2))));
				
		int dispRandX = 0;
		int dispRandY = 0;
		float slopeRandDisp = 0;
		
		//ROAD GRAVITY
		double max_RG_value = 100.0;
		double rg_value = Math.round((roadGravity/max_RG_value)*((areaWidth + areaHeight)/16)); 
		double max_search_index = Math.round(4*((int)rg_value*(1+(int)rg_value)));
//		statusOut.println("RG-Value:"+rg_value);
//		statusOut.println("Max-SearchIndex"+max_search_index);
		

		/*****************************************************************
		 * DISPERSION and BREED
		 *****************************************************************/
		
		int p = 0;	//Verteile soviel neue Pixel zufllig in geeigneten Bereichen, bis disp_value erreicht ist
		do{
			dispRandX = Math.round((float) Math.random()*areaWidth);	//Zufallsposition in X-Richtung
			dispRandY = Math.round((float) Math.random()*areaHeight);	//Zufallsposition in Y-Richtung
			if ( dispRandX < areaMinX || dispRandY < areaMinY || dispRandX > areaMaxX || dispRandY > areaMaxY ||
		            Float.isNaN(outputGrid.getRasterSampleAsFloat(dispRandX,dispRandY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
		             continue;
		        if (areaRestr.getRasterSampleAsFloat(dispRandX,dispRandY) != 0 || outputGrid.getRasterSampleAsFloat(dispRandX,dispRandY) == 1)
		        	continue;	//wenn in Area Restriction oder wenn schon Siedlung da ist, mach weiter
		        slopeRandDisp = (float) Math.random();						//Zufallszahl zum Abgleich mit Slope-Koeffizient
		        int getSlope = (int) slopeGrid.getRasterSampleAsFloat(dispRandX,dispRandY);	//holt sich den Slope-Wert aus dem Slope-Raster
		        Double doubleObj = (Double)slopeLUT.get(getSlope);		//holt den Slope-Wert aus dem Lookup-Table (get) und konvertiert in Double
		        double slopeDouble = doubleObj.doubleValue();			//konvertiert Double-Objekt in double-Wert
//		        statusOut.println(getSlope+","+slopeDouble+"----Random: "+slopeRandDisp);
		        if(slopeDouble < slopeRandDisp){
		        	tmpGrid[dispRandX][dispRandY] = true;	//true wird ins temporre Raster geschrieben
		        	anzDispTreffer++;
		        	//BREED
		        	float breedRnd = (float) Math.random()*100;
		        	int nbBreedX  = 0;
		        	int nbBreedY  = 0;
		        	int breedNb	  = 0;
		        	float NbValue = 0f;
		        	if (tmpGrid[dispRandX][dispRandY] == true && breedRnd < breed)
		        		breedNb = calculateNeighbors(dispRandX,dispRandY,NbValue);
		        			       
		        	int RNBreed1 = 0;
		        	int RNBreed2 = 0;
		        	float RNBreedSlope = 0f;
		        
		        	if (breedNb >= 2 && tmpGrid[dispRandX][dispRandY] == true){
		        		int anzBreedTemp = 0;
		        		int[] nbPossibleBreedUrban = null;
		        		RNBreed1 = 1 + Math.round((float)Math.random()*(breedNb-1));
		        		RNBreed2 = 1 + Math.round((float)Math.random()*(breedNb-1));
		        		if (RNBreed2 == RNBreed1)
		        			RNBreed2 = 1 + Math.round((float)Math.random()*(breedNb-1));	
		        		for (int cellBreed = 0; cellBreed < NEIGHBOR.length; cellBreed++){
		        			nbBreedX = dispRandX+NEIGHBOR[cellBreed][0];
		        			nbBreedY = dispRandY+NEIGHBOR[cellBreed][1];
		        			if ( nbBreedX < areaMinX || nbBreedY < areaMinY || nbBreedX > areaMaxX || nbBreedY > areaMaxY ||
		        					Float.isNaN(outputGrid.getRasterSampleAsFloat(nbBreedX,nbBreedY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
		        				continue;
		        			if (areaRestr.getRasterSampleAsFloat(nbBreedX,nbBreedY) != 0)
		        				continue;
		        			RNBreedSlope = (float)Math.random();
		        			int getSlopeNb = (int) slopeGrid.getRasterSampleAsFloat(nbBreedX,nbBreedY);
		        			Double doubleObjNb = (Double)slopeLUT.get(getSlopeNb);
		        			double slopeDoubleNb = doubleObjNb.doubleValue();
		        			float breedSample = outputGrid.getRasterSampleAsFloat(nbBreedX,nbBreedY);
		        			if (breedSample == 0f){
		        				anzBreedTemp++;
		        				nbPossibleBreedUrban = new int[] {nbBreedX,nbBreedY,anzBreedTemp};
		        				if (nbPossibleBreedUrban[2] == RNBreed1 || nbPossibleBreedUrban[2] == RNBreed2 && slopeDoubleNb < RNBreedSlope){
		        					tmpGrid[nbBreedX][nbBreedY] = true;
		        					anzBreedTreffer++;
		        				}
		        			}
		        		}
		        	} 
				} p++;
			} while (p < disp_value);	//while, statt for-Schleife, da berprfung erst am Ende vorgenommen werden kann
									//am Anfang der Schleife ist noch nicht klar, ob alle Pixel verteilt werden knnen
									//da ein Teil in Ausschlussflchen etc. landen wrde
	
		/*********************************************************************
		 * EDGE
		 *********************************************************************/
		
		//iterate over all cells
		for (int x = 0; x < inputGrid.getWidth(); x++)
			for (int y = 0; y < inputGrid.getHeight(); y++) {		//fr jede Zelle in x/y-Richtung
				
				int areaMinXadd = areaMinX + x;								//Dimensionen des Rasters
				int areaMinYadd = areaMinY + y;
				
				int anzNachbarn = 0;
				int noNachbarn = 0;
			
				
				for (int nbCellIdx = 0; nbCellIdx < NEIGHBOR.length; nbCellIdx++) {
					checkBreakingCommands();
					nbX = x+NEIGHBOR[nbCellIdx][0];		//Nachbarschaft der Zellen
			        nbY = y+NEIGHBOR[nbCellIdx][1];
			        nbCell = new int[] {nbX,nbY};		//jede Nachbarzelle
			        if ( nbX < areaMinX || nbY < areaMinY || nbX > areaMaxX || nbY > areaMaxY ||
			               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbX,nbY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
			             continue;
			        if (areaRestr.getRasterSampleAsFloat(nbX,nbY) != 0)
			        	continue;
					float rasterSample = outputGrid.getRasterSampleAsFloat(nbX,nbY);
					if (rasterSample > 0f)		//ist Nachbarzelle urban (1), wird sie gezhlt
						anzNachbarn++;
					if (rasterSample == 0f)
						noNachbarn++;
				}
				
				boolean lebt = outputGrid.getRasterSampleAsFloat(areaMinXadd , areaMinYadd ) > 0f;	//eine Zelle lebt, wenn sie 1 ist
				
				//hier ist die Abfrage wieviele Nachbarzellen am Leben sind
				//daraus ergibt sich, ob im tmpGrid die entsprechende Zelle
				//true oder false ist
				
				float SpR = (float) Math.random()*100;						//eine Zufallszahl zwischen 0 und 1
				int RN = 0; 	//hier wird eine Integer-Zufallszahl reingeschrieben
				
				if (lebt && (anzNachbarn>=2) && (SpR < spread) && (noNachbarn >=1)) {	//wenn eine Zelle lebt, mindestens 2 lebende Nachbarzellen hat, der Spread-Koeffizient grer als die fr die Zelle berechnete Zufallszahl und mindestens einen nicht lebenden Nachbarn hat
					int anzTemp = 0;
					int[] nbPossibleUrban = null;
					RN = 1 + Math.round((float)Math.random()*(noNachbarn-1));
					float RNEdgeSlope = 0;
					int nbUrbanX = 0;
					int nbUrbanY = 0;
					for (int nbCellChange = 0; nbCellChange < NEIGHBOR.length; nbCellChange++){	//dann starte eine Schleife
						checkBreakingCommands();
						nbX = x+NEIGHBOR[nbCellChange][0];
						nbY = y+NEIGHBOR[nbCellChange][1];
						if ( nbX < areaMinX || nbY < areaMinY || nbX > areaMaxX || nbY > areaMaxY ||
					               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbX,nbY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
					             continue;
						if (areaRestr.getRasterSampleAsFloat(nbX,nbY) != 0)
				        	continue;
						RNEdgeSlope = (float)Math.random();
	        			int getSlopeNb = (int) slopeGrid.getRasterSampleAsFloat(nbX,nbY);
	        			Double doubleObjNb = (Double)slopeLUT.get(getSlopeNb);
	        			double slopeDoubleNb = doubleObjNb.doubleValue();
						float rasterSample = outputGrid.getRasterSampleAsFloat(nbX,nbY);	//holt die Werte aus den jeweiligen Nachbarzellen
						if (rasterSample == 0f){	//wenn der Wert 0 ist
							anzTemp++;				//wird Anzahl der mglichen Zellen erhht
							nbPossibleUrban = new int[] {nbX,nbY,anzTemp};	//Koordinaten und Nummern dieser Zellen werden in Array geschrieben 
							int urbanNbTemp = 0;
							float NbValueEdge = 1f;
							if (nbPossibleUrban[2] == RN){	//wenn Nummer im "PossibleUrban"-Array der Zufallszahl entspricht, die zwischen 1 und Anzahl der mglichen Zellen liegt...
								//damit die Stdte nicht zu sehr "ausfransen" steht eine Zelle nur zur Umwandlung zur Verfgung, wenn auch sie mind. 2 urbane Nachbarn hat
								urbanNbTemp = calculateNeighbors(nbX,nbY,NbValueEdge);
								if (urbanNbTemp >= 2 && tmpGrid[nbX][nbY]==false && slopeDoubleNb < RNEdgeSlope){
									tmpGrid[nbX][nbY] = true;	//wird true ins temporre Raster geschrieben
									anzSpreadTreffer++;
								}
							} 
						}
						nbCell = new int[] {nbX,nbY,(int) rasterSample};	
					}
				}
				if (anzNachbarn>0) anyBodyAlive = true;		
			}
		
		/*********************************************************************
		 * ROAD
		 *********************************************************************/
		
		int anzGesamt = 0;
		int[] cellValue = null;
		
		//For-Schleife zhlt alle als true markierten Zellen im Temp-Raster
		for (int x = 0; x < inputGrid.getWidth();x++)
			for(int y = 0; y < inputGrid.getHeight();y++){
				int X = inputGrid.getMinX() + x;
				int Y = inputGrid.getMinY() + y;
				if (tmpGrid[X][Y]==true){
					anzGesamt++;
				}		
			}
		
		int RSearchRoad = 0;
		int r = 1;	//Verteile soviel neue Pixel zufllig in geeigneten Bereichen, bis disp_value erreicht ist
		
		//Rectangle gridBounds = new Rectangle(0, 0, inputGrid.getWidth(), inputGrid.getHeight()); //ein Rechteck wird definiert
		
		do{
			RSearchRoad = Math.round((float)Math.random()*anzGesamt);	//Zufallszahl zwischen 1 und Anzahl aller bisher neu hinzugekommener Pixel
			int anzTruePixels = 0;
			int roadNeighborhoodRange = 0;
			int RSearchNbRoad = 0;
			for (int x = 0; x < inputGrid.getWidth();x++){		//gesamtes Raster wird durchsucht
				for(int y = 0; y < inputGrid.getHeight();y++){
					int X = inputGrid.getMinX() + x;
					int Y = inputGrid.getMinY() + y;
					if (tmpGrid[X][Y]==true){
						anzTruePixels++;		//wird true-Pixel gefunden, wird anzTruePixels hochgezhlt
						cellValue = new int[] {X,Y,anzTruePixels};	//und Koordinaten in Array geschrieben
						int tempRoadCell[] = null;
						int tempNbRoadCell[] = null;
						if (cellValue[2] == RSearchRoad){
							//statusOut.println(cellValue[0]+","+cellValue[1]+","+cellValue[2]);
							//statusOut.println(r+","+RSearchRoad);
							r++;
							//JETZT NACH STRAE SUCHEN!!
							float roadValue = roadGrid.getRasterSampleAsFloat(X,Y);
//							statusOut.println(roadValue);
							if (roadValue > 0.0){
								tempRoadCell = new int[] {X,Y};
//								statusOut.println("Strasse in Entfernung 0: "+tempRoadCell[0]+","+tempRoadCell[1]);
								//roadWalk(tempRoadCell[0],tempRoadCell[1]);
								int nbCellNew[] = roadWalk(tempRoadCell[0],tempRoadCell[1]);
								if(nbCellNew != null){
									tmpGrid[nbCellNew[0]][nbCellNew[1]] = true;
//									statusOut.println(nbCellNew.length);
//									statusOut.println("Neue Zelle: "+nbCellNew[0]+","+nbCellNew[1]);
//									statusOut.println("++++++++++++++++++++++++++++++++++++++++");
									float nbRoadBreed = 0f;
									int nbRoadBreedNeighbors = 0;
									nbRoadBreedNeighbors = calculateNeighbors(nbCellNew[0],nbCellNew[1],nbRoadBreed);
									int RNRoadBreed1 = 0;
							        int RNRoadBreed2 = 0;
							        int nbRoadBreedX = 0;
							        int nbRoadBreedY = 0;
							        int anzRoadBreedTreffer1 = 0;
							        
									if (nbRoadBreedNeighbors >= 2){
										int anzBreedTemp = 0;
										int[] nbPossibleBreedUrban = null;
										RNRoadBreed1 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));
										RNRoadBreed2 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));
										float RNRoadBreedSlope = 0f;
										
										if (RNRoadBreed2 == RNRoadBreed1)
											RNRoadBreed2 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));	
										for (int cellBreed = 0; cellBreed < NEIGHBOR.length; cellBreed++){
							        		nbRoadBreedX = nbCellNew[0]+NEIGHBOR[cellBreed][0];
											nbRoadBreedY = nbCellNew[1]+NEIGHBOR[cellBreed][1];
											if ( nbRoadBreedX < areaMinX || nbRoadBreedY < areaMinY || nbRoadBreedX > areaMaxX || nbRoadBreedY > areaMaxY ||
										               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
										             continue;
											if (areaRestr.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY) != 0)
									        	continue;
											RNRoadBreedSlope = (float)Math.random();
						        			int getSlopeNb = (int) slopeGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY);
						        			Double doubleObjNb = (Double)slopeLUT.get(getSlopeNb);
						        			double slopeDoubleNb = doubleObjNb.doubleValue();
											float breedSample = outputGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY);
											if (breedSample == 0f){
												anzBreedTemp++;
												nbPossibleBreedUrban = new int[] {nbRoadBreedX,nbRoadBreedY,anzBreedTemp};
												if (nbPossibleBreedUrban[2] == RNRoadBreed1 || nbPossibleBreedUrban[2] == RNRoadBreed2 && slopeDoubleNb < RNRoadBreedSlope){
													tmpGrid[nbRoadBreedX][nbRoadBreedY] = true;
													anzRoadBreedTreffer1++;
													anzRoadTreffer++;
													statusOut.println("Road-Breed-Treffer: "+anzRoadBreedTreffer1);
												}
											}
										}
							        	
							        
									}
									
								} else{
//									statusOut.println("Keine neue Zelle gefunden");
								}
							} else{
								int anzRoadNb = 0;
								
								//HIER MIT FOR SCHLEIFE RECHTECK WACHSEN LASSEN!!!
								for (int roadSearch = 0; roadSearch <= rg_value; roadSearch++){
									if (anzRoadNb > 0)
										continue;
									for (int nbRoadCellsX = X - roadNeighborhoodRange; nbRoadCellsX <= X + roadNeighborhoodRange; nbRoadCellsX++){
										for (int nbRoadCellsY = Y - roadNeighborhoodRange; nbRoadCellsY <= Y + roadNeighborhoodRange; nbRoadCellsY++){
										//	if (gridBounds.contains(nbRoadCellsX, nbRoadCellsY))
												if ( nbRoadCellsX < areaMinX || nbRoadCellsY < areaMinY || nbRoadCellsX > areaMaxX || nbRoadCellsY > areaMaxY ||
														Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
													continue;
												if (areaRestr.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY) != 0)
													continue;
												float roadValueNb = roadGrid.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY);
												if (roadValueNb > 0f){
													anzRoadNb++;
												}
												//statusOut.println("Strasse in Entfernung 1: "+anzRoadNb+","+nbRoadCellsX+","+nbRoadCellsY);
										}
									}
									roadNeighborhoodRange++;
									RSearchNbRoad = Math.round((float)Math.random()*anzRoadNb);
									//statusOut.println("--> Zyklus "+roadNeighborhoodRange+"Nachbarn: "+anzRoadNb);
									//statusOut.println(RSearchNbRoad);
									int anzTempRoadNb = 0;
									for (int nbRoadCellsX = X - roadNeighborhoodRange; nbRoadCellsX <= X + roadNeighborhoodRange; nbRoadCellsX++){
										for (int nbRoadCellsY = Y - roadNeighborhoodRange; nbRoadCellsY <= Y + roadNeighborhoodRange; nbRoadCellsY++){
//						        			statusOut.println("TEST-ROAD: "+nbRoadCellsX+","+nbRoadCellsY);
										//	if (gridBounds.contains(nbRoadCellsX, nbRoadCellsY))
											if ( nbRoadCellsX < areaMinX || nbRoadCellsY < areaMinY || nbRoadCellsX > areaMaxX || nbRoadCellsY > areaMaxY ||
													Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
												continue;
											if (areaRestr.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY) != 0)
												continue;
											float roadValueNbTemp = roadGrid.getRasterSampleAsFloat(nbRoadCellsX,nbRoadCellsY);
											if (roadValueNbTemp > 0f){
												anzTempRoadNb++;
												tempNbRoadCell = new int[] {nbRoadCellsX,nbRoadCellsY,anzTempRoadNb};
												if (tempNbRoadCell[2]==RSearchNbRoad){
													//statusOut.println("Treffer: "+tempNbRoadCell[0]+","+tempNbRoadCell[1]+","+tempNbRoadCell[2]);
													tempRoadCell = tempNbRoadCell;
//													statusOut.println("Strasse in Entfernung "+roadSearch+": "+tempRoadCell[0]+","+tempRoadCell[1]+","+tempRoadCell[2]);
													//roadWalk(tempRoadCell[0],tempRoadCell[1]);
													int nbCellNew[] = roadWalk(tempRoadCell[0],tempRoadCell[1]);
													if(nbCellNew != null){
														tmpGrid[nbCellNew[0]][nbCellNew[1]] = true;
//														statusOut.println(nbCellNew.length);
//														statusOut.println("Neue Zelle: "+nbCellNew[0]+","+nbCellNew[1]);
//														statusOut.println("++++++++++++++++++++++++++++++++++++++++");
														
														float nbRoadBreed = 0f;
														int nbRoadBreedNeighbors = 0;
														nbRoadBreedNeighbors = calculateNeighbors(nbCellNew[0],nbCellNew[1],nbRoadBreed);
														int RNRoadBreed1 = 0;
												        int RNRoadBreed2 = 0;
												        int nbRoadBreedX = 0;
												        int nbRoadBreedY = 0;
												        int anzRoadBreedTreffer2 = 0;
												        
														if (nbRoadBreedNeighbors >= 2){
															int anzBreedTemp = 0;
															int[] nbPossibleBreedUrban = null;
															RNRoadBreed1 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));
															RNRoadBreed2 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));
															float RNRoadBreedSlope = 0f;
															if (RNRoadBreed2 == RNRoadBreed1)
																RNRoadBreed2 = 1 + Math.round((float)Math.random()*(nbRoadBreedNeighbors-1));	
															for (int cellBreed = 0; cellBreed < NEIGHBOR.length; cellBreed++){
												        		nbRoadBreedX = nbCellNew[0]+NEIGHBOR[cellBreed][0];
																nbRoadBreedY = nbCellNew[1]+NEIGHBOR[cellBreed][1];
																if ( nbRoadBreedX < areaMinX || nbRoadBreedY < areaMinY || nbRoadBreedX > areaMaxX || nbRoadBreedY > areaMaxY ||
															               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
															             continue;
																if (areaRestr.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY) != 0)
														        	continue;
																RNRoadBreedSlope = (float)Math.random();
											        			int getSlopeNb = (int) slopeGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY);
											        			Double doubleObjNb = (Double)slopeLUT.get(getSlopeNb);
											        			double slopeDoubleNb = doubleObjNb.doubleValue();
																float breedSample = outputGrid.getRasterSampleAsFloat(nbRoadBreedX,nbRoadBreedY);
																if (breedSample == 0f){
																	anzBreedTemp++;
																	nbPossibleBreedUrban = new int[] {nbRoadBreedX,nbRoadBreedY,anzBreedTemp};
																	if (nbPossibleBreedUrban[2] == RNRoadBreed1 || nbPossibleBreedUrban[2] == RNRoadBreed2 && slopeDoubleNb < RNRoadBreedSlope){
																		tmpGrid[nbRoadBreedX][nbRoadBreedY] = true;
																		anzRoadBreedTreffer2++;
																		anzRoadTreffer++;
																		statusOut.println("Road-Breed-Treffer: "+anzRoadBreedTreffer2);
																	}
																}
															}
												        	
												        
														}
													} else{
//														statusOut.println("Keine neue Zelle gefunden");
													}
												}
											}
										}
									}
								}
							}
						}
					}	
				}
			}
			
		} while (r <= breed);
		
		statusOut.println("Spread-Treffer: "+anzSpreadTreffer);
		statusOut.println("Dispersion-Treffer: "+anzDispTreffer);
		statusOut.println("Breed-Treffer: "+anzBreedTreffer);
		statusOut.println("Treffer gesamt: "+anzGesamt);
		statusOut.println("Anzahl Road-Treffer: "+anzRoadTreffer);
		anzGesamtTreffer = anzSpreadTreffer+anzDispTreffer+anzBreedTreffer+anzRoadTreffer;
		
		outStep.addGrid();
		
		
		// TempGrid auf das echte Grid schreiben
		for (int y = 1; y < inputGrid.getHeight()-1;y++ ) {
			for (int x = 1; x < inputGrid.getWidth()-1; x++ ) {
				
				int X = inputGrid.getMinX() + x;
				int Y = inputGrid.getMinY() + y;
				
								
				boolean lebt = outputGrid.getRasterSampleAsFloat(X , Y ) > 0f;	//eine Zelle lebt, wenn sie 1 ist
				boolean road = roadGrid.getRasterSampleAsFloat(X, Y) > 0f;	//eine Zelle beinhaltet eine Strae...
				if (tmpGrid[x][y] == false)	//wenn im temporren Raster nichts drinsteht...
					tmpGrid[x][y] = lebt;	//wird der Wert aus dem Ausgangsraster bernommen
				if(lebt == true)
					thisYear++;
				if(road == true)
					roadCount++;

				outputGrid.setRasterSample( tmpGrid[x][y] ? 1f : 0f ,X ,Y);	
				actUrb  = (Float)outputGrid.getRasterSample(x,y);
				outStep.getGrid(stepNo-1).setRasterSample(actUrb,x,y);
			}
		}
		
		//SELF-MODIFICATION
		growthRate = ((float)anzGesamtTreffer / (float)thisYear)*100;
		statusOut.println("Wachstumsrate: "+growthRate+", Urban: "+anzGesamtTreffer);
		double percent_urban = (100.0 *(thisYear+roadCount) / (totalPixels-roadCount-excldCount));
		
		//Boom-Year
		if(growthRate > criticalHigh){
			slope -= (float)percent_urban * (float)slopeSens;
			if(slope <= 0.01)
				slope = 1.0;
			roadGravity += (float)percent_urban * roadGravSens;
			if(roadGravity > 100.0)
				roadGravity = 100.0;
			if(disp < 100.0){
				disp *= boom;
				if(disp > 100.0)
					disp = 100.0;
				breed *= boom;
				if(breed > 100.0)
					breed = 100.0;
				spread *= boom;
				if(spread > 100.0)
					spread = 100.0;
			}
		}
		//Bust-Year
		if(growthRate < criticalLow){
			slope += (float)percent_urban * (float)slopeSens;
			if(slope > MaxSlopeResistance)
				slope = MaxSlopeResistance;
			roadGravity -= (float)percent_urban * roadGravSens;
			if(roadGravity <= 0.01)
				roadGravity = 1.0;
			if(growthRate < criticalLow && disp > 0){
				disp *= bust;
				if(disp <= 0.01)
					disp = 1.0;
				spread *= bust;
				if(spread <= 0.01)
					spread = 1.0;
				breed *= bust;
				if(breed <= 0.01)
					breed = 1.0;
			}
		}
		statusOut.println("thisYear: "+thisYear+", roadCount: "+roadCount+", excldCount: "+excldCount+", percent_urban: "+percent_urban);
		statusOut.println("Slope-Sensitivity: "+slope+", RoadGravity-Sensitivity: "+roadGravity+", Diffusion-Sensitivity: "+disp+", Breed-Sensitivity: "+breed+", Spread-Sensitivity: "+spread);
		
		
		
		if (!anyBodyAlive) {
			stepCount = stepNo;
			statusOut.println("Lebt keiner mehr... hre hier auf!");
		}
		
		
		System.out.println("Finished step " + (stepNo) + " in "
				+ ((System.currentTimeMillis() - localStartTime)) + " ms\n");
		localStartTime = System.currentTimeMillis();
	}
	
	private int[] roadWalk(int X,int Y){
//		statusOut.println(X+","+Y);
		boolean end_of_road;
		end_of_road = false;
		int run = 0;
		int run_value = 0;
		int[] nbCellNew = null;
		int[] nbCellTemp = null;
		while(!end_of_road){
			end_of_road = true;
			int nbRoadX = 0;
			int nbRoadY = 0;
			int areaMinX = inputGrid.getMinX();								//Dimensionen des Rasters
			int areaMinY = inputGrid.getMinY();
			int areaMaxX = inputGrid.getMinX() + inputGrid.getWidth() - 1;
			int areaMaxY = inputGrid.getMinY() + inputGrid.getHeight() - 1;
			int roadNb = 0;
			int roadNbTemp = 0;
			int RN = 0;
						
			int[] nbCell = null;
			for (int nbRoadCells = 0; nbRoadCells < NEIGHBOR.length; nbRoadCells++){
				nbRoadX = X+NEIGHBOR[nbRoadCells][0];
				nbRoadY = Y+NEIGHBOR[nbRoadCells][1];
				if ( nbRoadX < areaMinX || nbRoadY < areaMinY || nbRoadX > areaMaxX || nbRoadY > areaMaxY ||
			               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadX,nbRoadY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
			             continue;
				//if (areaRestr.getRasterSampleAsFloat(nbRoadX,nbRoadY) != 0)
		        //	continue;
				float roadSample = roadGrid.getRasterSampleAsFloat(nbRoadX,nbRoadY);
				if (roadSample > 0f)
					roadNb++;
			}
	//		statusOut.println("Nachbarn von temporaerer Strassenzelle: "+roadNb);
			
			if (roadNb > 0){
				RN = 1 + Math.round((float)Math.random()*(roadNb-1));
			
				for (int nbRoadCells = 0; nbRoadCells < NEIGHBOR.length; nbRoadCells++){
					nbRoadX = X+NEIGHBOR[nbRoadCells][0];
					nbRoadY = Y+NEIGHBOR[nbRoadCells][1];
					if ( nbRoadX < areaMinX || nbRoadY < areaMinY || nbRoadX > areaMaxX || nbRoadY > areaMaxY ||
							Float.isNaN(outputGrid.getRasterSampleAsFloat(nbRoadX,nbRoadY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
			             	continue;
					float roadSample = roadGrid.getRasterSampleAsFloat(nbRoadX,nbRoadY);
					if (roadSample > 0f){
						roadNbTemp++;
						nbCell = new int[] {nbRoadX,nbRoadY,roadNbTemp};
						if (nbCell[2] == RN){
							statusOut.println("Nachbarzelle gefunden: "+nbCell[0]+","+nbCell[1]+","+nbCell[2]);
							end_of_road = false;
							run++;
							X = nbRoadX;
							Y = nbRoadY;
							float roadSample2 = roadGrid.getRasterSampleAsFloat(X,Y);
							run_value = (int) (roadSample2 / 100 * disp);
					//		statusOut.println("Run: "+run);
					//		statusOut.println("Run_Value: "+run_value);
					//		statusOut.println(X+","+Y+","+roadSample2);
							if (run > run_value){
								end_of_road = true;
					//			statusOut.println("Endzelle erreicht!! Run:"+run+"RunValue: "+run_value);
								float NbValue = 0f; 
								int Nb = calculateNeighbors(X,Y,NbValue);
					//			statusOut.println("moegliche Nachbarzellen nach RoadWalk: "+Nb);
								RN = 1 + Math.round((float)Math.random()*(Nb-1));
				//				statusOut.println("Random: "+RN);
								if (Nb > 0){
									int nbUrbanizeX = 0;
									int nbUrbanizeY = 0;
									int NbTemp = 0;
									float RNRoadSlope = 0f;
						//			statusOut.println(X+","+Y);
									for (int urbanizeCells = 0; urbanizeCells < NEIGHBOR.length; urbanizeCells++){
										nbUrbanizeX = X+NEIGHBOR[urbanizeCells][0];
										nbUrbanizeY = Y+NEIGHBOR[urbanizeCells][1];
										if ( nbUrbanizeX < areaMinX || nbUrbanizeY < areaMinY || nbUrbanizeX > areaMaxX || nbUrbanizeY > areaMaxY ||
												Float.isNaN(outputGrid.getRasterSampleAsFloat(nbUrbanizeX,nbUrbanizeY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
											continue;
										if (areaRestr.getRasterSampleAsFloat(nbUrbanizeX,nbUrbanizeY) != 0)
											continue;
										float rasterSample = outputGrid.getRasterSampleAsFloat(nbUrbanizeX,nbUrbanizeY);
										RNRoadSlope = (float)Math.random();
					        			int getSlopeNb = (int) slopeGrid.getRasterSampleAsFloat(nbUrbanizeX,nbUrbanizeY);
					        			Double doubleObjNb = (Double)slopeLUT.get(getSlopeNb);
					        			double slopeDoubleNb = doubleObjNb.doubleValue();
										if (rasterSample == 0f){
											NbTemp++;
											nbCellTemp = new int[] {nbUrbanizeX,nbUrbanizeY,NbTemp};
											if (nbCellTemp[2] == RN && slopeDoubleNb < RNRoadSlope){
												nbCellNew = new int[] {nbCellTemp[0],nbCellTemp[1],nbCellTemp[2]};
											}
										}	
									}
								
							} else{
								nbCellNew = null;
							}	break;
						}
						
					}
					
				}
			}
			
			}
		}
		return nbCellNew;
		}
	
	private int calculateNeighbors(int X, int Y, float NbValue){
		int nbX = 0;
		int nbY = 0;
		int areaMinX = inputGrid.getMinX();								//Dimensionen des Rasters
		int areaMinY = inputGrid.getMinY();
		int areaMaxX = inputGrid.getMinX() + inputGrid.getWidth() - 1;
		int areaMaxY = inputGrid.getMinY() + inputGrid.getHeight() - 1;
		int Nb = 0;
		for (int nbCells = 0; nbCells < NEIGHBOR.length; nbCells++){
			nbX = X+NEIGHBOR[nbCells][0];
			nbY = Y+NEIGHBOR[nbCells][1];
			if ( nbX < areaMinX || nbY < areaMinY || nbX > areaMaxX || nbY > areaMaxY ||
		               Float.isNaN(outputGrid.getRasterSampleAsFloat(nbX,nbY)) ) //wenn Nachbarzelle auerhalb des Rasters (NaN) --> ignorieren
		             continue;
			if (areaRestr.getRasterSampleAsFloat(nbX,nbY) != 0)
	        	continue;
			float rasterSample = outputGrid.getRasterSampleAsFloat(nbX,nbY);
			if (rasterSample == NbValue)
				Nb++;
		}
	//	statusOut.println("..."+Nb+"...");
		return Nb;
	}
	
	protected static boolean checkNoData(int x, int y, WritableGrid grid) {
	    return Float.isNaN(grid.getRasterSampleAsFloat(x,y));
	  }
}
