/** XULU - This file is part of the eXtendable Unified Land Use Modelling Platform (XULU)

    This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
    This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
    You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA

    Diese Bibliothek ist freie Software; Sie dürfen sie unter den Bedingungen der GNU Lesser General Public License, wie von der Free Software Foundation veröffentlicht, weiterverteilen und/oder modifizieren; entweder gemäß Version 2.1 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
    Diese Bibliothek wird in der Hoffnung weiterverbreitet, daß sie nützlich sein wird, jedoch OHNE IRGENDEINE GARANTIE, auch ohne die implizierte Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Mehr Details finden Sie in der GNU Lesser General Public License.
    Sie sollten eine Kopie der GNU Lesser General Public License zusammen mit dieser Bibliothek erhalten haben; falls nicht, schreiben Sie an die Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 **/

package skrueger.gol;

import java.awt.image.DataBuffer;

import schmitzm.data.WritableGrid;
import schmitzm.data.property.ScalarProperty;
import edu.bonn.xulu.model.AbstractStepModel;

public class GameOfLife extends AbstractStepModel {
	private WritableGrid inputGrid;
	private WritableGrid outputGrid;

	public GameOfLife() {
		super(new GameOfLifeContentManager() );
		
		stepCount = 1000;
	}

	@Override
	public void performModelStep(int stepNo) {
		
		int[][] diffs = new int[][] { 
				{ -1,-1 }, {0,-1}, {1,-1} , 
				{ -1,0 },  {1,0},
				{ -1,1 },  {0,1}, {1,1}	
				};
		
		boolean anyBodyAlive = false;
		
		boolean[][] tmpGrid = new boolean[inputGrid.getWidth()][inputGrid.getHeight()];

		for (int i = 1; i < inputGrid.getWidth()-1; i++ ) {
			for (int ii = 1; ii < inputGrid.getHeight()-1; ii++ ) {
				checkBreakingCommands();

				int X = inputGrid.getMinX() + i;
				int Y = inputGrid.getMinY() + ii;
				
				int anzNachbarn = 0;
				
				for (int d = 0; d < diffs.length; d++) {
					float rasterSample = outputGrid.getRasterSampleAsFloat(X +diffs[d][0], Y +diffs[d][1]);
					if (rasterSample > 0f) 
						anzNachbarn++;
				}
				
				boolean lebt = outputGrid.getRasterSampleAsFloat(X , Y ) > 0f;
				
				// Neugruendung
				if (!lebt && (anzNachbarn==3)) 
					tmpGrid[i][ii] = true;
				else 
					if (lebt && ((anzNachbarn < 2) || (anzNachbarn > 3 )) )
						tmpGrid[i][ii] = false;
					else 
						tmpGrid[i][ii] = lebt;
					
					if (anzNachbarn>0) anyBodyAlive = true;
			}
		}

		// TempGrid auf das echte Grid schreiben
		for (int i = 1; i < inputGrid.getWidth()-1;i++ ) {
			for (int ii = 1; ii < inputGrid.getHeight()-1; ii++ ) {

				int X = inputGrid.getMinX() + i;
				int Y = inputGrid.getMinY() + ii;

				outputGrid.setRasterSample( tmpGrid[i][ii] ? 1f : 0f ,X ,Y);
		
				
			}
		}
		
		if (!anyBodyAlive) {
			stepCount = stepNo;
			statusOut.println("Lebt keiner mehr... höre hier auf!");
		}
	}

	@Override
	public void performModelDispose() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void performModelInit() {
		ScalarProperty sp = (ScalarProperty) this.contManager.getResource(0).getData();
		inputGrid = (WritableGrid) sp.getOneTimeReadAccess().getValue();
		
		sp = (ScalarProperty) this.contManager.getResource(1).getData();
		outputGrid = (WritableGrid) sp.getOneTimeWriteAccess().getValue();
		
		int startAnz = 0;
		for (int i = 0; i < inputGrid.getWidth();i++ ) {
			for (int ii = 0; ii < inputGrid.getHeight(); ii++ ) {
				if (inputGrid.getSampleType() == DataBuffer.TYPE_FLOAT) {
					float val = (float) Math.random();
					if (val > 0.7 ) {
						val = 1f; 
						startAnz++;
					}
						else val = 0f;
					
					inputGrid.setRasterSample(val, inputGrid.getMinX() + i, inputGrid.getMinY() + ii);
					outputGrid.setRasterSample(val, inputGrid.getMinX() + i, inputGrid.getMinY() + ii);
				} else {
					statusOut.println("schlecht!");
				}
			}
		}
		statusOut.println("Anzahl der startenden Individuen: "+startAnz);
		
		
	}
	
}
